/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*************************************************************************/

#include <shlobj.h>
#include <stdio.h>
#include "common.h"

// Fill a WAVEFORMATEX structure
int FillWaveFormatExStructure(WAVEFORMATEX *pWaveFormatEx, int nSampleRate, int nBitsPerSample, int nChannels)
{
	pWaveFormatEx->cbSize = 0;
	pWaveFormatEx->nSamplesPerSec = nSampleRate;
	pWaveFormatEx->wBitsPerSample = nBitsPerSample;
	pWaveFormatEx->nChannels = nChannels;
	pWaveFormatEx->wFormatTag = 1;
	pWaveFormatEx->nBlockAlign = (pWaveFormatEx->wBitsPerSample / 8) * pWaveFormatEx->nChannels;
	pWaveFormatEx->nAvgBytesPerSec = pWaveFormatEx->nBlockAlign * pWaveFormatEx->nSamplesPerSec;
	return 0;
}

int TimeStr2Int(char* timestr)
{
	char *p = strchr(timestr, ':');
	if (p) {
		return atoi(timestr) * 60 + atoi(p + 1);
	} else {
		return atoi(timestr);
	}
}

int FileExist(char* filename)
{
	WIN32_FIND_DATA f;
	HANDLE hFind = FindFirstFile(filename,&f);
	if (hFind != INVALID_HANDLE_VALUE) {
		FindClose(hFind);
		return 1;
	} else {
		return 0;
	}
}

long long GetFileBytes(char* filename)
{
	HANDLE hFile = CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	long long size;
	if (hFile==INVALID_HANDLE_VALUE) return -1;
	size = GetFileSize(hFile,((LPDWORD)&size) + 1);
	CloseHandle(hFile);
	return size;
}

int GetFileKB(char* filename)
{
	HANDLE hFile = CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	DWORD sizelo, sizehi;
	if (hFile==INVALID_HANDLE_VALUE) return -1;
	sizelo = GetFileSize(hFile,&sizehi);
	CloseHandle(hFile);
	return (sizehi << 22) | (sizelo >> 10);
}

char* strredup(char* buf, char* str, int len)
{
	if (len == 0) len = strlen(str);
	if (!buf) {
		buf = (char*)malloc(len + 1);
	} else {
		int buflen = strlen(buf);
		if (buflen < len) {
			if (buf) free(buf);
			buf = (char*)malloc(len + 1);
		}
	}
	memcpy(buf, str, len);
	buf[len] = 0;
	return buf;
}

int BrowseForPath(char *buffer)
{
	LPITEMIDLIST idlist;
	BROWSEINFO udtBI;
	char *p;

	ZeroMemory(&udtBI,sizeof(BROWSEINFO));
	udtBI.hwndOwner=0;
	udtBI.lpszTitle="Choose a folder";
	udtBI.ulFlags= BIF_RETURNONLYFSDIRS;
	idlist=SHBrowseForFolder(&udtBI);
	if (!idlist || !SHGetPathFromIDList(idlist,buffer))
		return 0;
	CoTaskMemFree(idlist);
	//update GUI
	p=buffer;
	while(*(++p));
	if (*(p-1)!='\\') {
		*(WORD*)p='\\';
	}
	return (int)(p+1-buffer);
}

int LoadTextFile(char* filename, char** buffer, int bufsize)
{
	DWORD bytes;
	HANDLE hFile=CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	int len;

	if (hFile==INVALID_HANDLE_VALUE) return -1;
	len=GetFileSize(hFile,0);
	if (len+1>bufsize) {
		bufsize=len+1;
		*buffer=(char*)realloc(*buffer, bufsize);
	}
	*(*buffer+len)=0;
    ReadFile(hFile, *buffer, len, &bytes, 0);
	CloseHandle(hFile);
	return bytes;
}

int SetHandleInheritance(LPHANDLE ph, BOOL fInherit)
{
	HANDLE hOld = *ph;
	if (!DuplicateHandle(GetCurrentProcess(), hOld, GetCurrentProcess(), ph, 0, fInherit, DUPLICATE_SAME_ACCESS)) {
		return -1;
	}
	CloseHandle(hOld);
	return 0;
}

BOOL IsDriveValid(char drv)
{
	DWORD d;
	int drvnum;
	if (drv >= 'A' && drv <='Z')
		drvnum = drv - 'A';
	else if (drv >= 'a' && drv <='z')
		drvnum = drv - 'a';
	else
		return FALSE;
	d = GetLogicalDrives();
	return ((d & (1 << drvnum)) != 0);
}

double GetFPS(int fpsint, int fpsScale)
{
	return (double)fpsint / (fpsScale ? fpsScale : 1);
}

double GetFPSScale(double fps, int* fpsint, int* fpsScale)
{
	// find out fps scale
	static int scale[] = {2, 4, 8, 16};
	double f;
	int i;
	f = fps - (double)(int)fps;
	if (!f) {
		*fpsScale = 1;
		*fpsint = (int)fps;
		return fps;
	} else if (f > 0.9) {
		*fpsScale = 1001;
		*fpsint = (int)(fps * 1001 + 1);
		return (double)*fpsint / 1001;
	}
	for (i = 0; i < sizeof(scale) / sizeof(int); i++) {
		f = fps * (*fpsScale = scale[i]);
		if (f == (double)(int)f) {
			*fpsint = (int)f;
			break;
		}
	}
	return f;
}

int RemoveFile(char* filename, int timeout)
{
	DeleteFile(filename);
	if (FileExist(filename)) {
		Sleep(timeout);
		DeleteFile(filename);
		if (FileExist(filename)) return -1;
	}
	return 0;
}

int IsNetworkPath(char* path)
{
	static char* prefix[] = {"http://", "ftp://", "rtsp://", "mms://"};
	int i;
	for (i = 0; i < sizeof(prefix) / sizeof(char*); i++)
		if (!strncmp(path, prefix[i], strlen(prefix[i]))) return 1;
	return 0;
}
