#include <Winsock2.h>
#include <stdlib.h>
#include <stdio.h>
#include "httpget.h"

/****************************************************************************
* HTTP protocol client
****************************************************************************/
#define HEADER_END_FLAG  "\r\n\r\n"
#define DEBUG printf
#define CONN_RETRIES 3
#define HTTP_GET_HEADER "%s %s HTTP/1.0\r\nAccept: */*\r\nConnection: %s\r\nUser-Agent: Mozilla/5.0\r\nHost: %s\r\n%s\r\n"
#define HTTP_POST_HEADER "POST %s HTTP/1.0\r\nContent-Type: application/x-www-form-urlencoded\r\nUser-Agent: AG130\r\nContent-Length: %d\r\n\r\n"

void initHttpReq(httpRequest* req, char* url, char* proxy)
{
	memset(req, 0, sizeof(httpRequest));
	req->url = url;
	req->proxy = proxy;
}

static char* parseURL(char* url, char *Hostname, httpRequest* param)
{
	char *ptr;
	int iLen;
	char *Path;

	if (strncmp(url,"http://",7) != 0 ) { 
		return NULL;
	}
	
	url += 7;
	ptr = strchr(url, '/');
	if (ptr == NULL) {
		Path="/";
		iLen = strlen(url);
	} else {
		iLen = (int)(ptr - url);
		Path = ptr;
	}
	ptr = strrchr(url, ':');
	if (ptr && *(ptr + 1) != '/') {
		unsigned short int port = atoi(ptr + 1);
		if (port) {
			param->port = port;
		}
		iLen = (int)(ptr - url);
	}
	memcpy(Hostname, url, iLen);
	Hostname[iLen] ='\0';
	if (!param->port) param->port = 80;
	return Path;
}
	
#define MAX_HEADER_SIZE 4095

int httpGet(httpRequest* param)
{
	char *Path;
	int rspHeaderBytes, receivedBytes;
	char Hostname[128];
	struct hostent *target_host;
	int ret = 0;

	param->state=HS_REQUESTING;
	param->iPayloadSize=0;

	if (param->proxy) {
		parseURL(param->proxy,Hostname,param);
		Path = param->url;
	} else {
		Path = parseURL(param->url,Hostname,param);
		if (!Path) return -1;
	}

	if (param->header) free(param->header);
	param->header = malloc(MAX_HEADER_SIZE + 1);
	if ((param->iBytesStart|param->iBytesEnd) == 0) {
		switch (param->method) {
		case HM_GET:
			sprintf(param->header, HTTP_GET_HEADER, "GET",
				Path, param->flags & FLAG_CLOSE_CONN ? "close" : "Keep-Alive", Hostname,"");
			break;
		case HM_HEAD:
			sprintf(param->header, HTTP_GET_HEADER, "HEAD",
				Path, param->flags & FLAG_CLOSE_CONN ? "close" : "Keep-Alive", Hostname,"");
			break;
		case HM_POST:
			sprintf(param->header,HTTP_POST_HEADER, Path, param->iPostDataSize);
			break;
		}
	} else {
		char tokenRange[48],*p=tokenRange;
		p+=sprintf(p, "Range: bytes=%u-", param->iBytesStart);
		if (param->iBytesEnd>0) {
			sprintf(p,"%u\r\n",param->iBytesEnd);
		} else {
			strcpy(p,"\r\n");
		}
		sprintf(param->header, HTTP_GET_HEADER, Path, Hostname, tokenRange);
	}
	
	do {

	if ((target_host = gethostbyname((const char*)Hostname)) == NULL) {
		ret = -1;
		continue;
	}

	{
		int offset=0;
		int hdrsize = (int)strlen(param->header);
		char *p=NULL;

		if (!param->sockfd) {
			struct sockaddr_in server_addr;
			memset(&server_addr.sin_zero,0,8);
			server_addr.sin_family = AF_INET;
			server_addr.sin_addr.s_addr = ((struct in_addr *)(target_host->h_addr))->s_addr;
			server_addr.sin_port = htons(param->port);
			DEBUG("Connecting to server...\n");
			if ((param->sockfd = socket(AF_INET,SOCK_STREAM,0)) == -1) {
				DEBUG("Failed to open socket\n");
				ret = -1;
				continue;
			}
			if (connect(param->sockfd,(struct sockaddr *)&server_addr,sizeof(struct sockaddr)) < 0) {
				DEBUG("Failed to connect\n");
				ret = -1;
				continue;
			}
		}
		DEBUG("Sending request...\n");
		do {
			ret = send(param->sockfd, param->header + offset, hdrsize - offset,0);
			if ( ret < 0 ) {
				continue;
			}
			offset += ret;
			DEBUG("Sent %d/%d bytes\n", ret, offset);
		} while(offset < hdrsize);

		if (param->method == 1) {
			offset = 0;
			do {
				ret = send(param->sockfd, param->postData + offset, param->iPostDataSize - offset,0);
				if ( ret < 0 ) {
					continue;
				}
				offset += ret;
				DEBUG("Sent %d/%d bytes of post data\n", ret, offset);
			} while(offset < param->iPostDataSize);
		}

		//receive header
		DEBUG("Receiving response...\n");
		receivedBytes=0;
		for(;;) {
			ret = recv(param->sockfd, param->header + receivedBytes, MAX_HEADER_SIZE-receivedBytes, 0);
			if (ret <= 0) break;
			receivedBytes += ret;
			param->header[receivedBytes] = 0;
			DEBUG("Received %d/%d bytes\n", ret, receivedBytes);
			if ((p = strstr(param->header,"\r\n\r\n"))) {
				*(p+2) = '\0';
				break;
			}
			if (receivedBytes == MAX_HEADER_SIZE) {
				p = param->header + receivedBytes - 4;
			}
		}
		if (!p) {
			DEBUG("Invalid server response\n");
			ret = -1;
			continue;
		} else {
			ret = 0;
			p+=4;
		}
		rspHeaderBytes = (int)(p - param->header);
	}

	//process header
	{
		char *p;
		DEBUG("Response header:\n%s\n", param->header);
		if (p = strstr(param->header,"HTTP/1.")) {
			param->iHttpCode = atoi(p+9);
		}
		if (param->iHttpCode == 404) {
			DEBUG("Invalid response or file not found on server\n");
			closesocket(param->sockfd);
			param->sockfd=0;
			param->state=HS_IDLE;
			free(param->header);
			param->header = 0;
			return -1;
		}
		while ((p = strstr(p, "\r\n"))) {
			char *q;
			q=strchr((p += 2),':');
			if (!q) continue;
			*q = 0;
			if (!stricmp(p,"Content-length")) {
				param->iPayloadSize=atoi(q+2);
			} else if (!stricmp(p,"Content-type")) {
				param->contentType = q+2;
			} else if (!stricmp(p, "Transfer-Encoding")) {
				if (!strncmp(p + 19, "chunked", 7)) {
					param->flags |= FLAG_CHUNKED;
				}
			}
			*q = ':';
		}
	}
	DEBUG("Payload bytes: %d\n",param->iPayloadSize);
	if (param->iPayloadSize == 0)
		param->iPayloadSize = param->iDataSize ? param->iDataSize - 1 : receivedBytes - rspHeaderBytes;
	if (param->flags & FLAG_REQUEST_ONLY) {
		closesocket(param->sockfd);
		param->state=HS_IDLE;
		return 0;
	}
	// receive payload
	if (param->method != HM_HEAD) {
		int iPayLoadBytes;
		int bytes;
		int recvBytes = 0;
		param->state=HS_RECEIVING;
		if (param->buffer) {
			iPayLoadBytes=param->iPayloadSize>=param->iDataSize?param->iDataSize-1:param->iPayloadSize;
		} else {
			DEBUG("Allocating %d bytes for payload buffer\n",param->iPayloadSize);
			param->buffer = calloc(1, param->iPayloadSize+1);
			iPayLoadBytes=param->iPayloadSize;
		}
		recvBytes = receivedBytes - rspHeaderBytes;
		if (recvBytes > 0) {
			DEBUG("Header includes %d bytes of payload\n", recvBytes);
			memcpy(param->buffer, param->header + rspHeaderBytes, recvBytes);
		}
		//receive payload data
		for (; recvBytes < iPayLoadBytes; recvBytes += bytes) {
			int bytesToRecv = iPayLoadBytes - recvBytes;
			DEBUG("Received: %d To receive: %d\n", recvBytes, bytesToRecv);
			bytes = recv(param->sockfd, param->buffer + recvBytes, bytesToRecv, 0);
			if (bytes<=0) {
				DEBUG("End of stream\n");
				break;
			}
		}
		DEBUG("Payload received: %d bytes\n", recvBytes);
		*(param->buffer + recvBytes)=0;
		param->iDataSize = recvBytes;
	}
	
	} while(0);
	if (param->sockfd) closesocket(param->sockfd);
	param->sockfd=0;
	param->state=HS_IDLE;
	if (!(param->flags & FLAG_KEEP_HEADER)) {
		free(param->header);
		param->header = 0;
	}
	return ret;
}
