/*******************************************************************
* Lite Window Handling Framework
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#ifndef WINDOWSHANDLE_H
#define WINDOWSHANDLE_H
#include <commdlg.h>
#include <commctrl.h>
#include <richedit.h>
#include <tchar.h>

#ifdef _USRDLL
#define EXPORTCLASS __declspec(dllexport)
#else
#define EXPORTCLASS
#endif

#define ORIGINAL -2147483646

LRESULT CALLBACK WndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);
int CALLBACK DlgProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);

typedef struct {
	int id;
	void* data;
} mtable;

class CMatchTable {
public:
	mtable* add(int id);
	int remove(int id);
	mtable* match(int id);
	mtable* finematch(int id);
	void reset();
	virtual void release();
	CMatchTable() {reset();}
	~CMatchTable() {reset();}
	mtable* table;
	int count;
	int idx;
};

class CWindow
{
public:
	void Move(int x,int y,int w = ORIGINAL,int h = ORIGINAL);
	void StretchFull();
	void StretchItemFull(HWND hwndItem);
	void Hide() {ShowWindow(hwnd,SW_HIDE);}
	void Show()	{ShowWindow(hwnd,SW_SHOW);}
	void SetForeground() {SetForegroundWindow(hwnd);}
	void Minimize() {ShowWindow(hwnd,SW_MINIMIZE);}
	void Maximize() {ShowWindow(hwnd,SW_MAXIMIZE);}
	void Restore() {ShowWindow(hwnd,SW_RESTORE);}
	void Focus() {SetFocus(hwnd);}
	void Copy() {SendMessage(hwnd, WM_COPY, 0, 0);}
	void Paste() {SendMessage(hwnd, WM_PASTE, 0, 0);}
	void Cut () {SendMessage(hwnd, WM_CUT, 0, 0);}
	void Undo() {SendMessage(hwnd, EM_UNDO, 0, 0);}
	void Redo() {SendMessage(hwnd, EM_REDO, 0, 0);}
	void Clear() {SendMessage(hwnd, WM_SETTEXT,0,0);}
	void SetCaption(LPCTSTR lpText) {SetWindowText(hwnd,lpText);}
	void Close() {
		SendMessage(hwnd, WM_CLOSE, 0, 0);
	}
	void Destroy() {
		if (!hwnd) return;
		HWND hwndToDestroy = hwnd;
		hwnd = 0;
		DestroyWindow(hwndToDestroy);
	}
	int Ask(TCHAR* text, int type = MB_YESNO, TCHAR* caption = "Confirm") {
		return MessageBox(hwnd, text, caption, type|MB_ICONQUESTION);
	}
	int Ask(int id, int type = MB_YESNO, TCHAR* caption = "Confirm");
	void MsgBox(TCHAR* text, TCHAR* caption = 0) {
		MessageBox(hwnd,text,caption ? caption : "Information",MB_OK|MB_ICONINFORMATION);
	}
	void MsgBox(int id, TCHAR* caption = 0);
	long GetWindowFlag(int nIndex) {
		return GetWindowLong(hwnd,nIndex);
	}
	void SetWindowFlag(int nIndex, long dwFlag) {
		long curflag=GetWindowFlag(nIndex);
		SetWindowLong(hwnd,nIndex,curflag|dwFlag);
	}
	void UnsetWindowFlag(int nIndex, long dwFlag) {
		long curflag=GetWindowFlag(nIndex);
		SetWindowLong(hwnd,nIndex,curflag&(~dwFlag));
	}
	void GetWindowPos(int* left, int* top, int* width = 0, int* height = 0);
	HWND GetParentWindow() {
		return (HWND)GetWindowLong(hwnd,GWL_HWNDPARENT);
	}
	virtual bool onShow(bool fShow){return false;}
	virtual bool onCreate(){return false;}
	virtual bool onSize(int width, int height){return false;}
	virtual bool onMove(){return false;}
	virtual bool onPosChange(){return false;}
	virtual bool onScroll(int dlgid, int code){return false;}
	virtual bool onCommand(int dlgid, int code){return false;}
	virtual bool onClick(int dlgid) { return false; }
	virtual bool onKeyPress(int vkey) { return false; }
	virtual bool onSelChange(int dlgid) { return false; }
	virtual bool onNotify(LPNMHDR pnmh){return false;}
	virtual bool onPaint(){return false;}
	virtual bool onClose(){ return false; }
	virtual bool onDestroy() {return true;}
	virtual bool onMouseMove(DWORD pos, int vkey){return false;}
	virtual bool onMouseLeave(){return false;}
	virtual bool onMouseClick(DWORD flags){return false;}
	virtual bool onTimer(int id){return false;}
	virtual bool onParentNotify(UINT msg, LPARAM lParam) {return false;}
	virtual bool onFocus() { return false; }
	virtual bool onChildDestroy() { return false; }
	virtual HWND Create(int style, int xstyle, HWND hwndParent, WNDCLASS& wc);
	virtual HWND Create(int style, int xstyle, CWindow* parent, WNDCLASS& wc);
	virtual HWND Create(int style, int xstyle, HWND hwndParent);
	virtual HWND Create(int style, int xstyle, CWindow* parent);
	virtual HWND CreateDlg(HINSTANCE hInst, int dlgid, HWND hwndParent);
	virtual HWND CreateDlg(HINSTANCE hInst, int dlgid, CWindow* parent);
	virtual int ShowDlg(HINSTANCE hInst, int dlgid, CWindow* parent);
	LRESULT SendMsg(UINT Msg,WPARAM wParam,LPARAM lParam) {
		return SendMessage(hwnd,Msg,wParam,lParam);
	}
	LRESULT SendMsg(UINT Msg) {
		return SendMessage(hwnd,Msg,0,0);
	}
	LRESULT SendDlgMsg(int dlgid,UINT Msg,WPARAM wParam,LPARAM lParam) {
		return SendDlgItemMessage(hwnd,dlgid,Msg,wParam,lParam);
	}
	LRESULT SendDlgMsg(int dlgid,UINT Msg) {
		return SendDlgItemMessage(hwnd,dlgid,Msg,0,0);
	}
	LRESULT PostDlgMsg(int dlgid,UINT Msg,WPARAM wParam,LPARAM lParam) {
		return PostMessage(GetHandle(dlgid),Msg,wParam,lParam);
	}
	HWND GetHandle(int dlgid) {
		return GetDlgItem(hwnd,dlgid);
	}
	bool IsItemChecked(int dlgid) {
		return (SendDlgItemMessage(hwnd,dlgid,BM_GETCHECK,0,0)==BST_CHECKED);
	}
	bool IsMenuChecked(UINT uId) {
		return ((GetMenuState(GetMenu(hwnd),uId,MF_BYCOMMAND) & MF_CHECKED)!=0);
	}
	void SetItemChecked(int dlgid, bool fChecked = true) {
		SendDlgItemMessage(hwnd,dlgid,BM_SETCHECK,fChecked ? BST_CHECKED : BST_UNCHECKED,0);
	}
	void SetMenuChecked(UINT uId, bool fChecked = true) {
		CheckMenuItem(GetMenu(hwnd),uId,MF_BYCOMMAND|(fChecked ? MF_CHECKED : MF_UNCHECKED));
	}
	void MoveCenter(HWND hWndParent = 0);
	void MoveCenter(int w, int h) {
		MoveWindow(hwnd,(GetSystemMetrics(SM_CXSCREEN)-w)/2,
			(GetSystemMetrics(SM_CYSCREEN)-h)/2,
			w,h,TRUE);
	}
	TCHAR* GetItemText(int dlgid);
	// Combo box related methods
	int ComboGetSel(int dlgid) {
		return SendDlgItemMessage(hwnd, dlgid, CB_GETCURSEL, 0, 0);
	}
	int ComboSetSel(int dlgid, int index) {
		return SendDlgItemMessage(hwnd, dlgid, CB_SETCURSEL, index, 0);
	}
	int ComboDelete(int dlgid, int index = -1);
	TCHAR* ComboGetString(int dlgid, int index = -1);
	void ComboAddString(int dlgid, TCHAR* str) {
		SendDlgItemMessage(hwnd, dlgid, CB_ADDSTRING, 0, (LPARAM)str);
	}
	void ComboAddStrings(int dlgid, TCHAR* itemstr, int sel = 0);
	void ComboAddStrings(int dlgid, int id, int sel = 0);
	void ComboReset(int dlgid) { SendDlgItemMessage(hwnd, dlgid, CB_RESETCONTENT, 0, 0); }
	int ComboGetOptionIndex(int dlgid, TCHAR* options[]);

	void Enable(int dlgid = 0, bool fEnabled = true) { EnableWindow(dlgid ? GetHandle(dlgid) : hwnd, fEnabled); }
	void Disable(int dlgid = 0) { EnableWindow(dlgid ? GetHandle(dlgid) : hwnd, FALSE); }
	int SetText(int dlgid, TCHAR* text) { return SendDlgItemMessage(hwnd, dlgid, WM_SETTEXT, 0, (LPARAM)text); }
	int GetText(int dlgid, TCHAR* buf, int bufsize) { return SendDlgItemMessage(hwnd, dlgid, WM_GETTEXT, bufsize, (LPARAM)buf); }
	char* GetText(int dlgid);
	void ShowCtrl(int dlgid, BOOL fShown = TRUE) { ShowWindow(GetDlgItem(hwnd, dlgid), fShown ? SW_SHOW : SW_HIDE); }
	void HideCtrl(int dlgid) { ShowWindow(GetDlgItem(hwnd, dlgid), SW_HIDE); }
	void EnableCtrl(int dlgid, BOOL fEnabled = TRUE) { EnableWindow(GetDlgItem(hwnd, dlgid), fEnabled); }
	void DisableCtrl(int dlgid) { EnableWindow(GetDlgItem(hwnd, dlgid), FALSE); }
	void GetBorderSize();
	int ShowFileDialog(int type, OPENFILENAME* pofn);
	void SetFont(LPCTSTR lfFaceName,int lfCharSet,int lfHeight,int lfWeight);
	bool DefMsgHandler(UINT msg,WPARAM wParam,LPARAM lParam);
	int GetShowState();
	bool SetAlpha(int value);
	int GetRadioBoxValue(int radiobox[]);
	int GetTextInt(int dlgid);
	void SetTextInt(int dlgid, int val);
	double GetTextFloat(int dlgid);
	void SetTextFloat(int dlgid, double val);
	CWindow():hwnd(NULL){ hInstance = GetModuleHandle(0); }
	~CWindow(){ Destroy(); }
	HWND hwnd;
	WORD borderWidth;
	WORD borderHeight;
	CWindow* wndParent;
	HMODULE hInstance;
protected:
	void RegWndClass();
};

typedef struct {
	TCHAR* text;
	int width;
} COL_INFO;

class CListView : public CWindow
{
public:
	int AddColumns(COL_INFO cols[]);
	int InsertItem(int item);
	int AddItem(TCHAR* items[]);
	int SetItem(int item, TCHAR* items[]);
	int GetText(int item, int subitem, TCHAR* buffer, int bufsize);
	int SetText(int item, int subitem, TCHAR* buffer);
	int RemoveItem(int item);
	int GetCount();
	int GetSelection(int item);
	int SetSelection(int item, int fSel);
	int GetCheck(int item)
	{
		return ListView_GetCheckState(hwnd, item);
	}
	void SetCheck(int item, int fCheck)
	{
		 ListView_SetCheckState(hwnd, item, fCheck);
	}
	int GetFocus();
	int SetListFocus(int item);
	void Reset();
	void SetExtendedStype(DWORD style)
	{
		ListView_SetExtendedListViewStyle(hwnd, style);
	}
	int Scroll(int x, int y)
	{
		return SendMessage(hwnd, LVM_SCROLL, x, y);
	}
};
#endif
