#include <windows.h>
#include <atlstr.h>
#include "xmlaccessor.h"

void CXML::Init()
{
	xmlInitParser();
}

void CXML::Cleanup()
{
	xmlCleanupParser();
}

void CXML::New(char* xmlVersion, char* rootNodeName, char* rootNodeValue)
{
	Close();
	//xmlInitParser();
	doc = xmlNewDoc((xmlChar*)xmlVersion);
	cur = xmlNewDocNode(doc, 0, (xmlChar*)rootNodeName, (xmlChar*)rootNodeValue);
	xmlDocSetRootElement(doc, cur);
}

int CXML::Open(char* filename)
{
	if (!xmlfile || strcmp(filename, xmlfile)) {
		Close();
		doc = xmlParseFile(filename);
		if (doc == NULL ) {
			return -1;
		}
		xmlfile = strdup(filename); 
	}
	cur = xmlDocGetRootElement(doc);
	return 0;
}

int CXML::Save(char* filename, char* encoding)
{
	int ret;
	if (encoding)
		ret = xmlSaveFileEnc(filename, doc, "UTF-8");
	else
		ret = xmlSaveFile(filename, doc);
	if (xmlfile) {
		free(xmlfile);
		xmlfile = 0;
	}
	if (!ret) {
		xmlfile = strdup(filename);
	}
	return ret;
}

int CXML::Read(char* xmlbuf, char* url, char* encoding)
{
	Close();
	doc = xmlReadMemory(xmlbuf, (int)strlen(xmlbuf), url ? url : "data.xml", encoding, 0);
	if (doc <= 0) return -1;
	cur = xmlDocGetRootElement(doc);
	return cur ? 0 : -1;
}

int CXML::Dump(char** buffer, char* encoding)
{
	int bufsize;
	xmlChar* xmlbuf;
	if (!encoding)
		xmlDocDumpFormatMemory(doc, &xmlbuf, &bufsize, 1);
	else
		xmlDocDumpFormatMemoryEnc(doc, &xmlbuf, &bufsize, encoding, 1);
	*buffer = (char*)malloc(bufsize);
	memcpy(*buffer, xmlbuf, bufsize);
	xmlFree(xmlbuf);
	return bufsize;
}

void CXML::Close()
{
	if (value) {
		xmlFree(value);
		value = 0;
	}
	if (doc) {
		xmlFreeDoc(doc);
		//xmlCleanupParser();
		doc = 0;
	}
	if (xmlfile) {
		free(xmlfile);
		xmlfile = 0;
	}
}

xmlNodePtr CXML::goToKey(char* path, int index)
{
	xmlNodePtr node = 0;
	xmlXPathContextPtr context = xmlXPathNewContext(doc);
	if (context == NULL) {
		printf("Error in xmlXPathNewContext\n");
		return 0;
	}
	result = xmlXPathEvalExpression((xmlChar*)path, context);
	xmlXPathFreeContext(context);
	if (result == NULL) {
		printf("Error in xmlXPathEvalExpression\n");
		return 0;
	}
	if (result->nodesetval && result->nodesetval->nodeNr > index) {
		node = result->nodesetval->nodeTab[index];
		cur = node;
	}
	xmlXPathFreeObject(result);
	return node;
}


char* CXML::getChildName(int index = 0)
{	
	if (value) {
		xmlFree(value);
		value = 0;
	}
	if (index == 0) {
		value = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
	} else if (result) {
		xmlNodeSetPtr nodeset = result->nodesetval;
		if (index >= nodeset->nodeNr)
			return NULL;
		value = xmlNodeListGetString(doc, nodeset->nodeTab[index]->xmlChildrenNode, 1);
	}
	return (char*)value;
}

int CXML::getChildCount(xmlNodePtr node)
{
	int count = 0;
	for (node = (node ? node : cur)->children; node; node = node->next) {
		if (node->type == XML_ELEMENT_NODE) count++;
	}
	return count;
}

xmlNodePtr CXML::goRoot()
{
	return (cur = xmlDocGetRootElement(doc));
}

xmlNodePtr CXML::goNext(xmlNodePtr node)
{
	node = (node ? node : cur)->next;
	while (node && node->type != XML_ELEMENT_NODE) node = node->next;
	if (node) cur = node;
	return node;
}

xmlNodePtr CXML::goPrev(xmlNodePtr node)
{
	node = (node ? node : cur)->prev;
	while (node && node->type != XML_ELEMENT_NODE) node = node->prev;
	if (node) cur = node;
	return node;
}

xmlNodePtr CXML::goChild(xmlNodePtr node)
{
	node = (node ? node : cur)->children;
	while (node && node->type != XML_ELEMENT_NODE) node = node->next;
	if (node) cur = node;
	return node;
}

xmlNodePtr CXML::goParent(xmlNodePtr node)
{
	node = (node ? node : cur)->parent;
	while (node && node->type != XML_ELEMENT_NODE) node = node->parent;
	if (node) cur = node;
	return node;
}

xmlNodePtr CXML::findNode(char* name, char* attrname, char* attrvalue, xmlNodePtr node)
{
	if (!node) node = cur;
	while (node) {
		if ((node->type == XML_ELEMENT_NODE && !xmlStrcmp(node->name, (xmlChar*)name)) &&
			((!attrname || !attrvalue) || !strcmp(attrvalue, getAttribute(attrname, node)))) {
			cur = node;
			return node;
		}
		node = node->next;
	}
	return 0;
}

bool CXML::isMatched(char* match, xmlNodePtr node)
{
	return !xmlStrcmp((node ? node : cur)->name, (const xmlChar *)match);
}

char* CXML::getNodeValue(int index, xmlNodePtr node)
{
	node = (node ? node : cur)->children;
	int i = 0;
	while (node) {
		if (node->type == XML_TEXT_NODE && i++ == index) break;
		node = node->next;
	}
	if (!node) return 0;
	char* text = (char*)node->content;
	if (!text || *text == '\n' || *text == '\r') return 0;
	return text;
}

char* CXML::getAttribute(char* propname, xmlNodePtr node)
{
	for (xmlAttrPtr attr = (node ? node : cur)->properties; attr; attr = attr->next) {
		if (!xmlStrcmp(attr->name, (xmlChar*)propname)) {
			return attr->children ? (char*)attr->children->content : "";
		}
	}
	return 0;
}

int CXML::getAttributeInt(char* propname, int def, xmlNodePtr node)
{
	for (xmlAttrPtr attr = (node ? node : cur)->properties; attr; attr = attr->next) {
		if (!xmlStrcmp(attr->name, (xmlChar*)propname)) {
			return attr->children && attr->children->content ? atoi((char*)attr->children->content) : def;
		}
	}
	return 0;
}
char* CXML::getNodeName(xmlNodePtr node)
{
	return (char*)(node ? node : cur)->name;
}

void CXML::setNodeName(char* nodename, xmlNodePtr node)
{
	//xmlNodeSetContent(cur, (const xmlChar*)value);
	if (cur) xmlNodeSetName((node ? node : cur), (xmlChar*)nodename);
}

void CXML::setNodeValue(char* value, xmlNodePtr node)
{
	xmlNodeSetContent((node ? node : cur), (const xmlChar*)value);
}

xmlNodePtr CXML::addChild(char* nodename, char* value, xmlNodePtr node)
{
	cur = xmlNewTextChild((node ? node : cur), NULL, (const xmlChar*)nodename, (const xmlChar*)value);
	return cur;
}

xmlAttrPtr CXML::addAttribute(char* name, char* value, xmlNodePtr node)
{
	return xmlNewProp(node ? node : cur, (xmlChar*)name, (xmlChar*)value);
}

xmlAttrPtr CXML::setAttribute(char* name, char* value, xmlNodePtr node)
{
	return xmlSetProp(node ? node : cur, (xmlChar*)name, (xmlChar*)value);
}
