/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#include "config.h"
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <mmreg.h>
#include "resource.h"

#include "processpil.h"
#include "windowhandle.h"
#include "datatypes.h"
#include "transcode.h"
#include "audioencoders.h"
#include "paramlist.h"
#include "preference.h"
#include "common.h"

#include "all.h"

void InvalidiateRiffHeader(RIFF_HEADER* hdr, ATTR_AUDIO* audio);

int CAudioEncoder::Open(CString& cmd, CString& destfile, T_ITEM* item)
{
	memset(&sp,0,sizeof(SHELL_PARAM));
	OVERLAPPED ov;
	if (flags & FLAG_PIPE) {
		CString pipename;
		memset(&ov,0,sizeof(ov));
		pipename.Format("\\\\.\\pipe\\%s%d.wav", OUTPIPENAME, GetCurrentProcessId());
		ov.hEvent = CreateEvent(0, TRUE, TRUE, 0);
		sp.fdWrite = CreateNamedPipe(pipename.GetBuffer(),PIPE_ACCESS_DUPLEX|FILE_FLAG_OVERLAPPED,PIPE_WAIT|PIPE_READMODE_MESSAGE|PIPE_TYPE_MESSAGE,1,4096,4096,1000,0);
		if (sp.fdWrite == INVALID_HANDLE_VALUE) {
			sp.fdWrite = 0;
			logger.Log(IDS_ERROR_CREATE_PIPE);
			return -1;
		}
		ConnectNamedPipe(sp.fdWrite, &ov);
	}
	if (!(flags & FLAG_PIPE) && !(flags & FLAG_STANDALONE)) {
		sp.flags = SHELL_REDIRECT_STDIN;
	}
	if ((flags & FLAG_READ_CONSOLE) && !pref.GetValueBool("overall.ui.console")) {
		sp.flags |= SHELL_REDIRECT_STDOUT;
		sp.buffer=(char*)malloc(4096);
		sp.iBufferSize=4096;
	}
	if (pref.GetValueBool("overall.ui.console")) sp.flags|=SHELL_SHOW_WINDOW;
	sp.pchCommandLine = cmd.GetBuffer();;
	pfnEvent(MC_MSG_CMD, sp.pchCommandLine);
	if (ShellExec(&sp)) {
		if (flags & FLAG_PIPE) CloseHandle(sp.fdWrite);
		logger.Log(IDS_FAIL_START_FE);
		return -1;
	}
	if (flags & FLAG_PIPE) {
		WaitForSingleObject(ov.hEvent, 3000);
		CloseHandle(ov.hEvent);
	}
	SetProcessPriorityBySetting(sp.piProcInfo.hProcess);
	percent = 0;
	return 0;
}

int CAudioEncoder::Close() {
	if (sp.piProcInfo.dwProcessId) {
		if (flags & FLAG_READ_CONSOLE) {
			CloseHandle(sp.fdRead);
			free(sp.buffer);
		}
		if (sp.fdWrite) CloseHandle(sp.fdWrite);
		logger.Log(IDS_WAIT_PROCESS);
		ShellWait(&sp, 1000);
		ShellTerminate(&sp);
		ShellClean(&sp);
		percent = 0;
	}
	return 0;
}

int CAudioEncoder::Encode(char *data, int bytes) {
	DWORD bytesWritten=0;
	if (!WriteFile(sp.fdWrite,data,bytes,&bytesWritten,NULL))
		return -1;
	return bytesWritten;
}

int CAudioEncoder::GetProgress()
{
	char *p;
	if (!(sp.flags & SHELL_REDIRECT_STDOUT)) {
		return ShellWait(&sp, 1000)?-1:0;
	}
	if (ShellRead(&sp, 1000)<=0) {
		percent = 0;
		return -1;
	}
	if ((p=strchr(sp.buffer,'%'))) {
		for (;;) {
			char ch = *(--p);
			if (!(ch >= '0' && ch <='9') && ch != '.') break;
		}
		percent = atoi(p + 1);
	}
	return percent;
}

/*******************************************************************************
* Winamp Encoder Wrapper
********************************************************************************/
int CWinampEnc::open(CString& encplugfile, CString& destfile, CString& conf, T_ITEM* item)
{
	CString conffile;
	//Save encoder settings
	if (!conf.IsEmpty()) {
		logger.Log("Writing configuration file");
		FILE *fp;
		conffile = pref.GetValueStr("overall.task.tempdir");
		char *tempdir = pref.GetValueStr("overall.task.tempdir");
		if (*tempdir)
			conffile = tempdir;
		else
			conffile = instinfo.apppath;
		conffile += INIFILE;
		fp = fopen(conffile.GetBuffer(), "w");
		if (fp<0) return -1;
		fprintf(fp,"%s",conf.GetBuffer());
		fclose(fp);
	}
	hEncPlugin = LoadLibrary(encplugfile.GetBuffer());
	if (!hEncPlugin) {
		logger.Log(IDS_ERROR_LOADING, encplugfile.GetBuffer());
		return -1;
	}
	CreateAudio3 = (CREATEAUDIO3TYPE)GetProcAddress(hEncPlugin, "CreateAudio3");
	if(!CreateAudio3) { 
		FreeLibrary(hEncPlugin);
		logger.Log(IDS_INCORRECT_VERSION, encplugfile.GetBuffer());
        return -1;
	}
	GetAudioTypes3 = (GETAUDIOTYPES3TYPE)GetProcAddress(hEncPlugin, "GetAudioTypes3");
	*(void **)&finishAudio3=(void *)GetProcAddress(hEncPlugin,"FinishAudio3");
	*(void **)&PrepareToFinish=(void *)GetProcAddress(hEncPlugin,"PrepareToFinish");
	encoder = CreateAudio3(item->target.audio->channels, item->target.audio->samplerate, item->target.audio->samplebits, mmioFOURCC('P','C','M',' '), &outtype, conffile.GetBuffer());
	DeleteFile(conffile.GetBuffer());
	if(!encoder) {
		logger.Log("Error starting encoder. This may due to incorrect encoding settings.");
		FreeLibrary(hEncPlugin);
		return -1;
	}
	outfile = destfile;
	fp=fopen(outfile.GetBuffer(), "wb");
	if (fp<=0) {
		logger.Log("Unable to create output file");
		FreeLibrary(hEncPlugin);
		return -1;
	}
	SetThreadPriorityBySetting(GetCurrentThread());
	logger.Log(IDS_LOADED, encplugfile.GetBuffer());
	return 0;
}

int CWinampEnc::encode(char* data, int bytes)
{
	char buffer[32768];
	int len=bytes,in_used,enclen,i=0;
	if (!encoder) return -1;
	while(len>0) {
		in_used=0;
		enclen=encoder->Encode(i++,data,len,&in_used,buffer,sizeof(buffer));
		if(in_used>0) {
			data += in_used;
			len -= in_used;
		}
		if (enclen>0) { 
			fwrite(buffer,enclen,1,fp);
		} else if (enclen<0) {
			return -1;
		}
	}
	return bytes;
}

int CWinampEnc::close()
{
	if (!encoder) return -1;
	fclose(fp);
	logger.Log(IDS_UNLOADING_DLL);
	if (finishAudio3) finishAudio3(outfile.GetBuffer(), encoder);
	FreeLibrary(hEncPlugin);
	SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_NORMAL);
	encoder=0;
	return 0;
}


/*******************************************************************************
* LameMP3 encoder
********************************************************************************/
CLameMP3::CLameMP3()
{
	flags=FLAG_RAW_PCM;
	prefNodePath = "audioenc.lame";
	logger.SetModuleName(GetName());
}

int CLameMP3::UpdateSettings(T_ITEM* item)
{
	const char modes[]={'s','j','d','m'};
	PARAM_OPT opts[] = {
		{".aq", "-q "},
		{".allfreq", "-k"},
		{".copyright", "-c"},
		{".nonOriginal", "-o"},
		{".protect", "-p"},
		{".nores", "--nores"},
		{".iso", "--strictly-enforce-ISO"},
		{".replayGain", "--replaygain-fast|--replaygain-accurate|--noreplaygain"},
		{"overall.ui.verbose", "--quiet|"},
		{0},
	};
	CPrefBase prefbase(prefNodePath);
	cl.Format("\"%s\" \"$(SourceFile)\" \"$(DestFile)\" --priority 1", pref.GetValueStr(".path"));
	if (!mcinst.fUseWaveFile) cl += " -r -x -s $(SampleRate)";
	if (!pref.GetValueBool(".allfreq")) {
		int lowpass = pref.GetValueInt(".lowpass");
		int hipass = pref.GetValueInt(".hipass");
		if (lowpass) cl.AppendFormat(" --lowpass %d", lowpass);
		if (hipass) cl.AppendFormat(" --highpass %d", hipass);
	}
	switch (pref.GetValueInt(".mode")) {
	case 2:	//CBR
		cl.AppendFormat(" --cbr -b %d", pref.GetValueInt(".bitrate"));
		break;
	case 1: //ABR
		cl.AppendFormat(" --abr %d", pref.GetValueInt(".bitrate"));
		break;
	default: //VBR
		cl.AppendFormat(" --vbr-new -V %d", 9 - pref.GetValueInt(".quality"));
		if (!pref.IsDefaultValue(".minBitrate"))
			cl.AppendFormat(" -b %d", pref.GetValueInt(".minBitrate"));
		if (!pref.IsDefaultValue(".maxBitrate"))
			cl.AppendFormat(" -B %d", pref.GetValueInt(".maxBitrate"));
		break;
	}
	int i=pref.GetValueInt(".stereo");
	if (item->target.audio->channels == 1) i=4;
	if (i>0 && i<=4) {
		if (item->target.audio->channels == 1) i = 4;
		cl.AppendFormat(" -m %c",modes[i-1]);
	}
	pref.GenOptions(cl, opts);
	if (pref.GetValueBool("overall.tagging.tag") && item->info.tags.valid)
		cl += " --tt \"$(Title)\" --ta \"$(Artist)\" --tl \"$(Album)\" --ty \"$(Year)\" --tn \"$(Track)\" --tg \"$(Genre)\" --tc \"$(Comment)\" ";
	return 0;
}

char* CLameMP3::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

int CLameMP3::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return (pref.GetValueInt(".mode") == 0) ? (5 + pref.GetValueInt(".quality")) * 16 : pref.GetValueInt(".bitrate");
}

/*******************************************************************************
* FAAC encoder
********************************************************************************/

CFaac::CFaac()
{
	prefNodePath = "audioenc.faac";
	flags=FLAG_RAW_PCM;
	logger.SetModuleName(GetName());
}

int CFaac::UpdateSettings(T_ITEM* item)
{
	PARAM_OPT opts[] = {
		{".version", "--mpeg-vers 2|--mpeg-vers 4"},
		{".objectType", "--obj-type LC|--obj-type Main|--obj-type SSR|--obj-type LTP"},
		{".tns", "--tns"},
		{".noMidSide", "--no-midside"},
		{".blockType", "--shortctl "},
		{0},
	};
	CPrefBase prefbase(prefNodePath);
	pref.SetBase(prefNodePath);
	cl.Format("\"%s\" -w -P -X -%c %d -R $(SampleRate) -B $(SampleBits) -C $(ChannelNum)",
		pref.GetValueStr(".path"),
		pref.GetValueInt(".mode")?'b':'q',
		pref.GetValueInt(".mode")?pref.GetValueInt(".bitrate"):pref.GetValueInt(".quality")
	);
	pref.GenOptions(cl, opts);
	if (pref.GetValueBool("overall.tagging.tag") && item->info.tags.valid)
		cl.AppendFormat(" --title \"$(Title)\" --artist \"$(Artist)\" --album \"$(Album)\" --track \"$(Track)\" --year \"$(Year)\" --genre \"$(Genre)\" --comment \"$(Comment)\"");
	cl += " -o \"$(DestFile)\" \"$(SourceFile)\"";
	return 0;
}

char* CFaac::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

int CFaac::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return (pref.GetValueInt(".mode") == 0) ?  pref.GetValueInt(".quality") : pref.GetValueInt(".bitrate");
}

/*******************************************************************************
* OGG/Vorbis encoder
********************************************************************************/
CVorbis::CVorbis()
{
	prefNodePath= "audioenc.vorbis";
	flags=FLAG_RAW_PCM;
	logger.SetModuleName(GetName());
}

int CVorbis::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	PARAM_OPT opts[] = {
		{".resample", "--resample "},
		{".downmix", "--downmix"},
		{"overall.ui.verbose", "--quiet|"},
		{0},
	};
	int n;
	cl.Format("\"%s\" -r -o \"$(DestFile)\" -C $(ChannelNum) -R $(SampleRate)", pref.GetValueStr(".path"));
	if (pref.GetValueBool(".mode")==0) {
		n = pref.GetValueInt(".quality");
		cl.AppendFormat(" -q %d,%d", n / 100, n % 100);
	} else {
		cl.AppendFormat(" -b %d", pref.GetValueInt(".bitrate"));
	}
	if (n = pref.GetValueInt(".minBitrate"))
		cl.AppendFormat(" -m %d", n);
	if (n = pref.GetValueInt(".maxBitrate"))
		cl.AppendFormat(" -M %d", n);
	if (pref.GetValueBool("overall.tagging.tag") && item->info.tags.valid) {
		cl += " -t \"$(Title)\" -a \"$(Artist)\" -l \"$(Album)\" -N \"$(Track)\" -d \"$(Year)\" -G \"$(Genre)\" ";
	}
	cl += " \"$(SourceFile)\"";
	return 0;
}

char* CVorbis::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

int CVorbis::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return (pref.GetValueInt(".mode") == 0) ? pref.GetValueInt(".quality") / 2 : pref.GetValueInt(".bitrate");
}

////////////////////////////////////////////////////////////////////////////////////
// AMR Encoder
////////////////////////////////////////////////////////////////////////////////////
#include "amr.h"

static int getWBBitrateMode(int bitrate)
{
    static unsigned char rates[]={ 8, //6.6k
                         10,//8.85
                         13,//12.65
                         15,//14.25
                         17,//15.85
                         18,//18.25
                         22,//19.85
                         23,//23.05
                         24,//23.85
	};
    int i;
    for (i=0;i<9;i++)  {
        if(bitrate <= rates[i]) return(i);
    }
    /*Return highest possible*/
    return(8);
}

CAmr::CAmr()
{
	flags = FLAG_RAW_PCM;
	prefNodePath = "audioenc.amr";
	logger.SetModuleName(GetName());
}

int CAmr::GetLowestSampleRate()
{ 
	CPrefBase prefbase(prefNodePath);
	return pref.GetValueInt(".codec") == 0 ? 8000 : 16000;
}

int CAmr::GetHighestSampleRate()
{
	CPrefBase prefbase(prefNodePath);
	return pref.GetValueInt(".codec") == 0 ? 8000 : 16000;	
}

int CAmr::Open(CString& cmd, CString& destfile, T_ITEM* item)
{
	void* pfnAmrInit;
	CPrefBase prefbase(prefNodePath);
	dll = new CDllHelper(pref.GetValueStr(".path"));
	if (!dll->hDll) {
		delete dll;
		logger.Log(IDS_FILE_NOT_FOUND, pref.GetValueStr(".path"));
		return -1;
	}
	if (codec == 0) {
		// AMR-NB
		pfnAmrInit = dll->GetAddr("Encoder_Interface_init");
		pfnEncode = dll->GetAddr("Encoder_Interface_Encode");
	} else {
		// AMR-WB
		pfnAmrInit = dll->GetAddr("E_IF_init");
		pfnEncode = dll->GetAddr("E_IF_encode");
	}
	if (!pfnAmrInit || !pfnEncode) {
		delete dll;
		logger.Log(IDS_INCORRECT_VERSION, GetFullName());
		return -1;
	}
	fp = fopen(destfile.GetBuffer(), "wb");
	if (!fp) {
		delete dll;
		return -1;
	}
	if (codec == 0) {
		state = (*(pfn_amrnb_init)pfnAmrInit)(allow_dtx);
		framesize = AMRNB_FRAME_SIZE * sizeof(short);
		// write AMR header
		fprintf(fp, "%s", AMRNB_MAGIC_NUMBER);
	} else {
		state = (*(pfn_amrwb_init)pfnAmrInit)();
		framesize = AMRWB_FRAME_SIZE * sizeof(short);
		// write AMR header
		fprintf(fp, "%s", AMRWB_MAGIC_NUMBER);
	}
	return 0;	
}

int CAmr::Encode(char *buffer, int bytes)
{
	int serial_bytes;
	if (codec == 0) {
#ifndef ETSI
	   unsigned char serial_data[32];
#else
		short serial_data[250] = {0};
#endif
		serial_bytes = (*(pfn_amrnb_encode)pfnEncode)(state, (Mode)mode, (AMR_WORD16*)buffer, serial_data, 0);
		return serial_bytes > 0 ? fwrite(serial_data, 1, serial_bytes, fp) : -1;
	} else {
		unsigned char serial_data[NB_SERIAL_MAX];
		serial_bytes = (*(pfn_amrwb_encode)pfnEncode)(state, mode, (AMR_WORD16*)buffer, serial_data, allow_dtx);
		return serial_bytes > 0 ? fwrite(serial_data, 1, serial_bytes, fp) : -1;
	}
}

int CAmr::Close()
{
	pfn_amr_exit pfnAmrExit;
	logger.Log(IDS_UNLOADING_DLL);
	if (codec == 0) {
		pfnAmrExit = (pfn_amr_exit)dll->GetAddr("Encoder_Interface_exit");
	} else {
		pfnAmrExit = (pfn_amr_exit)dll->GetAddr("E_IF_exit");
	}
	if (pfnAmrExit) (*pfnAmrExit)(state);
	fclose(fp);
	delete dll;
	return 0;
}

int CAmr::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	allow_dtx = pref.GetValueBool(".dtx") ? 1 : 0;
	codec = pref.GetValueInt(".codec");
	mode = pref.GetValueInt(".mode");
	if (codec ==0 && mode > 8) mode = 8;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////
// Windows Media Audio Encoder
////////////////////////////////////////////////////////////////////////////////////
CWma::CWma()
{
	prefNodePath = "audioenc.wma";
	flags = FLAG_STANDALONE|FLAG_READ_CONSOLE;
	logger.SetModuleName(GetName());
}

int CWma::UpdateSettings(T_ITEM* item)
{
	static const char* codecs[] = {"WMA9STD","WMSP9","WMA9LSL"};
	CPrefBase prefbase(prefNodePath);
	if (pref.GetValueInt(".codec") == 3) pref.SetValue(".mode", 2);
	cl.Format("cscript.exe \"%sWMCmd.vbs\" -audioonly -input \"$(SourceFile)\" -output \"$(DestFile)\" -a_codec %s -a_mode %d -a_setting ",
		pref.GetValueStr(".path"), codecs[pref.GetValueInt(".codec")], pref.GetValueInt(".mode"));
	if (pref.GetValueInt(".mode") == 2) {
		cl.AppendFormat("%s_$(SampleRateKhz)_$(ChannelNum)", pref.GetValueEnum(".quality"));
	} else {
		cl.AppendFormat("%d_$(SampleRateKhz)_$(ChannelNum)", pref.GetValueInt(".bitrate"));
	}
	return 0;
}

int CWma::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return (pref.GetValueInt(".mode") == 2) ? 0 : pref.GetValueInt(".bitrate");
}

/*******************************************************************************
* PCM file writer
********************************************************************************/
CPcm::CPcm()
{
	hFile=NULL;
	prefNodePath = "audioenc.pcm";
	flags |= FLAG_RAW_PCM;
	logger.SetModuleName(GetName());
}

char* CPcm::GetOutExt(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return pref.GetValueInt(".format") == 0 ? "wav" : "pcm";
}

int CPcm::Open(CString& cmd, CString& destfile, T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	InitRiffHeader(&hdr);
	InvalidiateRiffHeader(&hdr, item->target.audio);
	hFile=CreateFile(destfile,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,0,NULL);
	if (hFile == INVALID_HANDLE_VALUE) {
		hFile = 0;
		return -1;
	}
	if (pref.GetValueInt(".format") == 0) {
		DWORD bytes;
		WriteFile(hFile,&hdr,sizeof(RIFF_HEADER),&bytes,NULL);
	}
	hdr.data.chunkSize = 0;
	return 0;
}

int CPcm::Close()
{
	if (hFile) {
		DWORD bytes;
		SetFilePointer(hFile,0,NULL,FILE_BEGIN);
		InvalidiateRiffHeader(&hdr, 0);
		WriteFile(hFile,&hdr,sizeof(RIFF_HEADER),&bytes,NULL);
		CloseHandle(hFile);
		hFile = 0;
	}
	return 0;
}

int CPcm::Encode(char *data, int bytes)
{
	DWORD bytesWritten;
	if (!WriteFile(hFile,data,bytes,&bytesWritten,NULL)) return -1;
	hdr.data.chunkSize += bytesWritten;
	return (int)bytesWritten;
}

/*******************************************************************************
* Winamp AAC+V2 encoder
********************************************************************************/
#define AACP_DLL_NAME "enc_aacplus.dll"
CAACPlus::CAACPlus()
{
	flags=FLAG_RAW_PCM | FLAG_TO_TAG;
	prefNodePath = "audioenc.aacplus";
	CPrefBase prefbase(prefNodePath);
	logger.SetModuleName(GetName());
	CString path = pref.GetValueStr(".path");
	if (!FileExist((path + AACP_DLL_NAME).GetBuffer())) {
		for (char drv = 'C'; drv <= 'F'; drv++) {
			path.Format("%c:\\Program Files\\WinAmp\\Plugins\\", drv);
			if (FileExist((path + AACP_DLL_NAME).GetBuffer())) {
				pref.SetDefaultValue(".path", path.GetBuffer(), true);
				break;
			}
		}
	}
}

int CAACPlus::Open(CString& cmd, CString& destfile, T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	int maxBitrate=0;
	int minBitrate=0;
	int nBitrate = pref.GetValueInt(".bitrate") * 1000;
	int nType = pref.GetValueInt(".type");
	static int bitrateRange[3][6] = 
	{
		{
		//he - mono,	 stereo,			6ch
		8000, 64000,	16000, 128000,	96000, 213335
		},
		{
		//lc - mono,	 stereo,			6ch
		8000, 160000,	16000, 320000,	160000, 320000
		},
		{
		//hi - mono,	 stereo,			6ch
		8000, 160000,	96000, 256000,	8000, 256000
		}
	};

	switch(item->target.audio->channels)
	{
		case 1:
			minBitrate = bitrateRange[nType][0];
			maxBitrate = bitrateRange[nType][1];
			break;
		case 2:
			minBitrate = bitrateRange[nType][2];
			maxBitrate = bitrateRange[nType][3];
			break;
		default:
			minBitrate = bitrateRange[nType][4];
			maxBitrate = bitrateRange[nType][5];
			break;
	}

	if (nBitrate < minBitrate || nBitrate > maxBitrate) {
		logger.Log(IDS_INVALID_BITRATE);
		return -1;
	}

	CString conf;
	if (pref.GetValueInt(".mode")==3 && pref.GetValueInt(".bitrate")>48) {
		logger.Log(IDS_ERROR_PS);
		return -1;
	}
	//Save encoder settings
	const char* codecSection[] = {"audio_aacplus","audio_aac","audio_aacplushigh"};
	conf.Format("[%s]\nsamplerate=%d\nchannelmode=%d\nbitrate=%d\nv2enable=%c\nbitstream=%i\nsignallingmode=0\n",
		codecSection[nType],
		item->target.audio->samplerate,
		item->target.audio->channels == 1 ? 1 : pref.GetValueInt(".mode") + 1,
		nBitrate,
		pref.GetValueBool(".v1") ? '0' : '1',
		pref.GetValueInt(".version") ? 5 : 0
		);

	const unsigned int codecFourCC[] = {mmioFOURCC('A','A','C','P'), mmioFOURCC('A','A','C','r'), mmioFOURCC('A','A','C','H')};
	waenc.outtype = codecFourCC[nType];
	CString dllpath = pref.GetValueStr(".path");
	char curdir[MAX_PATH];
	int ret;
	GetCurrentDirectory(sizeof(curdir), curdir);
	SetCurrentDirectory(dllpath.GetBuffer());
	dllpath += "enc_aacplus.dll";
	ret = waenc.open(dllpath, destfile, conf, item);
	SetCurrentDirectory(curdir);
	if (ret) {
		CString msg;
		msg.Format(IDS_CT_INFO);
		pfnEvent(MC_MSG_ALERT, msg.GetBuffer());
		return -1;
	}
	return 0;
}

int CAACPlus::Encode(char *data, int bytes)
{
	return waenc.encode(data, bytes);
}

int CAACPlus::Close()
{
	if (!waenc.close()) {
		return 0;
	} else {
        return -1;
	}
}

int CAACPlus::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return pref.GetValueInt(".bitrate");
}

/*******************************************************************************
* 3GPP AAC+V2 encoder
********************************************************************************/
C3gppAacPlus::C3gppAacPlus()
{
	flags = FLAG_RAW_PCM | FLAG_TO_TAG;
	prefNodePath = "audioenc.aac3gpp";
	logger.SetModuleName(GetName());
}

int C3gppAacPlus::Open(CString& cmd, CString& destfile, T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	RIFF_HEADER hdr;
	dll = new CDllHelper;
	logger.Log(IDS_LOADING_DLL);
	InitRiffHeader(&hdr);
	InvalidiateRiffHeader(&hdr, item->target.audio);
	{
	char* s = strdup(pref.GetValueStr(".path"));
	char* p = strrchr(s, '\\');
	if (p) {
		char curdir[MAX_PATH];
		*p = 0;
		GetCurrentDirectory(sizeof(curdir), curdir);
		SetCurrentDirectory(s);
		dll->LoadDLL(pref.GetValueStr(".path"));
		SetCurrentDirectory(curdir);
	}
	free(s);
	}
	if (!dll->hDll) {
		delete dll;
		logger.Log(IDS_FILE_NOT_FOUND, pref.GetValueStr(".path"));
		return -1;
	}
	pfnAacPlusInit AacPlusInit;
	AacPlusInit = (pfnAacPlusInit)dll->GetAddr("AacPlusInit");
	AacPlusEncode = (pfnAacPlusEncode)dll->GetAddr("AacPlusEncode");
	AacPlusFinish = (pfnAacPlusFinish)dll->GetAddr("AacPlusFinish");
	if (!AacPlusInit || !AacPlusEncode || !AacPlusFinish) {
		delete dll;
		logger.Log(IDS_INCORRECT_VERSION);
		return -1;
	}
	framesize = 1024 * item->target.audio->samplebits * item->target.audio->channels / 8 * 2;
	logger.Log(IDS_SET_FRAME_SIZE, framesize);

	(*AacPlusInit)(destfile.GetBuffer(), pref.GetValueInt(".bitrate") * 1000, pref.GetValueInt(".ps"), &hdr.fmt);
	SetThreadPriorityBySetting(GetCurrentThread());
	return 0;
}

int C3gppAacPlus::Encode(char *buffer, int bytes)
{
	(*AacPlusEncode)(buffer, bytes);
	return bytes;
}

int C3gppAacPlus::Close()
{
	logger.Log(IDS_UNLOADING_DLL);
	(*AacPlusFinish)();
	delete dll;
	SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_NORMAL);
	return 0;
}

int C3gppAacPlus::UpdateSettings(T_ITEM* item)
{
	return 0;
}

int C3gppAacPlus::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return pref.GetValueInt(".bitrate");
}

/*******************************************************************************
* Helix producer
********************************************************************************/
CHelix::CHelix()
{
	flags=FLAG_READ_CONSOLE | FLAG_STANDALONE | FLAG_TO_TAG;
	prefNodePath = "audioenc.helix";
	logger.SetModuleName(GetName());
}

int CHelix::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	PARAM_OPT opts[] = {
		{".type", "-am music|-am voice"},
		{0},
	};
	CString audience = pref.GetValueEnum(".audience");
	cl.Format("\"%s\" -i \"$(SourceFile)\" -o \"$(DestFile)\" -dv -dt", pref.GetValueStr(".path"));
	pref.GenOptions(cl, opts);
	cl += " -ad \"";
	int i = audience.Find(" (");
	if (i == -1)
		cl += audience.TrimLeft();
	else
		cl += audience.Left(i).TrimLeft();
	cl += '\"';
	return 0;
}

char* CHelix::GetOutExt(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return (pref.GetValueInt(".audience")<=4)?"m4a":"ram";
}

int CHelix::GetProgress()
{
	char *p;
	if (!(sp.flags & SHELL_REDIRECT_STDOUT)) {
		return ShellWait(&sp, 1000)?-1:0;
	}
	if (ShellRead(&sp, 1000)<=0) {
		percent = 0;
		return -1;
	}
	if ((p=strstr(sp.buffer,"Encoding:"))) {
		pfnEvent(MC_WRITE_CONSOLE, p);
		p=strchr(p+16,'(');
		if (p) percent=atoi(p+1);
	}
	if (percent==100) {
		if (strstr(sp.buffer,"\nDone")) {
			percent=-1;
		}
	}
	if (percent<0) return -1;
	return percent;
}

char* CHelix::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

/*******************************************************************************
* Generic CLI Audio Encoder
********************************************************************************/

CGeneric::CGeneric()
{
	flags=0;
	prefNodePath = "audioenc.cli";
	logger.SetModuleName(GetName());
}

int CGeneric::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	if (!pref.GetValueBool(".stdin"))
		flags|=FLAG_STANDALONE;
	else
		flags&=~FLAG_STANDALONE;
	if (!pref.GetValueBool("overall.ui.console"))
		flags|=FLAG_READ_CONSOLE;
	else
		flags&=FLAG_READ_CONSOLE;
	cl = pref.GetValueStr(".command");
	return 0;
}

char* CGeneric::GetOutExt(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return pref.GetValueStr(".fileext");
}

/*******************************************************************************
* WavPack Encoder
********************************************************************************/
CWavPack::CWavPack()
{
	prefNodePath = "audioenc.wavpack";
	logger.SetModuleName(GetName());
}

int CWavPack::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	PARAM_OPT opts[] = {
		{".best", "-h"},
		{".extra", "-x"},
		{".options",""},
		{0},
	};
	cl.Format("\"%s\" -y -i -q",pref.GetValueStr(".path"));
	if (pref.GetValueBool("overall.tagging.tag") && item->info.tags.valid) {
		if (item->info.tags.artist)
			cl += " -w \"Artist=$(Artist)\"";
		if (item->info.tags.valid)
			cl += " -w \"Title=$(Title)\"";
		if (item->info.tags.album)
			cl += " -w \"Album=$(Album)\"";
		if (item->info.tags.track)
			cl += " -w \"Track=$(Track)\"";
		if (item->info.tags.year)
			cl += " -w \"Year=$(Year)\"";
		if (item->info.tags.genre)
			cl += " -w \"Genre=$(Genre)\"";
		if (item->info.tags.comment)
			cl += " -w \"Comment=$(Comment)\"";
	}
	if (pref.GetValueBool(".hybrid")) {
		cl.AppendFormat(" -b%d", pref.GetValueInt(".bitrate"));
		if (pref.GetValueBool(".correction")) cl += " -c";
	}
	pref.GenOptions(cl, opts, ' ');
	cl += " \"$(SourceFile)\" \"$(DestFile)\"";
	return 0;
}

char* CWavPack::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

/*******************************************************************************
* FLAC Encoder
********************************************************************************/

CFlac::CFlac()
{
	flags = FLAG_RAW_PCM | FLAG_TO_TAG;
	prefNodePath = "audioenc.flac";
	logger.SetModuleName(GetName());
}

int CFlac::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	PARAM_OPT opts[] = {
		{".exhaustive", "-e"},
		{".maxlpc", "-l "},
		{".level", "-"},
		{0},
	};
	cl.Format("\"%s\" -s -f --force-raw-format --endian=little --channels=$(ChannelNum) --bps=$(SampleBits) --sample-rate=$(SampleRate) --sign=signed",pref.GetValueStr(".path"));
	pref.GenOptions(cl, opts, ' ');
	cl += " -o \"$(DestFile)\" \"$(SourceFile)\"";
	return 0;
}

char* CFlac::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

/*******************************************************************************
* Monkey's Audio Encoder
********************************************************************************/

int CApe::Open(CString& cmd, CString& destfile, T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	const int level[]={COMPRESSION_LEVEL_FAST,COMPRESSION_LEVEL_NORMAL,COMPRESSION_LEVEL_HIGH,COMPRESSION_LEVEL_EXTRA_HIGH,COMPRESSION_LEVEL_INSANE};
	int nRetVal = 0;
	HMODULE hMACDll;
	// load the DLL
	hMACDll = LoadLibrary(pref.GetValueStr(".path"));
	if (hMACDll == NULL) {
		return -1;
	}
	// always check the interface version (so we don't crash if something changed)
	if (VersionCheckInterface(hMACDll) != 0 || GetFunctions(hMACDll) != 0) {
		FreeLibrary(hMACDll);
		return -1;
	}
	// set the input WAV format
	WAVEFORMATEX wfeAudioFormat;
	FillWaveFormatExStructure(&wfeAudioFormat, 44100, 16, 2);
	// create the encoder interface
	hAPECompress = MACDll.Create(&nRetVal);
	if (hAPECompress == NULL) {
		FreeLibrary(hMACDll);
		return -1;
	}
	// start the encoder
	nRetVal = MACDll.Start(hAPECompress, destfile, &wfeAudioFormat, MAX_AUDIO_BYTES_UNKNOWN,
	
		level[pref.GetValueInt(".level")], NULL, CREATE_WAV_HEADER_ON_DECOMPRESSION);
	if (nRetVal != 0) {
		MACDll.Destroy(hAPECompress);
		FreeLibrary(hMACDll);
		return -1;
	}
	SetThreadPriorityBySetting(GetCurrentThread());
	return 0;
}

int CApe::Encode(char *data, int bytes)
{
	int nRetVal = MACDll.AddData(hAPECompress, (unsigned char*)data, bytes);
	return (nRetVal == ERROR_SUCCESS)?bytes:-1;
}

int CApe::Close()
{
	MACDll.Finish(hAPECompress, NULL, 0, 0);
	MACDll.Destroy(hAPECompress);
	FreeLibrary(hMACDll);
	return 0;
}

CApe::CApe()
{
	flags=FLAG_RAW_PCM | FLAG_TO_TAG;
	prefNodePath= "audioenc.ape";
	logger.SetModuleName(GetName());
}

// GetFunctions - helper that gets the function addresses for the functions we need
int CApe::GetFunctions(HMODULE hMACDll)
{
	if (!hMACDll) return -1;

	// clear
	memset(&MACDll, 0, sizeof(MAC_COMPRESS_DLL));

	// load the functions
	MACDll.Create = (proc_APECompress_Create) GetProcAddress(hMACDll, "c_APECompress_Create");
	MACDll.Destroy = (proc_APECompress_Destroy) GetProcAddress(hMACDll, "c_APECompress_Destroy");
	MACDll.Start = (proc_APECompress_Start) GetProcAddress(hMACDll, "c_APECompress_Start");
	MACDll.AddData = (proc_APECompress_AddData) GetProcAddress(hMACDll, "c_APECompress_AddData");
	MACDll.Finish = (proc_APECompress_Finish) GetProcAddress(hMACDll, "c_APECompress_Finish");
	// error check
	if (!MACDll.Create || !MACDll.Destroy || !MACDll.Start  || !MACDll.AddData || !MACDll.Finish) {
		return -1;
	}
	return 0;
}

// Version checks the dll / interface
int CApe::VersionCheckInterface(HMODULE hMACDll)
{
	int nRetVal = -1;
	proc_GetInterfaceCompatibility GetInterfaceCompatibility = (proc_GetInterfaceCompatibility) GetProcAddress(hMACDll, "GetInterfaceCompatibility");
	if (GetInterfaceCompatibility) {
		nRetVal = GetInterfaceCompatibility(MAC_VERSION_NUMBER, TRUE, NULL);
	}
	return nRetVal;
}

/*******************************************************************************
* FFMPEG Encoder
*******************************************************************************/
CFFMpegAudio::CFFMpegAudio()
{ 
	flags = 0;
	prefNodePath = "audioenc.ffmpeg";
	logger.SetModuleName(GetName());
}

char* CFFMpegAudio::GetCodecName()
{
	CPrefBase prefbase(prefNodePath);
	static char *acodecname[] = {"mp2","mp3","ac3","amr_nb","amr_wb","adpcm_ima_wav","sonic"};
	return acodecname[pref.GetValueInt(".codec")];
}

int CFFMpegAudio::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	PARAM_OPT opts[] = {
		{".bitrate", "-ab "},
		{".samplerate", "-ar "},
		{0},
	};
	cl.Format("\"%s\" -f wav -i \"$(SourceFile)\" -ac $(ChannelNum)", pref.GetValueStr(".path"));
	if (char *p = GetCodecName())
		cl.AppendFormat(" -acodec %s", p);
	pref.GenOptions(cl, opts);
	cl.AppendFormat(" \"$(DestFile)\"");
	if (pref.GetValueBool("overall.tagging.tag") && item->info.tags.valid)
		cl += " -title \"$(Title)\" -author \"$(Artist)\" -comment \"$(Comment)\"";
	return 0;
}

char* CFFMpegAudio::GetOutExt(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	static char *extname[] = {"mp2","mp3","ac3","amr","amr","wav","sonic"};
	return extname[pref.GetValueInt(".codec")];
}

char* CFFMpegAudio::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

int CFFMpegAudio::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return pref.GetValueInt(".bitrate");
}

/*******************************************************************************
* MusePack Encoder 
*******************************************************************************/
CMusePack::CMusePack()
{
	prefNodePath = "audioenc.musepack";
	logger.SetModuleName(GetName());
}

int CMusePack::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	static const char* profiles[]={"telephone","thumb", "radio", "standard", "xtreme","insane","braindead"};
	cl.Format("\"%s\" --%s", pref.GetValueStr(".path"), profiles[pref.GetValueInt(".profile")]);
	if (pref.GetValueBool("overall.tagging.tag") && item->info.tags.valid)
		cl += " --title \"$(Title)\" --artist \"$(Artist)\" --album \"$(Album)\" --year \"$(Year)\" --track \"$(Track)\" --genre \"$(Genre)\" --tag Comment=\"$(Comment)\"";
	cl += " \"$(SourceFile)\" \"$(DestFile)\"";
	return 0;
}

char* CMusePack::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, "\n");
}

/*******************************************************************************
* Helix MP3 Encoder 
*******************************************************************************/
CHelixMP3::CHelixMP3()
{
	prefNodePath = "audioenc.helixmp3";
	logger.SetModuleName(GetName());
}

int CHelixMP3::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	PARAM_OPT opts[] = {
		{".stereo", "-M"},
		{".filtering", "-S0|-S1"},
		{".highFreq", "-HF"},
		{".sse", "-u2"},
		{".options", ""},
		{0},
	};
	int i;
	cl.Format("\"%s\" \"$(SourceFile)\" \"$(DestFile)\" -D", pref.GetValueStr(".path"));
	if (pref.GetValueBool(".mode") == 0) {
		cl.AppendFormat(" -V%d", pref.GetValueInt(".quality"));
	} else {
		if ((i = pref.GetValueInt(".bitrate")) == 0) {
			cl += " -B-1";
		} else {
			cl.AppendFormat(" -B%d", i);
		}
	}
	if (pref.GetValueInt(".mode") == 2 && (i = pref.GetValueInt(".subbands"))) {
		if (i == 0)
			cl += " -N-1";
		else
			cl.AppendFormat(" -N%d", (i - 1) * 4);
	}
	pref.GenOptions(cl, opts);
	return 0;
}

char* CHelixMP3::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

int CHelixMP3::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	return pref.GetValueBool(".mode") == 0 ? 0 : pref.GetValueInt(".bitrate");
}

/*******************************************************************************
* OptimFrog Encoder 
*******************************************************************************/
COptimFrog::COptimFrog()
{
	prefNodePath = "audioenc.ofr";
	flags = FLAG_STANDALONE | FLAG_READ_CONSOLE;
	logger.SetModuleName(GetName());
}

int COptimFrog::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	static const char* mode[]={"none","fast", "normal", "high", "best"};
	cl.Format( "\"%s\" --optimize %s \"$(SourceFile)\" --output \"$(DestFile)\"", pref.GetValueStr(".path"), mode[pref.GetValueInt(".mode")]);
	return 0;
}

char* COptimFrog::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, 0);
}

/*******************************************************************************
* AAC Lossless Encoder 
*******************************************************************************/
CMp4Als::CMp4Als()
{
	prefNodePath = "audioenc.als";
	flags = FLAG_RAW_PCM;
	logger.SetModuleName(GetName());
}

int CMp4Als::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	PARAM_OPT opts[] = {
		{".adaptive", "-a"},
		{".dualStereo", "-i"},
		{".ltp", "-p"},
		{".predictionOrder", "-o"},
		{".rlsmls", "-z"},
		{0},
	};
	cl.Format("\"%s\"", pref.GetValueStr(".path"));
	pref.GenOptions(cl, opts);
	cl += " -v -R -L$(AudioBytes) -F$(SampleRate) -W$(SampleBits) -C$(ChannelNum) \"$(SourceFile)\" \"$(DestFile)\"";
	return 0;
}

char* CMp4Als::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), "-h", 0);
}

/***********************************************************************************
* iTuneEncode
***********************************************************************************/
CItunesEnc::CItunesEnc()
{
	prefNodePath = "audioenc.itunes";
	flags = FLAG_STANDALONE | FLAG_READ_CONSOLE;
}

int CItunesEnc::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	cl.Format("\"%s\"%s", pref.GetValueStr(".path"), pref.GetValueBool(".keepFiles") ? "" : " -d");
	if (pref.GetValueBool("overall.tagging.tag") && item->info.tags.valid)
		cl += " -t \"$(Title)\" -a \"$(Artist)\" -l \"$(Album)\" -y \"$(Year)\" -n \"$(Track)\" -g \"$(Genre)\" -c \"$(Comment)\"";
	cl += " -i \"$(SourceFile)\" -o \"$(DestFile)\"";
	return 0;
}

/***********************************************************************************
* Nero AAC Encoder 
***********************************************************************************/
#define ASSERT
#include "AudioBase.h"
#include "AudioPluginManager.h"

typedef bool (* LPNERO_PLUGIN_GetPrimaryAudioObject)(IAudioComponent**);
typedef bool (* LPNERO_PLUGIN_ReadyToFinish)();

IAudioRawWriter *pAacIAURW;
IProcess *pAacIProcess;
LPNERO_PLUGIN_ReadyToFinish aacReadyToFinish;

int CNeroAAC::Open(CString& cmd, CString& destfile, T_ITEM* item)
{
	LPNERO_PLUGIN_GetPrimaryAudioObject aacGetPrimaryAudioObject;
	IConfigurable *pIC;
	IComponentEnum *pAacICU; 
	IAudioComponent *pAacIA, *pAacIAC;
	IURLAudioTargetFactory *pAacDest;
	IStatus *pIStatus;
	SWavFormat format;
	IUnknown *pIUAac;
	IInfoWriter *pAacIW;
	CAudioPluginMgr pluginMgr;
	IURLHolder *pIURLH;
	CPrefBase prefbase(prefNodePath);

	if (SetOptions()) return -1;

	{
	char curdir[MAX_PATH];
	GetCurrentDirectory(sizeof(curdir), curdir);
	SetCurrentDirectory(pref.GetValueStr(".path"));
	hAacEncDLL = LoadLibrary("aacenc32.dll");
	hAacDLL = LoadLibrary("aac.dll");
	SetCurrentDirectory(curdir);
	}

	if (!hAacEncDLL || !hAacDLL) return -1;
	format.m_iBitsPerSample = item->target.audio->samplebits;
	format.m_iChannels = item->target.audio->channels;
	format.m_iSamplesPerSecond = item->target.audio->samplerate;
	
	aacGetPrimaryAudioObject = (LPNERO_PLUGIN_GetPrimaryAudioObject)GetProcAddress(hAacDLL,"NERO_PLUGIN_GetPrimaryAudioObject");
	aacReadyToFinish = (LPNERO_PLUGIN_ReadyToFinish)GetProcAddress(hAacDLL,"NERO_PLUGIN_ReadyToFinish");
	if (!aacGetPrimaryAudioObject || !aacReadyToFinish || 
		!aacGetPrimaryAudioObject(&pAacIA) || !pAacIA ||
		pAacIA->GetType() != ACT_ComponentEnumerator ||
		pAacIA->QueryInterface(IID_IComponentEnum,(void **)&pAacICU) || !pAacICU ||
		pAacICU->GetCount() <= 1 ||
        !pAacICU->GetComponent(1,&pAacIAC) || !pAacIAC ||
		pAacIAC->GetType() != ACT_AudioURLTargetFactory ||
		pAacIAC->QueryInterface(IID_IConfigurable, (void**)&pIC) || !pIC ||
		pAacIAC->QueryInterface(IID_IURLAudioTargetFactory, (void**)&pAacDest) || !pAacDest ||
		!pAacIAC->Init(static_cast<IAudioPluginMgr*>(&pluginMgr), &pIStatus) || pIStatus ||
		!pAacDest->CreateURLAudioTarget(&pIUAac,format,&pIStatus) || pIStatus ||
		pIUAac->QueryInterface(IID_IAudioRawWriter, (void**)&pAacIAURW) || !pAacIAURW ||
		pIUAac->QueryInterface(IID_IInfoWriter, (void**)&pAacIW) || !pAacIAURW ||
		pIUAac->QueryInterface(IID_IURLHolder, (void**)&pIURLH) || !pIURLH ||
		!pIURLH->SetURL(destfile, &pIStatus) || pIStatus ||
		pIUAac->QueryInterface(IID_IProcess, (void**)&pAacIProcess) || !pAacIProcess ||
		!pAacIProcess->Start(&pIStatus) || pIStatus
	) {
		FreeLibrary(hAacDLL);
		FreeLibrary(hAacEncDLL);
		return -1;
	}
	return 0;
}

int CNeroAAC::Encode(char *data, int bytes)
{
	enum EAudioRawState state;
	IStatus *pIStatus;
	return (pAacIAURW->RawWrite((BYTE*)data, bytes, state, &pIStatus) && !pIStatus) ? bytes : 0;
}

int CNeroAAC::Close()
{
	IStatus *pIStatus;
	pAacIProcess->End(&pIStatus);
	aacReadyToFinish();
	FreeLibrary(hAacDLL);
	FreeLibrary(hAacEncDLL);
	return 0;
}

CNeroAAC::CNeroAAC()
{
	char* nerofiles[] = {"aacenc32.dll", "Aac.dll", 0};
	CString path = instinfo.apppath + "codecs\\";

	flags=FLAG_RAW_PCM;
	prefNodePath = "audioenc.neroaac";
	logger.SetModuleName(GetName());
	CPrefBase prefbase(prefNodePath);

	for (int i = 0; nerofiles[i]; i++) {
		CString dllfile = path + nerofiles[i];
		if (!FileExist(dllfile.GetBuffer())) break;
	}
	if (!nerofiles[i]) {
		pref.SetDefaultValue(".path", path.GetBuffer());
	} else {
		char buf[MAX_PATH];
		if (GetDLLDir(buf, MAX_PATH)) {
			char* p = buf + strlen(buf);
			if (*(p - 1) != '\\') *(WORD*)(p++) = '\\';
			strcpy(p, nerofiles[0]);
			if (FileExist(buf)) {
				*p = 0;
				pref.SetDefaultValue(".path", buf);
			}
		}
	}
	//remove NERO expiration
	RegDeleteKey(HKEY_CLASSES_ROOT, ".zhdb");
}

int CNeroAAC::GetDLLDir(char* buf, int bufsize)
{
	HKEY hKeyResult = 0;
	DWORD dwBytes = 0;
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\ahead\\shared", 0, KEY_READ, &hKeyResult) != ERROR_SUCCESS)
		return 0;
	dwBytes = bufsize;
	RegQueryValueEx(hKeyResult, "AudioPlugIn", 0, 0, (LPBYTE)buf, &dwBytes);
	RegCloseKey(hKeyResult);
	return dwBytes;
}

int CNeroAAC::SetOptions()
{
	CPrefBase prefbase(prefNodePath);
	HKEY hResult = 0;
	int ret = 0;
	DWORD dwVal = 0;
	static const char* keyname[] = {
		"DefAacProfile",
		"DefVbrMode",
		"DefBitRate",
		"DefCodecQuality",
		"DefDownmix",
		"DefExportAAC",
		"DefHintForStreaming",
		"DefUsePNS",
		"DefOptimize",
		0};
	DWORD keyvalue[] = {
		pref.GetValueInt(".profile") == 0 ? 4 : 5,
		(pref.GetValueInt(".mode") == 0) ? pref.GetValueInt(".quality") : 0,
		pref.GetValueInt(".bitrate"),
		pref.GetValueInt(".hq"),
		pref.GetValueInt(".downmix"),
		0,
		pref.GetValueInt(".hint"),
		pref.GetValueInt(".pns"),
		0};
	;

	if (RegCreateKeyEx(HKEY_CURRENT_USER,
		 "Software\\ahead\\Nero - Burning Rom\\Nero Digital Audio (HE-AAC)",
		 0,
		 "",
		 REG_OPTION_NON_VOLATILE,
		 KEY_ALL_ACCESS,
		 NULL,
		 &hResult,
		 &dwVal) != ERROR_SUCCESS) return -1;
	for (int i = 0; keyname[i]; i++) {
		dwVal = keyvalue[i];
		if (RegSetValueEx(hResult, keyname[i], 0, REG_DWORD, (LPBYTE)&dwVal, 4) != ERROR_SUCCESS) {
			ret = -1;
			break;
		}
	}
	RegCloseKey(hResult);
	RegFlushKey(HKEY_CURRENT_USER);
	return ret;
}

/*******************************************************************************
* Nero Reference Audio Encoder
*******************************************************************************/
CNeroRefEnc::CNeroRefEnc()
{
	prefNodePath = "audioenc.nero";
	flags = FLAG_TO_TAG;
	flags = FLAG_TO_TAG;
	logger.SetModuleName(GetName());
}

int CNeroRefEnc::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	PARAM_OPT opts[] = {
		{".hint", "-hinttrack"},
		{".format", "|-lc|-he|-hev2"},
		{0},
	};
	cl.Format("\"%s\" ", pref.GetValueStr(".path"));
	switch (pref.GetValueInt(".mode")) {
	case 1:
		cl.AppendFormat("-br %d", pref.GetValueInt(".bitrate") * 1000);
		break;
	case 2:
		cl.AppendFormat("-cbr %d", pref.GetValueInt(".bitrate") * 1000);
		break;
	default: {
		int q = pref.GetValueInt(".quality");
		cl.AppendFormat("-q %d.%02d", q / 100, q % 100);
		} break;
	}
	pref.GenOptions(cl, opts);
	cl += " -ignorelength -if \"$(SourceFile)\" -of \"$(DestFile)\"";
	return 0;
}

char* CNeroRefEnc::GetVersionInfo()
{
	CPrefBase prefbase(prefNodePath);
	return GetEncoderVersionInfo(pref.GetValueStr(".path"), 0, "\nERROR:");
}

int CNeroRefEnc::GetTargetBitRate(T_ITEM* item)
{
	CPrefBase prefbase(prefNodePath);
	if (pref.GetValueInt(".mode") != 0) {
		return pref.GetValueInt(".bitrate");
	} else {
		return 0;
	}
}
