/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#include "config.h"
#include <stdio.h>
#include "transcode.h"	
#include "processpil.h"
#include "datatypes.h"
#include "paramlist.h"
#include "preference.h"
#include "resource.h"
#include "common.h"
#include "winamphelper.h"
#include "audiosources.h"
#include "audioencoders.h"
#include "amr.h"

//extern CAudioEncoder* AudioEncoders[];
extern CAudioSource* AudioSources[];
extern MC_STATE* pinst;
extern PFN_EVENT_HANDLER pfnEvent;

int CAudioSource::GetPos()
{ 
	return bytesPerSec ? (int)BYTE2MS(bytescount, bytesPerSec) : 0;
}

int CAudioSource::prepare()
{
	bytescount = 0;
	bytesPerSec = item ? samplerate * channels * samplebits / 8 : 44100 * 4;
	totalbytes = duration > 0 ? MS2BYTE(duration, bytesPerSec) : -1;
	return 0;
}

int CAudioSource::read(char* buffer, int bufsize)
{
	DWORD bytes = 0;
	if (!hInput || !ReadFile(hInput, buffer, bufsize, &bytes, 0) || bytes == 0)
		return -1;
	bytescount += bytes;
	return bytes;
}

////////////////////////////////////////////////////////////////////////////////////
// Wave File Source
////////////////////////////////////////////////////////////////////////////////////
CWaveFileSource::CWaveFileSource()
{
	prefNodePath = "audiosrc.wavefile";
	logger.SetModuleName(GetName());
}

int CWaveFileSource::open(T_ITEM* item, int startpos)
{
	RIFF_HEADER hdr;
	DWORD bytes;
	totalbytes = -1;
	hInput = CreateFile(item->filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	if (hInput == INVALID_HANDLE_VALUE) return -1;
	this->item = item;
	ReadFile(hInput,&hdr,sizeof(RIFF_HEADER),&bytes,0);
	if (hdr.desc.chunkID=='FFIR' && hdr.desc.format=='EVAW' && hdr.fmt.chunkID==' tmf' && hdr.data.chunkID=='atad') {
		totalbytes = hdr.data.chunkSize;
		samplerate = hdr.fmt.dwSamplesPerSec;
		channels = hdr.fmt.wChannels;
		samplebits = hdr.fmt.wBitsPerSample;
		totalbytes = hdr.data.chunkSize;
	} else {
		SetFilePointer(hInput, 0, 0, FILE_BEGIN);
		channels = 2;
		samplerate = 44100;
		samplebits = 16;
		*((LPDWORD)&totalbytes) = GetFileSize(hInput,(LPDWORD)&totalbytes + 1);
	}
	item->info.duration = BYTE2MS(totalbytes, hdr.fmt.dwAvgBytesPerSec);
	if (prepare()) {
		close();
		return -1;
	}
	seek(startpos);
	return 0;
}

int CWaveFileSource::close()
{
	CloseHandle(hInput);
	hInput = 0;
	item = 0;
	return 0;
}

int CWaveFileSource::seek(int startpos)
{
	if (startpos) {
		long long bytepos = MS2BYTE(startpos, bytesPerSec) + sizeof(RIFF_HEADER);
		SetFilePointer(hInput, (DWORD)bytepos, (PLONG)&bytepos + 1, FILE_BEGIN);
	}
	return startpos;
}

////////////////////////////////////////////////////////////////////////////////////
// Lame Decoder Source
////////////////////////////////////////////////////////////////////////////////////
CLameDecoder::CLameDecoder()
{
	prefNodePath = "audiosrc.lame";
	logger.SetModuleName(GetName());
}

int CLameDecoder::open(T_ITEM* item, int startpos)
{
	CPrefBase prefbase(prefNodePath);
	CString cmd;
	PARAM_OPT opts[] = {
		{"audiosrc.lame.options", ""},
		{0}
	};
	totalbytes = -1;
	this->item = item;
	memset(&sp, 0, sizeof(sp));
	cmd.Format("\"%s\" --silent --decode -t \"%s\" -", pref.GetValueStr(".path"), item->filename);
	pref.GenOptions(cmd, opts, ' ');
	sp.pchCommandLine = cmd.GetBuffer();
	sp.flags = SHELL_REDIRECT_STDOUT;
	if (ShellExec(&sp)) return -1;
	// FIXME
	samplerate = item->info.audio.samplerate;
	samplebits = item->info.audio.samplebits;
	channels = item->info.audio.channels;
	if (prepare()) {
		close();
		return -1;
	}
	hInput = sp.fdRead;
	seek(startpos);
	return 0;
}

int CLameDecoder::close()
{
	ShellTerminate(&sp);
	ShellClean(&sp);
	hInput = 0;
	item = 0;
	return 0;
}

int CLameDecoder::seek(int startpos)
{
	return startpos;
}

////////////////////////////////////////////////////////////////////////////////////
// AMR Decoder
////////////////////////////////////////////////////////////////////////////////////
CAmrDecoder::CAmrDecoder()
{
	logger.SetModuleName(GetName());
}

int CAmrDecoder::open(T_ITEM* item, int startpos)
{
	CPrefBase prefbase(prefNodePath);
	char buf[8];
	totalbytes = -1;
	fp = fopen(item->filename,"rb");
	if (!fp) return -1;
	fread(buf, sizeof( char ), sizeof( AMRNB_MAGIC_NUMBER ) - 1, fp);
	if (memcmp(buf, AMRNB_MAGIC_NUMBER, sizeof(AMRNB_MAGIC_NUMBER) - 1)) {
		fclose(fp);
		return -1;
	}
	this->item = item;
	samplerate = 8000;
	channels = 1;
	samplebits = 16;
	frames = 0;
	if (prepare()) {
		close();
		return -1;
	}

	dll = new CDllHelper(pref.GetValueStr(".path"));
	if (!dll->hDll) {
		delete dll;
		logger.Log(IDS_FILE_NOT_FOUND, pref.GetValueStr(".path"));
		fclose(fp);
		return -1;
	}
	void* pfnAmrInit = dll->GetAddr("Decoder_Interface_init");
	pfnDecode = dll->GetAddr("Decoder_Interface_Decode");
	if (!pfnAmrInit || !pfnDecode) {
		delete dll;
		logger.Log(IDS_INCORRECT_VERSION, GetName());
		fclose(fp);
		return -1;
	}
	destate = (*(pfn_amrnb_dec_init)pfnAmrInit)();
	return 0;
}

int CAmrDecoder::close()
{
	pfn_amrnb_dec_exit pfnAmrExit;
	logger.Log(IDS_UNLOADING_DLL);
	pfnAmrExit = (pfn_amr_exit)dll->GetAddr("Dncoder_Interface_exit");
	if (pfnAmrExit) (*pfnAmrExit)(destate);
	fclose(fp);
	delete dll;
	item = 0;
	return 0;
}

int CAmrDecoder::read(char* buffer, int bytes)
{
	unsigned char analysis[32];
	static int block_size[16]={ 12, 13, 15, 17, 19, 20, 26, 31, 5, 0, 0, 0, 0, 0, 0, 0 };
	int dec_mode;
	
	if (bytes < AMRNB_FRAME_SIZE * sizeof(short)) return -1;
	if (fread(analysis, sizeof (unsigned char), 1, fp ) <= 0) return -1;
	dec_mode = (analysis[0] >> 3) & 0x000F;
	fread(&analysis[1], sizeof (char), block_size[dec_mode], fp );
	frames ++;
	(*(pfn_amrnb_decode)pfnDecode)(destate, analysis, (short*)buffer, 0);
	bytescount += AMRNB_FRAME_SIZE * sizeof(short);
	return AMRNB_FRAME_SIZE * sizeof(short);
}

////////////////////////////////////////////////////////////////////////////////////
// MPlayer Source
////////////////////////////////////////////////////////////////////////////////////

/*
char* strrstr(char* str, char* substr)
{
	char *p,*s=NULL;
	p=str;
	for(;;) {
		p=strstr(p,substr);
		if (!p) break;
		s=p++;
	}
	return s;
}
*/

int CMplayerSource::open(T_ITEM* item, int startpos)
{
	CPrefBase prefbase(prefNodePath);
	CString cmd;
	HANDLE hPipeWrite;
	CString file;

	totalbytes = -1;

	if (!hInput) {
		if (!CreatePipe(&hInput, &hPipeWrite, 0, AUDIO_BLOCK_SIZE)) return -1;
		SetHandleInheritance(&hPipeWrite, TRUE);
		if (pref.GetValueInt("overall.task.mode") != TM_JOIN) {
			file.Format("\"%s\"", item->filename);
		} else {
			for (T_ITEM* itemtmp = item; itemtmp; itemtmp = itemtmp->next) {
				file.AppendFormat("\"%s\" ", itemtmp->filename);
			}
		}
		if (!mcinst.fAudioEncoding) {
			cmd.Format("\"%smencoder.exe\" %s -quiet -oac pcm -pcmopts pipe=%d:pipeonly -ovc frameno -of rawvideo -o nul",
				pref.GetValueStr("overall.mencoder.path"),
				file.GetBuffer(),
				hPipeWrite);
			GenCommonOptions(item, cmd, 1);
		} else {
			cmd.Format("\"%s%s\" %s -quiet -vo null -vc null -ao pcm:nowaveheader:fast:pipe=%d",
				pref.GetValueStr("overall.mplayer.path"),
				pref.GetValueBool(".dll") ? MP_DLL : MP_EXE,
				file.GetBuffer(),
				hPipeWrite);
			/*
			cmd.AppendFormat("-demuxer rawvideo -rawvideo w=1:h=1 -audiofile %s -quiet -of rawaudio -ovc copy",
				file.GetBuffer());
			*/
			GenCommonOptions(item, cmd, 0);
		}
		GenAudioFilterOptions(item, cmd, samplerate);
		if (startpos) {
			cmd.AppendFormat(" -ss %d", startpos / 1000);
		}
		if (mcinst.audiochannel && !pref.GetValueBool("audiofilter.internal.downmix")) {
			cmd.AppendFormat(" -aid %d", mcinst.audiochannel);
		}
		pfnEvent(MC_MSG_CMD, cmd.GetBuffer());
		if (pref.GetValueBool(".dll")) {
			dll = new CDllHelper;
			if (!dll->LoadDLL(pref.GetValueStr(".path"), MP_DLL)) {
				logger.Log(IDS_FILE_NOT_FOUND, "MPlayer DLL");
				close();
				return -1;
			}
			if (dll->StartMain(cmd.GetBuffer())) {
				logger.Log(IDS_ERROR_DECODE_AUDIO, GetName());
				close();
				return -1;
			}
			SetThreadPriorityBySetting(dll->hThread);
		} else {
			sp = (SHELL_PARAM*)calloc(1, sizeof(SHELL_PARAM));
			sp->pchCommandLine = cmd.GetBuffer();
			if (ShellExec(sp)) {
				CloseHandle(hInput);
				hInput = 0;
				return -1;
			}
		}
		CloseHandle(hPipeWrite);
	}
	DWORD bytes = 0;
	for (int i = 0; (!PeekNamedPipe(hInput,0,0,0,&bytes,0) || bytes == 0) && i < 200; i++) {
		if (dll) {
			if (dll->Wait(50) != WAIT_TIMEOUT) {
				// MPlayer exited unexpectedly
				close();
				return -1;
			}
		} else if (sp) {
			if (ShellWait(sp, 100)) {
				// MPlayer exited unexpectedly
				close();
				return -1;
			}
		} else {
			Sleep(100);
		}
	}
	if (bytes == 0) {
		// not data received from info pipe
		close();
		return -1;
	}

	if (!ReadFile(hInput, &samplerate, sizeof(int), &bytes, 0) ||
		!ReadFile(hInput, &channels, sizeof(int), &bytes, 0)
		) {
		close();
		return -1;
	}
	this->item = item;
	samplebits = 16; 
	prepare();
	return 0;
}

int CMplayerSource::extract(T_ITEM* item, char* destfile, int startpos)
{
	CString cmd;
	cmd.Format("\"%smplayer.exe\" \"%s\" -quiet -vo null", pref.GetValueStr("overall.mplayer.path"), item->filename);
	GenAudioFilterOptions(item, cmd, 0);
	GenCommonOptions(item, cmd, 0);
	if (startpos) {
		cmd.AppendFormat(" -ss %d", startpos / 1000);
	} else {
		cmd += " -vc dummy";
	}

	cmd += " -ao pcm:file=";
	cmd += destfile;
	sp = (SHELL_PARAM*)calloc(1, sizeof(SHELL_PARAM));
	sp->pchCommandLine = cmd.GetBuffer();
	pfnEvent(MC_MSG_CMD, sp->pchCommandLine);
	if (ShellExec(sp)) return -1;
	while (!ShellWait(sp, 1000) && inst.state==STATE_ENCODING);
	ShellClean(sp);
	free(sp);
	return FileExist(destfile) ? 0 : -1;
}

int CMplayerSource::close()
{
	if (dll) {
		dll->Stop();
		delete dll;
		dll = 0;
	} else if (sp) {
		ShellTerminate(sp);
		ShellClean(sp);
		free(sp);
		sp = 0;
	}
	CloseHandle(hInput);
	hInput = 0;
	state=0;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////
// Winamp Input Plugin Source
////////////////////////////////////////////////////////////////////////////////////

#define IN_BUFFER_SIZE 16*1024
static CWinampInput* src=0;
static HANDLE hPipeWrite = 0, hEventFinish;
static BOOL fInputReady = false;

// our fake output module....
extern "C" static int Open(int srate, int nch, int bitspersamp, int bufferlenms, int prebufferms) {
	src->samplerate = srate;
	src->channels = nch;
	src->samplebits = bitspersamp;
	return 0;
}

extern "C" static int CanWrite()
{
	if (fInputReady) {
		return (inst.state==STATE_ENCODING) ? AUDIO_BLOCK_SIZE : -1;
	}
	SetThreadPriorityBySetting(GetCurrentThread());
	return -1;
}

extern "C" static int IsPlaying() {
	// arriving here indicates the end of the stream
	fInputReady = false;
	if (hPipeWrite) {
		CloseHandle(hPipeWrite);
		hPipeWrite = 0;
	}
	if (src->hInput) {
		CloseHandle(src->hInput);
		src->hInput = 0;
	}
	SetEvent(hEventFinish);
	return 0;
}

extern "C" static int Write(char *buf, int len) {
	if (len>0) {
		DWORD bytes;
		return (hPipeWrite && WriteFile(hPipeWrite, buf, len, &bytes, 0)) ? 0 : -1;
	} else {
		IsPlaying();
		return 0;
	}
}

extern "C" static int WriteWaveFile(char *buf, int len) {
	if (len>0) {
		DWORD dwBytes;
		if (!WriteFile(src->hWaveFile, buf, len, &dwBytes, 0)) return -1;
		return 0;
	} else {
		fInputReady = false;
		return 0;
	}
}

extern "C" static void Close(){
}

extern "C" Out_Module outmod;
extern "C" INPLUGIN_INFO inplugins;

int CWinampInput::open(T_ITEM* item, int startpos)
{
	totalbytes = -1;
	if (!CreatePipe(&hInput, &hPipeWrite, 0, AUDIO_BLOCK_SIZE * 2)) return -1;
	this->item = item;
	hEventFinish = CreateEvent(0, FALSE, FALSE, 0);
	bytescount = 0;
	totalbytes = 0;
	src=this;
	mod=OpenInputPluginByExt(item->filename);
	if (!mod) {
		logger.Log(IDS_ERROR_DECODE_AUDIO, GetName());
		return -1;
	}
	logger.Log(IDS_LOADED, mod->description);
	outmod.Write = Write;
	outmod.CanWrite = CanWrite;
	outmod.IsPlaying = IsPlaying;
	outmod.Open = Open;
	mod->GetFileInfo(item->filename, 0, &item->info.duration);
	if(mod->Play(item->filename)) {
		close();
		return -1;
	}
	mod->Pause();
	if (prepare()) {
		close();
		return -1;
	}
	if (startpos) seek(startpos);
	if (mod->IsPaused()) mod->UnPause();
#ifndef CONFIG_WIN9X
	SetPriorityClass(GetCurrentProcess(), BELOW_NORMAL_PRIORITY_CLASS);
#else
	SetPriorityClass(GetCurrentProcess(), IDLE_PRIORITY_CLASS);
#endif
	hEventBufferReady = CreateEvent(0, FALSE, FALSE, 0);
	fInputReady = true;
	return 0;
}

int CWinampInput::extract(T_ITEM* item, char* outfile, int startpos)
{
	CPrefBase prefbase(prefNodePath);
	DWORD dwBytes;
	RIFF_HEADER hdr;
	hEventFinish = CreateEvent(0, FALSE, FALSE, 0);
	fInputReady = false;
	src = this;
	InitRiffHeader(&hdr);
	hdr.data.chunkSize = 0;
	mod=OpenInputPluginByExt(item->filename);
	if (!mod) return -1;
	outmod.Write = WriteWaveFile;
	outmod.CanWrite = CanWrite;
	outmod.IsPlaying = IsPlaying;
	outmod.Open = Open;
	if(mod->Play(item->filename)) {
		UnloadWinampInPlugin(mod);
		mod = 0;
		return -1;
	}
	InvalidiateRiffHeader(&hdr, &item->info.audio);
	hWaveFile = CreateFile(outfile,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,0,NULL);
	if (hWaveFile == INVALID_HANDLE_VALUE) {
		stop();
		UnloadWinampInPlugin(mod);
		return -1;
	}
	WriteFile(hWaveFile, &hdr, sizeof(RIFF_HEADER), &dwBytes, NULL);
	logger.Log(IDS_EXTRACTING_WAVE, mod->description);
	if (startpos) seek(startpos);
#ifndef CONFIG_WIN9X
	SetPriorityClass(GetCurrentProcess(), BELOW_NORMAL_PRIORITY_CLASS);
#else
	SetPriorityClass(GetCurrentProcess(), IDLE_PRIORITY_CLASS);
#endif
	fInputReady = true;
	WaitForSingleObject(hEventFinish, INFINITE);
	stop();
	UnloadWinampInPlugin(mod);
	mod=0;
	src=0;
	SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
	hdr.data.chunkSize = (DWORD)totalbytes;
	hdr.desc.chunkSize = hdr.data.chunkSize + 36;
	SetFilePointer(hWaveFile,0,NULL,FILE_BEGIN);
	WriteFile(hWaveFile, &hdr, sizeof(RIFF_HEADER), &dwBytes, NULL);
	CloseHandle(hWaveFile);
	CloseHandle(hEventFinish);
	return 0;
}

int CWinampInput::seek(int startpos)
{
	mod->SetOutputTime(startpos);
	return startpos;
}

int CWinampInput::close()
{
	if (!mod) return -1;
	logger.Log(IDS_UNLOADED, mod->description);
	if (hInput) {
		CloseHandle(hInput);
		hInput = 0;
	}
	if (hPipeWrite) {
		CloseHandle(hPipeWrite);
		hPipeWrite = 0;
	}
	SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
	CloseHandle(hEventFinish);
	stop();
	UnloadWinampInPlugin(mod);
	mod=0;
	src=0;
	return 0;
}

int CWinampInput::stop()
{
	mod->Stop();
	if (fInputReady) {
		IsPlaying();
		logger.Log(IDS_STOPPED, mod->description);
	}
	return 0;
}

In_Module* CWinampInput::OpenInputPluginByExt(char* filename) {
	char* wapath = pref.GetValueStr("audiosrc.winamp.path");
	char buf[MAX_PATH];
	INPLUGIN_INFO* cur;
	char* ext=strrchr(filename,'.');
	if (!ext)
		ext="";
	else
		ext++;

	for (cur = inplugins.next; cur; cur = cur->next) {
		if (ISBITSET(cur->flags, FLAG_DISABLED)) continue;
		sprintf(buf,"%s%s", wapath, cur->pluginfile);
		In_Module* inmod=LoadWinampInPlugin(buf, 0);
		if (inmod) {
			char *p;
			strcpy(buf,cur->exts);
			if (buf[0]) {
				for (p = strtok(buf,";"); p && stricmp(ext,p); p=strtok(0,";"));
				if (!p) {
					UnloadWinampInPlugin(inmod);
					continue;
				}
			} else {
				if(inmod->Play(filename)) {
					UnloadWinampInPlugin(inmod);
					continue;
				}
				inmod->Stop();
			}
			logger.Log(IDS_FOUND_MATCH_PLUGIN, ext);
			return inmod;
		}
	}
	return 0;
}
