/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*************************************************************************/

#include "httpd.h"
#include "httpget.h"
#define TEST_PATTERN "mchttptest%d"

void DoRegisterHandlers(CHttpd* httpd);
static char* testPattern = 0;

static int uhTest(UrlHandlerParam* param)
{
	if (!testPattern) return 0;
	param->iDataBytes = sprintf(param->pucBuffer, "%s", testPattern);
	param->fileType = HTTPFILETYPE_TEXT;
	return FLAG_DATA_RAW;
}

#define HTTPD_ATTEMPTS 10

int CHttpd::Start(int port, int maxclient, char* webroot, bool local)
{
	char buf[64];
	sprintf(buf, TEST_PATTERN, GetCurrentProcessId());
	testPattern = strdup(buf);
	
	memset(&httpParam, 0, sizeof(httpParam));
	httpParam.maxClients = maxclient ? maxclient : 32;
	httpParam.maxReqPerConn = 99;
	httpParam.pchWebPath = webroot ? webroot : "webroot";
	httpParam.pxUrlHandler = (UrlHandler*)calloc(1, sizeof(UrlHandler));
	httpParam.flags = local ? FLAG_LOCAL_BIND : 0;
	httpParam.httpPort = port;
	DoRegisterHandlers(this);
	RegisterHandler("~test", uhTest);
	//test HTTP connection
	for (int attempt = 0; attempt < HTTPD_ATTEMPTS; attempt++, port++) {
		httpParam.httpPort = port;
		if (mwServerStart(&httpParam)) continue;
		httpRequest req;
		memset(&req, 0, sizeof(req));
		sprintf(buf, "http://127.0.0.1:%d/~test", port);
		req.url = buf;
		req.buffer = buf;
		req.iDataSize = sizeof(buf);
		if (!httpGet(&req) && !strcmp(buf, testPattern)) {
			break;
		}
		mwServerShutdown(&httpParam);
	}
	UnregisterHandler("~test");
	if (attempt == HTTPD_ATTEMPTS) {
		return 0;
	}
	return port;
}

bool CHttpd::IsRunning()
{
	return httpParam.bWebserverRunning ? true : false;
}

void CHttpd::Stop()
{
	if (IsRunning()) {
		mwServerShutdown(&httpParam);
		UnregisterHandler("~stats");
		free(httpParam.pxUrlHandler);
		if (testPattern) free(testPattern);
	}
}

int CHttpd::RegisterHandler(char* prefix, PFNURLCALLBACK handler)
{
	int n;
	for (n = 0; httpParam.pxUrlHandler[n].pchUrlPrefix; n++) {
		if (!strcmp(prefix, httpParam.pxUrlHandler[n].pchUrlPrefix)) return 0;
	}
	if (prefix) {
		httpParam.pxUrlHandler = (UrlHandler*)realloc(httpParam.pxUrlHandler, sizeof(UrlHandler) * (n + 2));
		memset(&httpParam.pxUrlHandler[n + 1], 0, sizeof(UrlHandler));
		httpParam.pxUrlHandler[n].pchUrlPrefix = prefix;
		httpParam.pxUrlHandler[n].pfnUrlHandler = handler;
	}
	return n;
}

int CHttpd::UnregisterHandler(char* prefix)
{
	int n;
	for (n = 0; httpParam.pxUrlHandler[n].pchUrlPrefix && strcmp(prefix, httpParam.pxUrlHandler[n].pchUrlPrefix); n++);
	if (!httpParam.pxUrlHandler[n].pchUrlPrefix) return -1;
	for (; ; n++) {
		httpParam.pxUrlHandler[n] = httpParam.pxUrlHandler[n + 1];
		if (!httpParam.pxUrlHandler[n].pchUrlPrefix) break;
	}
	return 0;
}
