/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*
* The include file for MediaCoder Core DLL
*************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
* Riff header structure
*******************************************************************************/
typedef struct {
	unsigned long  chunkID;
	long	chunkSize;
	long	format;
} CHUNK_DESC;

typedef struct {
  unsigned long  chunkID;
  long           chunkSize;
  short          wAudioFormat;
  unsigned short wChannels;
  unsigned long  dwSamplesPerSec;
  unsigned long  dwAvgBytesPerSec;
  unsigned short wBlockAlign;
  unsigned short wBitsPerSample;
/* Note: there may be additional fields here, depending upon wAudioFormat. */
} FORMAT_CHUNK;

typedef struct {
	unsigned long chunkID;
	long chunkSize;
} DATA_CHUNK;

typedef struct {
	CHUNK_DESC		desc;
	FORMAT_CHUNK	fmt;
	DATA_CHUNK		data;
} RIFF_HEADER;

/*******************************************************************************
* Audio/Video/Container identities
*******************************************************************************/
typedef enum {
	AC_LAMEMP3 = 0,
	AC_VORBIS,
	AC_NEROREF,
	AC_3GPPAAC,
	AC_AACPLUS,
	AC_FAAC,
	AC_HELIX,
	AC_WMA,
	AC_HMP3,
	AC_MPC,
	AC_FFMPEG,
	AC_AMR,
	AC_WAVPACK,
	AC_FLAC,
	AC_APE,
	AC_ALS,
	AC_OFR,
	AC_CLI,
	AC_PCM,
	AC_COPY,
	AC_NULL,
	AC_NONE,
	AC_COUNT,
	AC_ITUNES,
	AC_NEROAAC,
} AUDIO_CODEC;

typedef enum {
	VC_NONE=0,
	VC_COPY,
	VC_XVID,
	VC_H264,
	VC_MPEG1,
	VC_MPEG2,
	VC_MPEG4,
	VC_MSMPEG4V2,
	VC_H261,
	VC_H263,
	VC_H263P,
	VC_SNOW,
	VC_DIVX3,
	VC_WMV7,
	VC_WMV8,
	VC_FLV,
	VC_DV,
	VC_MJPEG,
	VC_LJPEG,
	VC_COUNT,
} VIDEO_CODEC;

typedef enum {
	VE_MENCODER = 0,
	VE_FFMPEG,
	VE_X264,
	VE_COUNT,
} VIDEO_ENCODER;

typedef enum {
	CF_DEFAULT = 0,
	CF_AVI,
	CF_MKV,
	CF_MP4,
	CF_MPEG,
	CF_LAVF,
	CF_PMP,
	CF_RAW,
	CF_COUNT,
} CONTAINER_FORMAT;

typedef enum {
	MT_MEDIAINFO = 0,
	MT_SHIBATCH,
	MT_COUNT,
} MEDIA_TOOLS;

// Track Types
enum {
	TT_DVD = 0,
	TT_VCD,
	TT_CDDA,
	TT_CUESHEET,
	TT_HTTP,
	TT_FTP,
	TT_RTSP,
	TT_MMS,
	TT_UDP,
	TT_LINEIN,
	TT_COUNT
};

/*******************************************************************************
* Transcoding information structures
*******************************************************************************/
typedef enum {
	ST_IDLE=0,					// no work is in progress
	ST_EXTRACTING_PCM,			// raw PCM is being extracted
	ST_PROCESSING_AUDIO,		// audio is being transcoded
	ST_PROCESSING_VIDEO,		// video is being transcoded
	ST_MUXING,					// audio/video streams are being muxed
} STAGES;

typedef struct {
	char* artist;
	char* title;
	char* album;
	char* genre;
	char* comment;
	short year;
	short track;
	short trackcount;
	short valid;
} TAGS;

typedef enum {
	CT_UNKNOWN=0,
	CT_AUDIO,
	CT_VIDEO,
	CT_SUBTITLE,
	CT_CUE,
	CT_QUEUE,
	CT_SPECIAL,
} TRACK_TYPE;

typedef enum {
	FT_UNKNOWN = 0, FT_MPEG_PS, FT_MPEG_TS, FT_MP4, FT_MPA, FT_3GP, FT_MP3, FT_M4A, FT_OGG, FT_AAC, FT_RA, FT_RM, FT_AMR,
	FT_WAV, FT_MPC, FT_WMA, FT_APE, FT_FLAC, FT_WV, FT_ALS, FT_CDA, FT_MKA, FT_MKV, FT_AVI, FT_MOV, FT_WMV, FT_263, FT_264,
	FT_CUE, FT_M3U, FT_IDX, FT_SUB, FT_NET, 
} EXT_TYPE;

typedef struct {
	char* fileext;					// file extension name
	EXT_TYPE exttype;				// extension type
	char* desc;					// description text of file type
	TRACK_TYPE type;				// track type
} FILE_TYPE;

typedef struct {
	int bitrate;					// audio bit rate
	int bitratemode;				// bit rate mode (1 for CBR, 2 for VBR, 0 for unknown)
	int samplerate;					// sampling rate
	int channels;					// number of channels
	int samplebits;					// bits of samples
	char* codec;					// audio codec name string
} ATTR_AUDIO;

typedef struct {
	int bitrate;					// video bit rate
	int width;						// video width in pixels
	int height;						// video height in pixels
	int fps;						// video frame rate
	int fpsScale;					// video frame rate scale
	char* codec;					// video codec name string
} ATTR_VIDEO;

typedef struct {
	FILE_TYPE* filetype;			// file type information
	int duration;					// duration in millisecond
	long long filesize;				// file size in bytes
	char* container;				// container name string
	ATTR_VIDEO video;				// video attributes
	ATTR_AUDIO audio;				// audio attributes
	char* discroot;				// root path of the disc
	int track;						// track number of the disc
	int chapter;					// chapter number of the track
	int begin;						// begin position
	int end;						// end position
	int audiodelay;					// audio delay
	TAGS tags;						// file tags
} MEDIA_INFO;

typedef struct {
	short x;
	short y;
	short w;
	short h;
} CROP_SETTINGS;

typedef struct {
	CONTAINER_FORMAT container;		// target container
	VIDEO_ENCODER videoenc;			// target video encoder
	AUDIO_CODEC audioformat;		// target audio format
	VIDEO_CODEC videoformat;		// target video format
	unsigned int destsize;			// desired stream size (in KB)
	float ratio;					// compression ratio
	int splitDuration;				// split duration (ms)
	ATTR_AUDIO* audio;
	ATTR_VIDEO video;
	CROP_SETTINGS crop;
} TARGET_SETTINGS;

#define IF_PRIVATE_FORMAT 0x1		// use private target format setting
#define IF_INFO_RETRIEVED 0x2		// information of the media file is retrieved
#define IF_DONE 0x4					// the item is done with processing

typedef struct _T_ITEM {
	struct _T_ITEM *next;			// point to the next T_ITEM entry
	char* filename;					// source file name
	char* subfile;					// subtitle file
	char* destfile;					// destination file name
	unsigned int flags;				// flags of the item
	MEDIA_INFO info;				// information of the source file
	TARGET_SETTINGS target;			// target settings
} T_ITEM;

typedef enum {
	MC_MSG_INFO = 1,				// display information message
	MC_MSG_ALERT,					// display alert message
	MC_MSG_CMD,						// display command line
	MC_MSG_POPUP,					// pop up a message
	MC_MSG_YESNOCANCEL,				// display a YES/NO/CANCEL confirmation message
	MC_MSG_YESNO,					// display a YES/NO confirmation message
	MC_UPDATE_LOG,					// update log text display
	MC_UPDATE_PROGRESS,				// update progress display
	MC_UPDATE_STATS,				// update statistics display
	MC_UPDATE_TIME,					// update time display
	MC_UPDATE_TITLE,				// update window title
	MC_UPDATE_STATE,				// update state display
	MC_UPDATE_UI,					// update user interface
	MC_WRITE_CONSOLE,				// update console display
	MC_JOB_START,					// indicate a job is to be started
	MC_JOB_FINISH,					// indicate a job is to be finished
} MC_EVENT;

// event results
#define ER_OK 0
#define ER_BYPASSED -1

typedef enum {
	EA_HALT = 0,
	EA_CONTINUE,
} EVENT_ACTIONS;

typedef struct {
	unsigned int pos;
	unsigned int frame;
	float percent;
	float fps;
	unsigned int remaintime;
	unsigned int cursize;
	unsigned int estsize;
	unsigned int vbitrate;
	unsigned int abitrate;
	unsigned int skippedframes;
	unsigned int badframes;
	unsigned int duplicatedframes;
	unsigned int audiosamples;
	unsigned int videosamples;
} VIDEO_STATS;

typedef struct {
	float percent;
	int pos;
	int speed;
	int size;
	int length;
	int bitrate;
	int estsize;
	long long bytes;
} AUDIO_STATS;

typedef struct {
	DWORD dwEncStartTime;
	DWORD dwTaskStartTime;
	DWORD dwClipStartTime;
	DWORD dwElapsed;
	unsigned int decodedMB;
	unsigned int encodedKB;
	int fileprocessed;
	int encodedFiles;
	int skippedFiles;
	int errorFiles;
	float percent;
} TASK_STATS;

typedef struct _MC_AUDIO_FILTER {
	int (*pfnAudioDsp)(short* pwSamples, int iNumOfSamples);
	int samplerate;
	int channels;
	int bps;
	struct _MC_AUDIO_FILTER* next;
} MC_AUDIO_FILTER;

typedef enum {
	STATE_IDLE=0,		// no work in progress
	STATE_ENCODING,		// task is in progress
	STATE_STOPPING,		// task is going to be stopped
	STATE_PLAYING,		// a file is playing (when not in transcoding progress)
	STATE_SKIPPING,		// a file is going to be skipped
	STATE_ERROR,		// an has been encountered
	STATE_SHUTDOWN,		// the program is going to shut down
} STATES;

/*******************************************************************************
* CPU information
*******************************************************************************/
typedef struct cpucaps_s {
	int cpuType;
	int cpuStepping;
	int hasMMX;
	int hasMMX2;
	int has3DNow;
	int has3DNowExt;
	int hasSSE;
	int hasSSE2;
	int isX86;
	unsigned cl_size; /* size of cache line */
	int hasAltiVec;
	int hasTSC;
	int freq;
	char vendor[20];
	char* cpuFullName;
} CpuCaps;

/*******************************************************************************
* transcoding state
*******************************************************************************/
typedef struct {
	STATES state;					// current state
	STAGES stage;					// current stage of the job
	int currentpass;				// current pass of the job
	int totalpass;					// number of passes of the job
	int totalfiles;					// number of files in the queue
	TASK_STATS stats;				// task statistics
	AUDIO_STATS astats;				// audio statistics
	VIDEO_STATS vstats;				// video statistics
	CpuCaps cpucaps;				// CPU capability information
	int httpdPort;
} MC_STATE;

typedef int (*PFN_EVENT_HANDLER)(MC_EVENT event, void* arg);

/*******************************************************************************
* API functions to start and control transcoding task
*******************************************************************************/
int mcTranscodeItem(T_ITEM* item);
int mcTranscodeQueue(T_ITEM* queue);
int mcStopTranscode();
STATES mcGetState();
void mcSetState(STATES state);
void mcInvalidiateStatsData();

/*******************************************************************************
* API functions to perform initialization and clean-ups
*******************************************************************************/
MC_STATE* mcInit(
	PFN_EVENT_HANDLER pfnEventCallback	// pointer to event handler function
);

int mcPostInit(HWND hwndMain);
void mcUninit();

/*******************************************************************************
* API functions and data types related with file type identification
*******************************************************************************/
typedef enum {
	DUMMY_FLAG = 0,
} CF_FLAGS;

FILE_TYPE* mcIdentifyFileType(char *filename);
FILE_TYPE* mcGetFileTypeByIndex(int index);

/*******************************************************************************
* API functions and data types related with container
*******************************************************************************/
typedef struct {
	char* filename;
	TRACK_TYPE type;
	int flags;
	int delay;
	int trackid;
} CONTAINER_TRACKS;

int mcContainerAddFile(TRACK_TYPE type, char* filename, CF_FLAGS flags);
int mcContainerMuxFiles(T_ITEM* item, char* destfile);
int mcContainerPurge(T_ITEM* item, int fDelete);
char* mcContainerGetFile(TRACK_TYPE type, int num);

/*******************************************************************************
* API functions to create/add/remove transcoding items
*******************************************************************************/
int mcAddItem(T_ITEM* itemtoadd);
void mcRemoveAllItems();
int mcRemoveItem(T_ITEM* item);
T_ITEM* mcMakeItem(char* filename);
void mcFreeItem(T_ITEM* item);
void mcAlloc(void** pmem, int bytes);
void mcFree(void** pmem);
char* mcStrDup(char* str);
void mcSetItemStr(char** pstr, char* str);
T_ITEM* mcGetItemByName(char* filename);
T_ITEM* mcGetItemByIndex(int index);
T_ITEM* mcGetNextItem(T_ITEM* prev);

/*******************************************************************************
* API functions to load/save/get/set preference values
*******************************************************************************/
int mcLoadParam(char* cfgfile, int fInit);
int mcSaveParam(char* cfgfile);
void mcApplyParam();
void* mcGetPrefValue(char* key);
int mcGetPrefValueInt(char* key);
char* mcGetPrefValueStr(char* key);
int mcGetPrefValueStrMax(char* key);
char* mcGetPrefValueEnum(char* key, int index);
float mcGetPrefValueFloat(char* key);
void mcSetPrefValue(char* key, void* value);
void mcSetPrefValueInt(char* key, int value);
void mcSetPrefValueFloat(char* key, float value);
void mcSetPrefValueStr(char* key, char* value);
int mcSetPrefValueXML(char* xml, char* basekey);
char* mcGetPrefValueXML(char* key, int level, char* basekey);

typedef enum { 
	PP_KEY = 0,
	PP_TYPE,
	PP_FLAG,
	PP_RANGE,
	PP_CAPTION,
	PP_DESC,
} PARAM_PROPERTY;

int mcGetPrefProperty(char* ley, PARAM_PROPERTY property, void* pval);


/*******************************************************************************
* Task mode
*******************************************************************************/
typedef enum {
	TM_NORMAL = 0,
	TM_SPLIT,
	TM_JOIN
} TASK_MODE;

/*******************************************************************************
* Show configuration window
*******************************************************************************/
typedef enum {
	CD_PREF = 0,
	CD_AUDIO_SRC,
	CD_DSP_PLUGIN,
	CD_AUDIO_ENC,
	CD_CONTAINER,
	CD_SETUP,
} CONFIG_DIALOG;

char* mcShowConfig(CONFIG_DIALOG type, void* arg);

/*******************************************************************************
* mcGetToolName / mcGetToolVersion
* Get the name / version of a tool (including decoders, encoders, container wrappers
* and taggers
*******************************************************************************/
typedef enum {
	TL_AUDIO_ENCODER = 0,
	TL_AUDIO_DECODER,
	TL_VIDEO_CODEC,
	TL_VIDEO_ENCODER,
	TL_VIDEO_DECODER,
	TL_CONTAINER,
	TL_TAGGER,
	TL_TOOL,
} MC_TOOLS;

char* mcGetToolName(MC_TOOLS type, int nametype, int id);
char* mcGetToolVersion(MC_TOOLS type, int id);

/*******************************************************************************
* Get the command line of a tool (including decoders, encoders, container
* wrappers and taggers
*******************************************************************************/
char* mcGetToolCommand(MC_TOOLS type, int pass);

/*******************************************************************************
* Call and run media tools (tagger, file info reader etc.)
*******************************************************************************/
char* mcRunMediaTool(MEDIA_TOOLS type, int action, T_ITEM* item);

/*******************************************************************************
* Media player related API functions
*******************************************************************************/
int mcPlayerStart(T_ITEM* item, HWND hWndVideo, int flags, char* extraopts);

#define PF_NOFILTER 0x1
#define PF_PREVIEW 0x2
#define PF_READ_CONSOLE 0x4

int mcPlayerStop();
int mcPlayerCommand(char* cmd);
int mcCropDetectStart(T_ITEM* item, HWND hWndVideo);
void mcCropDetectGetResult(int result[], BOOL fStop);

/*******************************************************************************
* mcGetMediaInfo
* Retrieve the information of a media file
*******************************************************************************/
int mcGetMediaInfo(T_ITEM* item);

/*******************************************************************************
* mcGetTargetParam/mcSetTargetParam
* Get/Set target settings
*******************************************************************************/
typedef enum {
	TP_AUDIO_BITRATE,
	TP_AUDIO_BITRATE_MODE,
	TP_VIDEO_BITRATE,
	TP_VIDEO_BITRATE_MODE,
} TARGET_PARAM;

int mcGetTargetParam(TARGET_PARAM type);
int mcSetTargetParam(TARGET_PARAM type, int val);

/*******************************************************************************
* CD/VCD/DVD/CueSheet handling API functions
*******************************************************************************/
char* mcGetDiscInfo(int disctype, int track, char* discroot);
int mcGetDriveTrackCount(int drvno);
int mcGetDriveCount();

/*******************************************************************************
* Misc. API functions
*******************************************************************************/
char* mcGetLogText(int maxsize);
void mcSetLogText(char* logtext);
void mcLogInfo(int id, char* arg);
int mcStartHttpd();
void mcStopHttpd();
char* mcGetVersionInfo();
int mcGetBuildNo();
char* mcGetLastError();

#ifdef __cplusplus
}
#endif
