/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/
#include <windows.h>
#include <stdio.h>
#include "processpil.h"
#include "windowhandle.h"
#include "datatypes.h"
#include "transcode.h"
#include "mplayerwrapper.h"
#include "paramlistdef.h"
#include "preference.h"

extern PFN_EVENT_HANDLER pfnEvent;
#define EVENT pfnEvent

static CMPlayer* me = 0;

extern "C" int vo_event(int code)
{
	switch (code) {
	case 0:
		me->state = STATE_IDLE;
		break;
	}
	return 0;
}

DWORD WINAPI PlayThread(CMPlayer *player)
{
	HANDLE hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);
	if (player->flags & MF_REDIRECT_CONSOLE) {
		if ((player->flags & MF_AUTO_READ_CONSOLE) && player->mpx.fdRead) {
			player->console = (char*)malloc(CONSOLE_BUF_SIZE);
			DWORD bytes;
			while (ReadFile(player->mpx.fdRead, player->console, CONSOLE_BUF_SIZE - 1, &bytes, 0)) {
				player->console[bytes] = 0;
				if (player->flags & MF_WRITE_CONSOLE) {
					WriteFile(hStdOut, player->console, bytes, &bytes, 0);
				}
				if (player->callback) (*player->callback)(player);
			}
			free(player->console);
			player->console = 0;
		} else {
			while (!ShellWait(&player->mpx, 100)) {
				DWORD bytes;
				if (PeekNamedPipe(player->mpx.fdRead, 0, 0, 0, &bytes, 0) && bytes > 0) {
					if (player->callback) (*player->callback)(player);
				}
			}
		}
	} else {
		while (!ShellWait(&player->mpx, 1000)) {
			if (player->callback) (*player->callback)(player);
		}
	}
	ShellClean(&player->mpx);
	player->state=STATE_IDLE;
	if (player->callback) (*player->callback)(player);
	return 0;
}

int CMPlayer::Command(char* cmd)
{
#ifdef WIN32
	DWORD ret;
	CString cmdstr(cmd);
	cmdstr += '\n';
	if (!WriteFile((HANDLE)mpx.fdWrite,cmdstr.GetBuffer(),cmdstr.GetLength(),&ret,NULL)) return -1;
#else
	if (write(mpx.fdWrite,cmd,strlen(cmd)) <= 0) return -1;
	write(mpx.fdWrite,"\n",1);
#endif
	return 0;
}

int CMPlayer::Read(char* buf, int bufsize)
{
	int bytes=0;
#ifdef WIN32
	int interval = 10;
	for (; (!PeekNamedPipe((HANDLE)mpx.fdRead,0,0,0,(LPDWORD)&bytes,0) || bytes == 0) && interval < 100; interval += 5) Sleep(interval);
	if (bytes == 0) return -1;
	ReadFile((HANDLE)mpx.fdRead, buf, bufsize - 1, (LPDWORD)&bytes, NULL);
#else
	bytes=read(mpx.fdRead,buf,bufsize-1);
#endif
	*(buf+bytes)=0;
	return bytes;
}

int CMPlayer::Stop()
{
	if (state==STATE_PLAYING) {
		Command("pause\n");
		Command("quit\n");
		ShellWait(&mpx, 1000);
		ShellTerminate(&mpx);
		hThread=NULL;
		state = STATE_IDLE;
		return 0;
	}
	return -1;
}

bool CMPlayer::onPosChange()
{
	return false;
}

void CMPlayer::RefreshVideo()
{
	Command("refresh_vo\n");
}

int CMPlayer::Play(char* filename)
{
	CString cmd;
	CString file;

	console = "";
	playingfile = filename;
	if (state == STATE_PLAYING) {
		cmd.Format("loadfile \"%s\"\n", filename);
		Command(cmd.GetBuffer());
		return 0;
	}
	memset(&mpx,0,sizeof(mpx));
	
	if (!strncmp(filename, "http://", 7) || !strncmp(filename, "ftp://", 6)) {
		char *p = pref.GetValueStr("overall.generic.proxy");
		if (*p) p = strstr(p, "://");
		if (p) {
			file.Format("http_proxy://%s/%s", p + 3, filename);
		}
	} else {
		file.Format("\"%s\"", filename);
	}
	cmd.Format("\"%smplayer.exe\" %s -slave -quiet",
		mppath.GetBuffer(), file.GetBuffer());

	if (!options.IsEmpty()) {
		cmd += options;
	}

	mpx.pchCommandLine=cmd.GetBuffer();
	mpx.flags=SHELL_REDIRECT_STDIN;
	if (flags & MF_REDIRECT_CONSOLE) {
		mpx.flags |= SHELL_REDIRECT_STDOUT;
	}
	EVENT(MC_MSG_CMD, mpx.pchCommandLine);
	if (ShellExec(&mpx)) {
		MessageBox(NULL,"MPlayer not found",NULL,MB_OK|MB_ICONINFORMATION);
		return -1;
	}
	SetPriorityClass(mpx.piProcInfo.hProcess, HIGH_PRIORITY_CLASS);
	if (mpx.fdRead) {
		DWORD bytes;
		while (PeekNamedPipe((HANDLE)mpx.fdRead, 0, 0, 0, &bytes, 0) && bytes == 0) Sleep(100);
	}
	state=STATE_PLAYING;
	DWORD id;
	hThread = CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)PlayThread,(void*)this,0,&id);
	return 0;
}
