/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#ifndef _PREFERENCE_H
#define _PREFERENCE_H

#include "paramlistdef.h"
#include "xmlaccessor.h"

#define FLAG_SAVE_ALL 0x1
#define FLAG_TYPE 0x2
#define FLAG_TEXT 0x4
#define FLAG_ENUM 0x8
#define FLAG_VALUE 0x10
#define FLAG_DESC 0x20
#define FLAG_RANGE 0x40

typedef struct {
	char* key;
	char* opt;
} PARAM_OPT;

class CPrefBase
{
public:
	CPrefBase(char* path = 0);
	void Set(char* path = 0);
	~CPrefBase();
private:
	PARAM_DATA* prevnode;
};

class CPrefHelper
{
public:
	CPrefHelper(char* rootname):basenode(0),cd(0) { rootNodeName = rootname; }

	void* GetValue(char* key);
	int GetValueInt(char* key);
	char* GetValueStr(char* key);
	bool GetValueBool(char* key);
	float GetValueFloat(char* key);
	char* GetValueEnum(char* key, int index = -1);
	char* GetValueEnum(PARAM_DATA* pd, int index = -1);

	void SetValue(char* key, int value);
	void SetValue(char* key, char* value);
	void SetValue(char* key, bool value);
	void SetValue(char* key, float value);
	void SetValue(char* key, void* value);
	void SetValue(char* key, CString& value);
	
	void SetValue(PARAM_DATA* pd, void* value);
	void SetValue(PARAM_DATA* pd, int value)  {
		pd->value = (void*)value;
	}
	void SetValue(PARAM_DATA* pd, char* value) {
		if (value)
			strncpy((char*)pd->value, value, pd->max);
		else
			*(char*)pd->value = 0;
	}
	void SetValue(PARAM_DATA* pd, bool value) {
		pd->value = value ? (void*)1 : 0;
	}
	void SetValue(PARAM_DATA* pd, float value) {
		*(float*)pd->value = value;
	}

	void Revert(PARAM_DATA* pd);
	void Revert(char* key);

	bool IsDefaultValue(char* key);
	bool IsDefaultValue(PARAM_DATA* pd);

	void SetDefaultValue(char* key, void* value, bool setval = false);
	void SetDefaultValue(PARAM_DATA* pd, void* value, bool setval = false);

	void LoadValue(char* file, char* key);

	void SetEnums(char* key, char* enums);

	void GetStringValue(char* key, CString& str);
	void GetStringValue(PARAM_DATA* pd, CString& str);
	int GetStringValue(PARAM_DATA* pd, char* buf, int bufsize);
	int GetStringValue(char* key, char* buf, int bufsize);
	void SetStringValue(char* key, char* value);
	void SetStringValue(PARAM_DATA* pd, char* value);
	void AdjustValue(PARAM_DATA* pd, char* value);
	PARAM_DATA* GetEntry(char* key = 0);
	void GenOptions(CString& cmd, PARAM_OPT paramopts[]);
	void GenOptions(CString& cmd, PARAM_OPT paramopts[], char* delimiter);
	void GenOptions(CString& cmd, PARAM_OPT paramopts[], int delimiter);

	PARAM_DATA* SetBase(char* path = 0);

	void Secharset(const char* tocode = 0, const char* fromcode = 0);
	char* Convercharset(char* str, int len = 0);

	int Init(char* xmlfile);
	void InitParams(PARAM_DATA* pd);
	void Destroy();
	void DestroyParams(PARAM_DATA* params);
	int Save(char* filename);
	int Load(char* filename, char* key = 0, char* rootpath = 0);

	int ReadXml(char* buf, char* basekey);
	int WriteXml(char** pbuf, char* basekey, int level, int flags = -1);
	void LoadParams(char* path);
	void SaveParams(PARAM_DATA* pd, int flags, int level = -1);

	PARAM_DATA* basenode;
private:
	PARAM_DATA root;
	char* rootNodeName;
	void* cd;
	CXML xml;
};

extern CPrefHelper pref;

#endif