/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#include <tchar.h>
#include "atlstr.h"
#include "paramlistdef.h"
#include "logger.h"
#include "mccore.h"
#include "mchelper.h"

#define INIFILE "mc.ini"
#define OUTPIPENAME "outpipe"

#define AUDIO_BLOCK_SIZE (16 * 1024)

typedef struct {
	char* data;
	int bytes;
	unsigned int totalbytes;
	int length;
	unsigned short int count;
} AUDIO_ENCODE_DATA;

typedef enum {
	OA_ASK = 0,
	OA_OVERWRITE,
	OA_BACKUP,
	OA_SKIP,
	OA_NOTEXIST,
} OV_ACTION;

class CInstInfo {
public:
	CString destfile;
	CString audiofile;
	CString apppath;
};

typedef struct {
	bool fAudioDecodable;
	bool fAudioEncoding;
	bool fUseWaveFile;
	bool fNeedMux;
	bool fSoleAudio;
	int commonPartLen;
	int audiochannel;
	char* filterbuf;
	char* audiodata;
	MC_AUDIO_FILTER *audiofilter;
	HWND hwndMain;
	int scrw;
	int scrh;
	int audioDelay;
	int audioFrameSize;
	int bitrate;
	OV_ACTION ovaction;
	HANDLE hEvent;
	HANDLE hPumpThread;
	int startpos;
	int duration;
	int fileNameSuffix;
	int startTimeOffset;
	int taskMode;
	int splitTime;
	int aliveCounter;
	char* errmsg;
} MC_INST_INTERNAL;

class CEncoder {
public:
	CEncoder():prefNodePath(0) {}
	/* encoder properties ********************************************************/
	virtual char* GetName() = 0;
	virtual char* GetFullName() { return GetName(); }
	virtual char* GetOutExt(T_ITEM* item) {return "out";}
	virtual char* GetInExts() {return 0;}
	virtual char* GetVersionInfo() { return 0; }
	virtual int UpdateSettings(T_ITEM* item) { return 0; }
	virtual int GetProgress() { return -1; }
	virtual int GetTargetBitRate(T_ITEM* item = 0) { return 0; }
	char* prefNodePath;
};

int EncodeAudioData(char* buf, int len, T_ITEM* item);
int ReadFileTags(T_ITEM* item);
int WriteFileTags(char* destfile, T_ITEM* item);
void PurgeTags(TAGS* tags);
void QualifyVariables(T_ITEM* item, CString& cmd, char *in);
void GetVariable(T_ITEM* item, char* varNameString, CString& str);
void InitRiffHeader(RIFF_HEADER *hdr);
void SetThreadPriorityBySetting(HANDLE hThread);
void SetProcessPriorityBySetting(HANDLE hProcess);
char* GetEncoderVersionInfo(char* path, char* options, char* delimiter);
void GenAudioFilterOptions(T_ITEM* item, CString& str, int srate);
void GenVideoFilterOptions(T_ITEM* item, CString& str);
void GenCommonOptions(T_ITEM* item, CString& str, int enc);
void InvalidiateRiffHeader(RIFF_HEADER* hdr, ATTR_AUDIO* audio);
void GetFPSString(int fps, int fpsScale, CString& str);

extern MC_STATE inst;
extern MC_INST_INTERNAL mcinst;
extern CInstInfo instinfo;
extern FILE_TYPE filetypes[];
extern PFN_EVENT_HANDLER pfnEvent;
extern char* prefFile;
