/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*************************************************************************/

#include <sys/stat.h>
#include "config.h"
#include "processpil.h"
#include "datatypes.h"
#include "transcode.h"
#include "audioencoders.h"
#include "videoencoders.h"
#include "preference.h"
#include "resource.h"

extern CAudioEncoder* AudioEncoders[];

int CMencoder::open()
{
	if (!fLoadDll) {
		sp = (SHELL_PARAM*)malloc(sizeof(SHELL_PARAM));
		memset(sp, 0, sizeof(SHELL_PARAM));
		if (!pref.GetValueBool("overall.ui.console")) {
			sp->flags = SHELL_REDIRECT_STDOUT;
		}
	} else {
		sp = 0;
		dll = new CDllHelper;
		if (!dll->LoadDLL(pref.GetValueStr("overall.mencoder.path"), ME_DLL)) return -1;
	}
	return 0;
}

int CMencoder::GetTargetBitRate(T_ITEM* item)
{
	if (!item) return -1;
	switch (item->target.videoformat) {
	case VC_NONE:
		return 0;
	case VC_COPY:
		return item->info.video.bitrate;
	default:
		if (pref.GetValueInt("overall.video.mode") == 1) {
			return -1;
		} else {
			return item->target.video.bitrate;
		}
	}
}

void CMencoder::GetAudioOptions(T_ITEM* item, CString& str)
{
	switch (item->target.audioformat) {
	case AC_LAMEMP3: {
		PARAM_OPT opts[] = {
			{".aq", "aq="},
			{".allfreq", "highpassfreq=-1:lowpassfreq=-1"},
			{0}
		};
		pref.SetBase("audioenc.lame");
		str += " -oac mp3lame -lameopts ";
		switch (pref.GetValueInt(".mode")) {
		case 0:	//VBR
			str.AppendFormat("vbr=2:q=%d", 9 - pref.GetValueInt(".quality"));
			break;
		case 1: //ABR
			str.AppendFormat("abr:br=%d",pref.GetValueInt(".bitrate"));
			break;
		default: //CBR
			str.AppendFormat("cbr:br=%d",pref.GetValueInt(".bitrate"));
			break;
		}
		pref.GenOptions(str, opts, ':');
		if (int i = pref.GetValueInt(".stereo")) {
			str.AppendFormat(":mode=%d", i-1);
		}
		} break;
	case AC_FAAC: {
		PARAM_OPT opts[] = {
			{".version", "mpeg=2|mpeg=4"},
			{".tns", "tns"},
			{0},
		};
		pref.SetBase("audioenc.faac");
		str.AppendFormat(" -oac faac -faacopts %s=%d",
			pref.GetValueInt(".mode")?"br":"quality",
			pref.GetValueInt(pref.GetValueInt(".mode") ? ".bitrate" : ".quality"));
		pref.GenOptions(str, opts, ':');
		} break;
	case AC_PCM:
		str += " -oac pcm";
		break;
	case AC_FFMPEG: {
		static const char *acodecname[]={"mp2","mp3","ac3","amr_nb","amr_wb","adpcm_ima_wav","sonic"};
		str.AppendFormat(" -oac lavc -lavcopts acodec=%s:abitrate=%d",
			acodecname[pref.GetValueInt("audioenc.ffmpeg.codec")],
			pref.GetValueInt("audioenc.ffmpeg.bitrate"));
		} break;
	case AC_COPY:
		str += " -oac copy";
		break;
	default:
		str += " -nosound";
		return;
	}
	pref.SetBase();
	if (item->target.container == CF_DEFAULT || item->target.container == CF_AVI) {
		float audiodelay = item->info.audiodelay ? (float)item->info.audiodelay / 1000 : (float)pref.GetValueInt("overall.audio.delay");
		if (audiodelay) str.AppendFormat(" -audio-delay %f" , audiodelay);
	}
	GenAudioFilterOptions(item, str, 0);
}

char* CMencoder::GetVersionInfo()
{
	CString path(pref.GetValueStr("overall.mencoder.path"));
	path += "mencoder.exe";
	return GetEncoderVersionInfo(path.GetBuffer(), 0, "(C)");
}

int CMencoder::Encode(CString& cmd)
{
	if (!fLoadDll) {
		sp->pchCommandLine = cmd.GetBuffer();
		pfnEvent(MC_MSG_CMD, sp->pchCommandLine);
		if (ShellExec(sp)) return -1;
		SetProcessPriorityBySetting(sp->piProcInfo.hProcess);
	} else {
		if (dll->StartMain(cmd.GetBuffer())) return -1;
		SetThreadPriorityBySetting(dll->hThread);
	}
	return 0;
}

int CMencoder::close()
{
	if (!fLoadDll) {
		ShellTerminate(sp);
		ShellClean(sp);
		free(sp);
		return 0;
	} else {
		dll->Stop();
		dll->Unload();
		delete dll;
		dll = 0;
	}
	return 0;
}

void CMencoder::ParseConsole(char* consoleText, int consoleBytes)
{
	char *ql = 0, *qr = 0, *p = consoleText + consoleBytes;
	char *s = 0;

	while (p>consoleText && *p!='\n') {
		if (*p=='(')
			ql = p;
		else if (*p==')')
			qr = p;
		else if (*(DWORD*)p == ':soP') {
			s = p;
			*(s - 1) = 0;
		}
		p--;
	}
	while (--p>=consoleText) {
		if (*p=='(')
			ql = p;
		else if (*p==')')
			qr = p;
		else if (*(DWORD*)p == ':soP') {
			s = p;
			break;
		}
	}
	if (!qr || !ql || !s) return;

	pfnEvent(MC_WRITE_CONSOLE, s);

	{
		float f = (float)atof(ql+1);
		if (f) inst.vstats.percent = f;
	}

	do {
		p = strtok(s,":");
		if (!(p = strtok(0," "))) continue;
		inst.vstats.pos = atoi(p);
		if (!(p = strtok(0," "))) continue;
		inst.vstats.frame = atoi(p);
		if (!(p = strtok(qr + 2," "))) continue;
		inst.vstats.fps = (float)atof(p);
		p=strtok(0,"  ");
		if (!(p = strtok(0,"  "))) continue;
		inst.vstats.remaintime = atoi(p);
		if (!(p = strtok(0,"  "))) continue;
		inst.vstats.estsize = atoi(p) << 10;
		p=strtok(0,":");
		p=strtok(0," ");
		if (!(p = strtok(0,":"))) continue;
		p++;
		inst.vstats.vbitrate = atoi(p);
		if (!(p = strtok(0,"]"))) continue;
		if (!mcinst.fNeedMux) {
			inst.vstats.abitrate = atoi(p);
		} else if (inst.vstats.pos > 0) {
			struct stat info;
			if (!stat(instinfo.audiofile.GetBuffer(), &info)) {
				inst.vstats.abitrate = info.st_size / inst.vstats.pos / (1000 / 8);
				if (inst.vstats.percent >= 0.01) 
					inst.vstats.estsize += (int)((float)(info.st_size / 10) / inst.vstats.percent) / 1024;
			}
		}
	} while (0);
}

int CMencoder::GetProgress()
{
	if (!fLoadDll) {
		if (sp->flags & SHELL_REDIRECT_STDOUT) {
			char buf[1024];
			int bytes;
			int bytessum = 0;
			while ( inst.state == STATE_ENCODING && bytessum < sizeof(buf) - 1) {
				if (!ReadFile(sp->fdRead, buf + bytessum, sizeof(buf)- 1 - bytessum, (LPDWORD)&bytes,0)) {
					return -1;
				}
                bytessum += bytes;
			}
			buf[bytessum] = 0;
			ParseConsole(buf, bytessum);
			return (int)inst.vstats.percent;
		} else {
			return (ShellWait(sp, 500)) ? -1 : 0;
		}
	} else {
		if (dll->Wait(500) != WAIT_OBJECT_0) {
			return -1;
		}
		return (int)inst.stats.percent;
	}
}

static const char *lavcvcodecs[]={"mpeg1video","mpeg2video","mpeg4","msmpeg4v2","h261","h263","h263p","snow","msmpeg4","wmv1","wmv2","flv","dvvideo","mjpeg","ljpeg"};

int CMencoder::UpdateSettings(T_ITEM* item)
{
	int i;
	CString mecmd;
	char* p;
	CPrefBase prefbase("overall.mencoder");
	for (i = 0; i < MAX_PASS_COUNT; i++) cmds[i] = "";

	fLoadDll = pref.GetValueBool(".dll");
	mecmd.Format("\"%s%s\" \"$(SourceFile)\"",
		pref.GetValueStr(".path"),
		fLoadDll ? ME_DLL : ME_EXE);
	if (fLoadDll) {
		mecmd += " -really-quiet";
	}

	GenCommonOptions(item, mecmd, 1);
	GenAudioFilterOptions(item, mecmd, 0);
	GenVideoFilterOptions(item, mecmd);

	if (strcmp(GetOutExt(item), "avi")) {
		mecmd += " -of rawvideo";
	}
	if (item->target.video.fps && item->target.video.fpsScale) {
		mecmd += " -ofps ";
		GetFPSString(item->target.video.fps, item->target.video.fpsScale, mecmd);
	}

	if (*(p = pref.GetValueStr(".options"))) {
		mecmd.AppendFormat(" %s", p);
	}

	if (pref.GetValueInt("overall.video.mode") > 1) {
		mecmd.AppendFormat(" -passlogfile \"$(TempDir)mcpass%d.log\"",
			GetCurrentProcessId());
	}

	switch (item->target.videoformat) {
	case VC_XVID: {
		PARAM_OPT opts[] = {
			{".interlaced", "interlacing"},
			{".qpel", "qpel="},
			{".gmc", "gmc"},
			{".trellis", "trellis"},
			{".chroma_me", "chroma_me"},
			{".chroma_opt", "chroma_opt"},
			{".hqac", "hq_ac"},
			{".packed", "packed"},
			{".greyscale", "greyscale"},
			{".cartoon", "cartoon"},
			{".quantType", "quant_type=h263|quant_type=mpeg"},
			{"overall.video.aspect", "aspect="},
			{".options", ""},
			{0},
		};
		PARAM_OPT fullopts[] = {
			{".bframe", "max_bframes="},
			{".meQuality", "me_quality="},
			{".vhq", "vhq="},
			{0},
		};
		const char* profiles[] = {"sp0", "sp1", "sp2", "sp3", "asp0", "asp1", "asp2", "asp3", "asp4", "asp5", "dxnhandheld", "dxnportntsc", "dxnportpal", "dxnhtntsc", "dxnhtpal", "dxnhdtv"};
		CString fullargs;
		prefbase.Set("videoenc.xvid");
		int profile = pref.GetValueInt(".profile");
		if (pref.GetValueBool(".dx50")) mecmd += " -ffourcc DX50";
		mecmd.AppendFormat(" -ovc xvid -xvidencopts threads=%d", pref.GetValueInt(".threads"));
		if (profile) {
			mecmd.AppendFormat(":profile=%s", profiles[profile - 1]);
		}
		if (!profile || !pref.GetValueBool(".strict")) {
			pref.GenOptions(mecmd, opts, ':');
			pref.GenOptions(fullargs, fullopts, ':');
		}

		switch (pref.GetValueInt("overall.video.mode")) {
		case 0:	// fixed bitrate
			cmds[0] = mecmd + fullargs + ":bitrate=$(VideoBitrate)";
			break;
		case 1:	// fixed quant
			cmds[0] += mecmd + fullargs;
			cmds[0].AppendFormat(":fixed_quant=%d", 31 * (101 - pref.GetValueInt("overall.video.quality")) / 100);
			break;
		case 2:	// 2-pass
			cmds[0] = mecmd;
			cmds[0].AppendFormat(":turbo=%d:pass=1", pref.GetValueInt(".turbo"));
			cmds[1] = mecmd + fullargs + ":pass=2:bitrate=$(VideoBitrate)";
			break;
		}
		} break;
	case VC_H264: {
		prefbase.Set("videoenc.x264");
		PARAM_OPT opts[] = {
			{".level", "level_idc="},
			{".moreDetails", "dct_decimate|nodct_decimate"},
			{".bframes", "bframes="},
			{".keyint", "keyint="},
			{".keyint_min", "keyint_min="},
			{".frameref", "frameref="},
			{".qpmin", "qp_min="},
			{".qpmax", "qp_max="},
			{".qpstep", "qp_step="},
			{".qpconstant", "qp_constant="},
			{".cabac", "nocabac|cabac"},
			{".vbvMaxRate", "vbv_maxrate="},
			{".vbvBufSize", "vbv_bufsize="},
			{".directPredict", "direct_pred="},
			{".weightedPredict", "noweight_b|weight_b"},
			{".i4x4", "noi4x4|i4x4"},
			{".i8x8", "noi8x8|i8x8"},
			{".b8x8mv", "nob8x8mv|b8x8mv"},
			{".p8x8mv", "no8x8mv|8x8mv"},
			{".p4x4mv", "no4x4mv|4x4mv"},
			{".p8x8dct", "no8x8dct|8x8dct"},
			{".mixedReference", "nomixed_refs|mixed_refs"},
			{".brdo", "nobrdo|brdo"},
			{".trellis", "trellis="},
			{".bime", "nobime|bime"},
			{".fastPSskip", "nofast_pskip|fast_pskip"},
			{".threads", "threads="},
			{".options", ""},
			{0},
		};
		PARAM_OPT fullopts[] = {
			{".me", "me="},
			{".meRange", "me_range="},
			{".subq", "subq="},
			{0},
		};
		mecmd.AppendFormat(" -ovc x264 -x264encopts qcomp=%3.2f", (float)pref.GetValueInt(".qcomp") / 100);
		pref.GenOptions(mecmd, opts, ':');
		switch (pref.GetValueInt("overall.video.mode")) {
		case 0:	// fixed bitrate
			cmds[0] = mecmd + ":bitrate=$(VideoBitrate)";
			pref.GenOptions(cmds[0], fullopts, ':');
			break;
		case 1:
			cmds[0] = mecmd;
			pref.GenOptions(cmds[0], fullopts, ':');
			cmds[0].AppendFormat(":crf=%d", (100 - pref.GetValueInt("overall.video.quality")) / 2);
			break;
		case 2:	// 2-pass
			cmds[0] = mecmd;
			cmds[0].AppendFormat(":pass=1:turbo=%d", pref.GetValueStr(".turbo"));
			cmds[1] = mecmd + ":bitrate=$(VideoBitrate):pass=2";
			pref.GenOptions(cmds[1], fullopts, ':');
			break;
		case 3:
			cmds[0] = mecmd + ":pass=1:turbo=2";
			pref.GenOptions(mecmd, fullopts, ':');
			cmds[1] = mecmd;
			cmds[1].AppendFormat(":pass=2:turbo=%d:bitrate=$(VideoBitrate)", pref.GetValueInt(".turbo"));
			cmds[2] = mecmd + ":pass=3:bitrate=$(VideoBitrate)";
			break;
		}
		} break;
	case VC_MPEG1:
	case VC_MPEG2:
	case VC_MPEG4:
	case VC_H261:
	case VC_H263:
	case VC_H263P:
	case VC_WMV7:
	case VC_WMV8:
	case VC_DIVX3:
	case VC_MSMPEG4V2:
	case VC_SNOW:
	case VC_FLV:
	case VC_DV:
	case VC_MJPEG:
	case VC_LJPEG:
	{
		switch (pref.GetValueInt("overall.video.container")) {
		case CF_MPEG: {
			PARAM_OPT opts[] = {
				{"container.mpeg.format", "mpeg1|mpeg2|xvcd|xsvcd|dvd"},
				{"overall.video.aspect", "aspect="},
				{0}
			};
			mecmd += " -of mpeg -mpegopts format";
			pref.GenOptions(mecmd, opts, '=');
			} break;
		case CF_LAVF: {
			PARAM_OPT opts[] = {
				{"container.lavf.format", "mpg|asf|avi|mp4|rm|mov|flv|dv"},
				{0}
			};
			mecmd += " -of lavf -lavfopts i_certify_that_my_video_stream_does_not_use_b_frames:format";
			pref.GenOptions(mecmd, opts, '=');
			} break;
		}

		prefbase.Set("videoenc.ffmpeg");
		PARAM_OPT opts[] = {
			{".mbd", "mbd="},
			{".obmc", "obmc"},
			{".qpel", "qpel"},
			{".subq", "subq="},
			{".trellis", "trell"},
			{".keyint", "keyint="},
			{".v4mv", "v4mv"},
			{".vrcMinRate", "vrc_minrate="},
			{".vrcMaxRate", "vrc_maxrate="},
			{".vrcBufSize", "vrc_buf_size="},
			{".nr", "nr="},
			{".qns", "qns="},
			{".hq", "precmp=2:subcmp=2:cmp=2:dia=-10:predia=-10:cbp:mv0:vqmin=1:lmin=1:dc=10"},
			{"overall.video.aspect", "aspect="},
			{".options", ""},
			{0},
		};
		mecmd.AppendFormat(" -ovc lavc -lavcopts vcodec=%s", lavcvcodecs[item->target.videoformat - VC_MPEG1]);
		pref.GenOptions(mecmd, opts, ':');
		switch (pref.GetValueInt("overall.video.mode")) {
		case 0:
			cmds[0] = mecmd + ":vbitrate=$(VideoBitrate)";
			break;
		case 2:
			cmds[0] = mecmd + ":vpass=1:turbo";
			cmds[1] = mecmd + ":vbitrate=$(VideoBitrate):vpass=2";
			break;
		case 3:
			cmds[0] = mecmd + ":vpass=1:turbo";
			cmds[1] = mecmd + ":vbitrate=$(VideoBitrate):vpass=2";
			cmds[2] = mecmd + ":vbitrate=$(VideoBitrate):vpass=3";
			break;
		}
		} break;
	case VC_COPY:
		cmds[0] = mecmd += " -ovc copy";
		break;
	default:
		cmds[0] = mecmd += " -ovc null";
	}

	passcount = 0;
	for (i = 0; i < MAX_PASS_COUNT && cmds[i] != ""; i++);
	if (!i) return 0;
	passcount = i;;
	for (i = 0; i < passcount - 1; i++) {
		cmds[i] += " -nosound -o NUL";
	}
	switch (pref.GetValueInt("overall.video.container")) {
	case CF_AVI:
	case CF_DEFAULT:
	case CF_LAVF:
	case CF_MPEG:
		GetAudioOptions(item, cmds[i]);
		break;
	default:
		cmds[i] += " -nosound";
	}
	cmds[i] += " -o \"$(DestFile)\"";
	return passcount;
}

bool CMencoder::CheckAudioSupport(int id)
{
	switch (id) {
	case AC_LAMEMP3:
	case AC_FAAC:
	case AC_PCM:
	case AC_FFMPEG:
	case AC_COPY:
	case AC_NONE:
		return true;
	default:
		return false;
	}
}

bool CMencoder::CheckContainerSupport(int id)
{
	switch (id) {
	case CF_AVI:
	case CF_MPEG:
	case CF_LAVF:
		return true;
	default:
		return false;
	}
}


char* CMencoder::GetOutExt(T_ITEM* item)
{
	switch (item->target.container) {
	case CF_RAW:
	case CF_MP4:
		switch (item->target.videoformat) {
		case VC_XVID:
			return "m4v";
		case VC_H263:
			return "263";
		case VC_H264:
			return "264";
		}
		break;
	case CF_PMP:
		switch (item->target.videoformat) {
		case VC_H264:
			return "264";
		}
		break;
	}
	return "avi";
}

int Cffmpeg::UpdateSettings(T_ITEM* item)
{
	CPrefBase prefbase("videoencoder.ffmpeg");
	PARAM_OPT opts[] = {
		{"audioenc.ffmpeg.bitrate", "-ab "},
		{"audioenc.ffmpeg.samplerate", "-ar "},
		{0}
	};

	cmds[0].Format("\"%s\" -i \"$(SourceFile)\"", pref.GetValueStr(".path"));
	cmds[0].AppendFormat(" -b %d", pref.GetValueInt("overall.video.bitrate"));
	if (item->target.videoformat >= VC_MPEG1)
		cmds[0].AppendFormat(" -vcodec %s", lavcvcodecs[item->target.videoformat - VC_MPEG1]);
	if (item->target.video.fps && item->target.video.fpsScale) {
		cmds[0] += " -r";
		GetFPSString(item->target.video.fps, item->target.video.fpsScale, cmds[0]);
	}
	if (pref.GetValueBool("videofilters.scale.enabled")) {
		int i = pref.GetValueInt("videofilters.scale.preset");
		if (!i) {
			cmds[0].AppendFormat(" -s %dx%d", pref.GetValueInt("videofilters.scale.wdith"), pref.GetValueInt("videofilters.scale.height"));
		} else {
			char *v = strdup(pref.GetValueEnum("videofilters.scale.preset"));
			cmds[0].AppendFormat(" -s %s", strtok(v, " "));
			free(v);
		}
	}
	if (item->target.audioformat == AC_FFMPEG) {
		cmds[0].AppendFormat(" -acodec %s", 
			((CFFMpegAudio*)AudioEncoders[AC_FFMPEG])->GetCodecName()
		);
		pref.GenOptions(cmds[0], opts, ' ');
	}
	cmds[0] += " \"$(DestFile)\"";
	passcount = 1;
	return 1;
}

bool Cffmpeg::CheckAudioSupport(int id)
{
	switch (id) {
	case AC_FFMPEG:
	case AC_NONE:
		return true;
	default:
		pfnEvent(MC_MSG_ALERT, "To use FFMPEG as video encoder with the default container, please choose FFMPEG as audio encoder");
		return false;
	}
}

bool Cffmpeg::CheckContainerSupport(int id)
{
	switch (id) {
	case CF_AVI:
	case CF_LAVF:
		return true;
	default:
		return false;
	}
}

int Cffmpeg::GetProgress()
{
	return ShellWait(&sp, 1000) ? -1 : 0;
}

int Cffmpeg::open() { 
	memset(&sp, 0, sizeof(sp));
	return 0;
}
int Cffmpeg::Encode(CString& cmd)
{
	sp.pchCommandLine = cmd.GetBuffer();
	if (ShellExec(&sp)) return -1;
	ShellSetPriority(&sp,pref.GetValueInt("overall.generic.priority"));
	return 0;
}
int Cffmpeg::close() { 
	ShellTerminate(&sp);
	ShellClean(&sp);
	return 0;
}
