/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*************************************************************************/

#include "windows.h"
#include "atlstr.h"
#include "transcode.h"
#include <stdio.h>
#include "httpd.h"
#include "preference.h"
#include "xmlaccessor.h"
#include "winamphelper.h"
#include "httpget.h"
#include "revision.h"

extern "C" INPLUGIN_INFO inplugins;
extern "C" DSPPLUGIN_INFO* dspplugins;

//////////////////////////////////////////////////////////////////////////
// callback from the web server whenever a valid request comes in
//////////////////////////////////////////////////////////////////////////
static int uhStats(UrlHandlerParam* param)
{
	static const char *pageHead="<html><body class='body'><table border=1 cellpadding=0 cellspacing=0 width=280 class='body'>";
	static const char *pageCellBegin="<tr><td width=140>%s</td><td width=140>";
	static const char *pageCellEnd="</td></tr>";
	static const char *pageTail="</body></html>";
	char *p;
	char buf[30];
	HttpStats *stats=&(((HttpParam*)param->hp)->stats);
	HttpRequest *req=&param->hs->request;

	mwGetHttpDateTime(time(NULL),buf);

	p = (char*)param->pucBuffer;
	//generate page
	p+=sprintf(p,pageHead);

	p+=sprintf(p,pageCellBegin,"Your IP address:");
	p+=sprintf(p,"%d.%d.%d.%d%s",
		req->ipAddr.caddr[3],
		req->ipAddr.caddr[2],
		req->ipAddr.caddr[1],
		req->ipAddr.caddr[0],
		pageCellEnd);

	p+=sprintf(p,pageCellBegin,"Current time:");
	p+=sprintf(p,"%s%s",buf,pageCellEnd);

	p+=sprintf(p,pageCellBegin,"Server uptime:",time(NULL)-stats->startTime);
	p+=sprintf(p,"%d sec(s)%s",time(NULL)-stats->startTime,pageCellEnd);

	p+=sprintf(p,pageCellBegin,"Connected clients:");
	p+=sprintf(p,"%d%s",stats->clientCount,pageCellEnd);

	p+=sprintf(p,pageCellBegin,"Maximum clients:");
	p+=sprintf(p,"%d%s",stats->clientCountMax,pageCellEnd);

	p+=sprintf(p,pageCellBegin,"Requests:");
	p+=sprintf(p,"%d%s",stats->reqCount,pageCellEnd);

	p+=sprintf(p,pageCellBegin,"Files sent:");
	p+=sprintf(p,"%d%s",stats->fileSentCount,pageCellEnd);

	p+=sprintf(p,pageCellBegin,"Bytes sent:");
	p+=sprintf(p,"%d bytes%s</table>",stats->fileSentBytes,pageCellEnd);

	p+=sprintf(p,pageTail);

	//return data to server
	param->iDataBytes = (int)(p - (char*)(param->pucBuffer));
	param->fileType=HTTPFILETYPE_HTML;
	return FLAG_DATA_RAW;
}

static int uhPrefDefault(UrlHandlerParam* param)
{
	param->hs->fd = open(prefFile, O_RDONLY|0x8000);
	param->fileType = HTTPFILETYPE_XML;
	return FLAG_DATA_FD;
}

static int uhPrefXml(UrlHandlerParam* param)
{
	char *xmltext;
	char *path = 0;
	int flags = FLAG_SAVE_ALL;
	int level = -1;
	if (mwParseQueryString(param) > 0) {
		path = mwGetVarValue(param->pxVars, "path");
		if (mwGetVarValue(param->pxVars, "modified"))
			flags &= ~FLAG_SAVE_ALL;
		if (mwGetVarValue(param->pxVars, "type"))
			flags |= FLAG_TYPE;
		if (mwGetVarValue(param->pxVars, "text"))
			flags |= FLAG_TEXT;
		if (mwGetVarValue(param->pxVars, "enum"))
			flags |= FLAG_ENUM;
		if (mwGetVarValue(param->pxVars, "desc"))
			flags |= FLAG_DESC;
		if (mwGetVarValue(param->pxVars, "value"))
			flags |= FLAG_VALUE;
		if (mwGetVarValue(param->pxVars, "range"))
			flags |= FLAG_RANGE;
		level = mwGetVarValueInt(param->pxVars, "level", -1);
	}
    param->iDataBytes = pref.WriteXml(&xmltext, path, level, flags);
	if (!param->iDataBytes) {
		param->iDataBytes = sprintf(param->pucBuffer, "Invalid preference path - %s", path);
		param->fileType = HTTPFILETYPE_TEXT;
		return FLAG_DATA_RAW;
	}
	param->pucBuffer = xmltext;
	param->fileType = HTTPFILETYPE_XML;
	return FLAG_DATA_RAW | FLAG_TO_FREE;
}

static int uhPrefGet(UrlHandlerParam* param)
{
	param->fileType = HTTPFILETYPE_TEXT;
	if (mwParseQueryString(param) <= 0) {
		param->iDataBytes = sprintf(param->pucBuffer, "Invalid request");
		return FLAG_DATA_RAW;
	}
	char *path = mwGetVarValue(param->pxVars, "path");
	if (path) {
		param->iDataBytes = pref.GetStringValue(path, param->pucBuffer, param->iDataBytes);
	} else {
		param->iDataBytes = sprintf("Invalid path - %s", path);
	}
	return FLAG_DATA_RAW;
}

static int uhPrefSet(UrlHandlerParam* param)
{
	char *p, *redirect = 0;
	int count = 0;

	if (param->hs->request.pucPayload) {
		if (pref.ReadXml((char*)param->hs->request.pucPayload, "pref.xml")) {
			param->iDataBytes = sprintf(param->pucBuffer, "Error parsing preference XML");
			param->fileType = HTTPFILETYPE_TEXT;
			return FLAG_DATA_RAW;
		}
	} else if (mwParseQueryString(param) <= 0) {
		param->iDataBytes = sprintf(param->pucBuffer, "Invalid request");
		param->fileType = HTTPFILETYPE_TEXT;
		return FLAG_DATA_RAW;
	}

	for (int i = 0;	(p = param->pxVars[i].name); i++) {
		if (!strcmp(p, "redirect") && !redirect) {
			redirect = param->pxVars[i].value;
		} else {
			pref.SetStringValue(param->pxVars[i].name, param->pxVars[i].value);
			count++;
		}
	}
	
	pfnEvent(MC_UPDATE_UI, 0);

	if (redirect) {
		strcpy(param->pucBuffer, redirect);
		return FLAG_DATA_REDIRECT;
	} else {
		param->iDataBytes = sprintf(param->pucBuffer, "%d parameter(s) modified", count);
		param->fileType = HTTPFILETYPE_TEXT;
		return FLAG_DATA_RAW;
	}
}

static int uhPrefRevert(UrlHandlerParam* param)
{
	param->fileType = HTTPFILETYPE_TEXT;
	if (mwParseQueryString(param) <= 0) {
		param->iDataBytes = sprintf(param->pucBuffer, "Invalid request");
		return FLAG_DATA_RAW;
	}
	char *path = mwGetVarValue(param->pxVars, "path");
	if (path) {
		pref.Revert(path);
		param->iDataBytes = pref.GetStringValue(path, param->pucBuffer, param->iDataBytes);
	} else {
		param->iDataBytes = sprintf("Invalid path - %s", path);
	}
	return FLAG_DATA_RAW;
}

static int uhPluginXml(UrlHandlerParam* param)
{
	char* type;
	char* config;
	char* id;
	int enabled = -1;

	CXML xml;
	xml.New("1.0", "MediaCoderPlugins");

	if (mwParseQueryString(param) > 0) {
		char *p;
		type = mwGetVarValue(param->pxVars, "type");
		config = mwGetVarValue(param->pxVars, "config");
		id = mwGetVarValue(param->pxVars, "id");
		p = mwGetVarValue(param->pxVars, "enabled");
		if (p) enabled = (*p != '1' && stricmp(p, "true")) ? 0 : 1;
	} 
	
	if (!type || !strcmp(type, "input")) {
		xml.addChild("input");
		for (INPLUGIN_INFO* cur = inplugins.next; cur; cur = cur->next) {
			if (id) {
				if (!strcmp(id, cur->pluginfile)) {
					if (enabled == 1) {
						CLRBIT(cur->flags, FLAG_DISABLED);
					} else if (enabled == 0) {
						SETBIT(cur->flags, FLAG_DISABLED);
					}
					if (config) {
						CString path;
						path.Format("%s%s", pref.GetValueStr("audiosrc.winamp.path"), cur->pluginfile);
						In_Module* inmod = LoadWinampInPlugin(path.GetBuffer(), 0);
						if (inmod) {
							((In_Module*)inmod)->Config(0);
							UnloadWinampInPlugin(inmod);
						}
					}
				} else {
					continue;
				}
			}
			xml.addChild("plugin");
			xml.addAttribute("id", cur->pluginfile);
			xml.addAttribute("desc", cur->desc);
			xml.addAttribute("enabled", ISBITSET(cur->flags, FLAG_DISABLED) ? "0" : "1");
			xml.goParent();
		}
		xml.goParent();
	}
	if (!type || !strcmp(type, "dsp")) {
		xml.addChild("dsp");
		for (int i = 0; dspplugins[i].hDllPlugin; i++, i++) {
			if (id) {
				if (!strcmp(id, dspplugins[i].pluginfile)) {
					if (enabled == 1) {
						CLRBIT(dspplugins[i].flags, FLAG_DISABLED);
					} else if (enabled == 0) {
						SETBIT(dspplugins[i].flags, FLAG_DISABLED);
					}
				}
				if (config) {
					if (!LoadDspPlugin(&dspplugins[i])) {
						dspplugins[i].mod->Config(dspplugins[i].mod);
						UnloadDspPlugin(&dspplugins[i]);
					}
				}
			}

			xml.addChild("plugin");
			xml.addAttribute("id", dspplugins[i].pluginfile);
			xml.addAttribute("desc", dspplugins[i].hdr->description);
			xml.addAttribute("enabled", ISBITSET(dspplugins[i].flags, FLAG_DISABLED) ? "0" : "1");
			xml.goParent();
		}
	}
	param->iDataBytes = xml.Dump(&param->pucBuffer, "UTF-8");
	param->fileType = HTTPFILETYPE_XML;
	return FLAG_DATA_RAW | FLAG_TO_FREE;
}

static int uhOpenUrl(UrlHandlerParam* param)
{
	int len = 0;
	if (mwParseQueryString(param) > 0) {
		char* p = mwGetVarValue(param->pxVars, "url");
		if (p) {
			len = _snprintf(param->pucBuffer, param->iDataBytes, "http://%s", p);
			ShellExecute(0, "open", param->pucBuffer, 0, 0, SW_SHOW);
		}
	}
	param->iDataBytes = len;
	param->fileType = HTTPFILETYPE_TEXT;
	return FLAG_DATA_RAW;
}

/*
static const char* adshtml = "<script type='text/javascript'><!--\n"
"google_ad_client = 'pub-1619364995581263';\n"
"google_ad_width = 234;\n"
"google_ad_height = 60;\n"
"google_ad_format = '234x60_as';\n"
"google_ad_type = 'text';\n"
"google_ad_channel ='';\n"
"google_color_border = 'FFFFFF';\n"
"google_color_bg = 'FFFFFF';\n"
"google_color_link = '0000FF';\n"
"google_color_text = '000000';\n"
"google_color_url = '008000';\n"
"//--></script><script type='text/javascript' src='http://pagead2.googlesyndication.com/pagead/show_ads.js'></script>";
*/
static const char* paypalhtml = "<html><head><script language='javascript'>\n"
"function doDonate() {"
"	var xmlhttp = new XMLHttpRequest;\n"
"	xmlhttp.open('GET', '/openurl?url=mediacoder.sourceforge.net/donate.htm', false);\n"
"	xmlhttp.send(null);\n"
"}</script></head><body><p align='right'><br><img style='cursor:pointer' src='/img/donate.png' onClick='doDonate()'></p></body></html>";

static int uhAdsFrame(UrlHandlerParam* param)
{
	/*
	bool ad = pref.GetValueBool("overall.ui.ads");
	param->iDataBytes = sprintf(param->pucBuffer,
		"<html><head>%s</head><body leftmargin='0' topmargin='0' rightmargin='0' bottommargin='0'>%s</body></html>",
		ad ? "<meta content='convert, transcode, MP3, Ogg Vorbis AAC, AAC+/Parametric Stereo, AMR, MusePack, WMA, RealAudio, FLAC, WavPack, Monkey's Audio, OptimFrog, AAC Lossless, WMA Lossless, WAV, H.264, Xvid, DivX, MPEG 1/2/4, H.263, 3ivx, RealVideo, Windows Media Video, DV, AVI, MPEG/VOB, Matroska, MP4, RealMedia, ASF/WMV, Quicktime, OGM, CD, VCD, DVD, CUE Sheet' name='keywords'>" : "",
		ad ? adshtml : paypalhtml);
	*/
	param->iDataBytes = sprintf(param->pucBuffer, "%s", paypalhtml);
	param->fileType = HTTPFILETYPE_HTML;
	return FLAG_DATA_RAW;
}

HANDLE hEventSetupComplete = 0;


static int uhSetupSave(UrlHandlerParam* param)
{
	if (mwParseQueryString(param) > 0) {
		char *p;
		if (p = mwGetVarValue(param->pxVars, "ackbuild")) {
			pref.SetValue("overall.generic.ignoredBuild", atoi(p));
			strcpy(param->pucBuffer, "http://mediacoder.sourceforge.net/noprompt.htm");
			return FLAG_DATA_REDIRECT;
		}
		if (p = mwGetVarValue(param->pxVars, "unload")) {
			if (*p == '0' || !mcGetPrefValueInt("overall.ui.showsetup")) {
				SetEvent(hEventSetupComplete);
			}
			strcpy(param->pucBuffer, "OK");
			param->iDataBytes = 2;
			param->fileType = HTTPFILETYPE_TEXT;
			return FLAG_DATA_RAW;
		}
		if (!(p = mwGetVarValue(param->pxVars, "nextstep"))) {
			strcpy(param->pucBuffer, "<html><body><script language='javascript'>window.close();</script></body></html>");
			param->iDataBytes = strlen(param->pucBuffer);
			param->fileType = HTTPFILETYPE_HTML;
			return FLAG_DATA_RAW;
		}
		int stepid = atoi(p);
		sprintf(param->pucBuffer, "setup%d.htm", stepid);
		if (p = mwGetVarValue(param->pxVars, "agree")) {
			if (*p == '1') {
				mcSetPrefValueInt("overall.ui.showsetup", mcGetBuildNo());
			} else {
				mcSetPrefValueInt("overall.ui.showsetup", 0);
				sprintf(param->pucBuffer, "deny.htm", atoi(p));
				return FLAG_DATA_REDIRECT;
			}
		}
		if (p = mwGetVarValue(param->pxVars, "uimode")) {
			mcSetPrefValueInt("overall.ui.mode", atoi(p));
		}
		if (p = mwGetVarValue(param->pxVars, "checkupdate")) {
			mcSetPrefValueInt("overall.generic.checkUpdate", atoi(p));
		}
		if (p = mwGetVarValue(param->pxVars, "extlibs")) {
			mcSetPrefValueInt("overall.decoding.mediainfo", atoi(p));
		}
		return FLAG_DATA_REDIRECT;
	} else {
		return 0;
	}
}

static int uhGetNetVideoUrl(UrlHandlerParam* param)
{
	char* pageurl;
	if (mwParseQueryString(param) <= 0) return 0;
	pageurl = mwGetVarValue(param->pxVars, "url");
	char *buf = param->pucBuffer;
	char *p, *s;
	httpRequest req;
	sprintf(buf, "http://videodownloader.net/get/?url=%s&orig=ffp&ver=1", pageurl);
	initHttpReq(&req, buf, 0);
	req.buffer = buf;
	req.flags = FLAG_CLOSE_CONN;
	req.iDataSize = param->iDataBytes;
	if (httpGet(&req)) return 0;
	
	p = strstr(req.buffer, "<span id=\"download_details2\">");
	if (!p) return 0;
	p = strstr(p, "<a href=\"");
	if (!p) return 0;
	req.url = p + 9;
	p = strchr(req.url, '\"');
	if (p) *p = 0;
	req.method = HM_HEAD;
	req.flags = FLAG_CLOSE_CONN | FLAG_KEEP_HEADER;

	char *q = buf;
	q += sprintf(q, "<html><body><a href=\"");
	for(s = 0; ; ) {
		if (httpGet(&req)) return 0;
		p = strstr(req.header, "Location: ");
		if (!p) break;
		s = p + 10;
		p = strchr(s, '\r');
		*p = 0;
		req.url = s;
		strcpy(q, s);
	}
	if (s) {
		strcat(param->pucBuffer, "\">Download</a></body></html>");
		param->iDataBytes = strlen(param->pucBuffer);
	}
	free(req.header);
	if (!s) return 0;
	param->fileType = HTTPFILETYPE_HTML;
	return FLAG_DATA_RAW;
}

void DoRegisterHandlers(CHttpd* httpd)
{
	httpd->RegisterHandler("pref/get", uhPrefGet);
	httpd->RegisterHandler("pref/set", uhPrefSet);
	httpd->RegisterHandler("pref/revert", uhPrefRevert);
	httpd->RegisterHandler("pref/pref.xml", uhPrefXml);
	httpd->RegisterHandler("pref/plugin.xml", uhPluginXml);
	httpd->RegisterHandler("pref/default.xml", uhPrefDefault);
	httpd->RegisterHandler("pref/adsframe.htm", uhAdsFrame);
	httpd->RegisterHandler("openurl", uhOpenUrl);
	httpd->RegisterHandler("netvideo", uhGetNetVideoUrl);
	httpd->RegisterHandler("setup/savedata", uhSetupSave);
	//httpd->RegisterHandler("~stats", uhStats);
}
