/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#include "config.h"
#include <tchar.h>
#include <stdio.h>
#include "atlstr.h"
#include "resource.h"
#include "menc.h"
#include "audiodialogs.h"

CDlgLameMP3::CDlgLameMP3()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_SLIDER_AUDIO, "audioenc.lame.quality"},
		{IDC_AUDIO_BITRATE, "audioenc.lame.quality"},
		{IDC_SLIDER_SPEED, "audioenc.lame.aq"},
		{IDC_AUDIO_QUALITY, "audioenc.lame.aq"},
		{IDC_RATE_MODE, "audioenc.lame.mode"},
		{IDC_STEREO_MODE, "audioenc.lame.stereo"},
		{0}
	};
	ctrls = s_ctrls;
	dialogid = IDD_LAMEMP3;
}

void CDlgLameMP3::Refresh()
{
	int range[2];
	int mode = mcGetPrefValueInt("audioenc.lame.mode");
	ctrls[0].prefkey = mode ? "audioenc.lame.bitrate" : "audioenc.lame.quality";
	ctrls[1].prefkey = ctrls[0].prefkey;
	mcGetPrefProperty(ctrls[0].prefkey, PP_RANGE, range);
	SendDlgMsg(IDC_SLIDER_AUDIO,TBM_SETRANGE,0,MAKELPARAM(range[0],range[1]));
	SendDlgMsg(IDC_SLIDER_AUDIO,TBM_SETPAGESIZE,0,mode ? 16 : 1);
	ShowWindow(GetHandle(IDC_KBPS2), mode == 0 ? SW_HIDE : SW_SHOW);
}

bool CDlgLameMP3::onSelChange(int dlgid)
{
	return false;
}

static char bitrate_3gp[] = {48, 36, 32, 28, 24, 20, 16, 12, 0};

CDlg3GPP::CDlg3GPP()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_ENABLE_PS, "audioenc.aac3gpp.ps"},
		{IDC_SLIDER1, "audioenc.aac3gpp.bitrate"},
		{IDC_BITRATE, "audioenc.aac3gpp.bitrate"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_3GPPAAC;
}

CDlgAMR::CDlgAMR()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_ENABLE_DTX, "audioenc.amr.dtx"},
		{IDC_RADIO1, "audioenc.amr.codec", 0},
		{IDC_RADIO2, "audioenc.amr.codec", 1},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_AMR;
}

void CDlgAMR::RefreshBitrates(int codec)
{
	static char bitrate_amrwb[] = {8, 10, 13, 15, 17, 18, 22, 23, 24, 0};
	static char bitrate_amrnb[] = {4, 5, 6, 7, 8, 9, 10, 12, 12, 0};
	char *bitrates = codec == 0 ? bitrate_amrnb : bitrate_amrwb;
	SendDlgMsg(IDC_PRESET, CB_RESETCONTENT, 0, 0);
	for (int i = 0; bitrates[i]; i++) {
		char preset[8];
		sprintf(preset, "%d Kbps", bitrates[i]);
		SendDlgMsg(IDC_PRESET, CB_ADDSTRING, 0, (LPARAM)preset);
	}
	SendDlgMsg(IDC_PRESET, CB_SETCURSEL, mcGetPrefValueInt("audioenc.amr.mode"), 0);
}

bool CDlgAMR::onSelChange(int dlgid)
{
	if (dlgid != IDC_PRESET) return false;
	int i = SendDlgMsg(IDC_PRESET, CB_GETCURSEL, 0, 0);
	mcSetPrefValueInt("audioenc.amr.mode", i);
	return true;
}

void CDlgAMR::Refresh()
{
	RefreshBitrates(mcGetPrefValueInt("audioenc.amr.codec"));
}

CDlgFFMPEG::CDlgFFMPEG()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_CODEC, "audioenc.ffmpeg.codec"},
		{IDC_SLIDER1, "audioenc.ffmpeg.bitrate"},
		{IDC_BITRATE, "audioenc.ffmpeg.bitrate"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_FFMPEGAUDIO;
}

CDlgFaac::CDlgFaac()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_TB_BITRATE, "audioenc.faac.quality"},
		{IDC_BITRATE, "audioenc.faac.quality"},
		{IDC_COMBO1, "audioenc.faac.objectType"},
		{IDC_MPEG2, "audioenc.faac.version", 0},
		{IDC_MPEG4, "audioenc.faac.version", 1},
		{IDC_OPT_QUALITY, "audioenc.faac.mode", 0},
		{IDC_OPT_BITRATE, "audioenc.faac.mode", 1},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_FAAC;
}

void CDlgFaac::Refresh()
{
	if (mcGetPrefValueInt("audioenc.faac.mode")==0) {
		ctrls[0].prefkey = "audioenc.faac.quality";
		SendDlgMsg(IDC_TB_BITRATE,TBM_SETRANGE,TRUE,MAKELONG(10,500));
		SendDlgMsg(IDC_TB_BITRATE,TBM_SETPAGESIZE,0,50);
	} else {
		ctrls[0].prefkey = "audioenc.faac.bitrate";
		SendDlgMsg(IDC_TB_BITRATE,TBM_SETRANGE,TRUE,MAKELONG(16,256));
		SendDlgMsg(IDC_TB_BITRATE,TBM_SETPAGESIZE,0,8);
	}
	ctrls[1].prefkey = ctrls[0].prefkey;
}

CDlgVorbis::CDlgVorbis()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_TB_BITRATE, "audioenc.vorbis.quality"},
		{IDC_BITRATE, "audioenc.vorbis.quality"},
		{IDC_OPT_QUALITY, "audioenc.vorbis.mode", 0},
		{IDC_OPT_BITRATE, "audioenc.vorbis.mode", 1},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_VORBIS;
}

void CDlgVorbis::Refresh()
{
	int range[2];
	int mode = mcGetPrefValueInt("audioenc.vorbis.mode");
	ctrls[0].prefkey = mode ? "audioenc.vorbis.bitrate" : "audioenc.vorbis.quality";
	ctrls[1].prefkey = ctrls[0].prefkey;
	mcGetPrefProperty(ctrls[0].prefkey, PP_RANGE, range);
	SendDlgMsg(IDC_TB_BITRATE,TBM_SETRANGE,0,MAKELPARAM(range[0],range[1]));
	SendDlgMsg(IDC_TB_BITRATE,TBM_SETPAGESIZE,0, mode ? 16 : 10);
	ShowWindow(GetHandle(IDC_KBPS), mode == 0 ? SW_HIDE : SW_SHOW);
}

CDlgAACPlus::CDlgAACPlus()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_TB_BITRATE, "audioenc.aacplus.bitrate"},
		{IDC_BITRATE, "audioenc.aacplus.bitrate"},
		{IDC_MODE, "audioenc.aacplus.mode"},
		{IDC_AAC_TYPE, "audioenc.aacplus.type"},
		{IDC_USEV1, "audioenc.aacplus.v1"},
		{IDC_MPEG2, "audioenc.aacplus.version", 0},
		{IDC_MPEG4, "audioenc.aacplus.version", 1},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_AACPLUS;
}

bool CDlgAACPlus::onCreate2()
{
	SendDlgMsg(IDC_TB_BITRATE,TBM_SETPAGESIZE,0,8);
	return true;
}

void CDlgAACPlus::Refresh()
{
	int pos = SendDlgMsg(IDC_TB_BITRATE,TBM_GETPOS,0,0);
	int type = mcGetPrefValueInt("audioenc.aacplus.type");
	if (pos > 48 && mcGetPrefValueInt("audioenc.aacplus.mode") == 3) {
		mcSetPrefValueInt("audioenc.aacplus.mode",1);
	}
	if (pos <= 128 && type == 2) {
		type = 0;
	}
	if (pos > 128 && type == 0) {
		type = 2;
	}
	if (pos > 256) {
		type = 1;
	}
	mcSetPrefValueInt("audioenc.aacplus.type", type);
}

CDlgHelix::CDlgHelix()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_PROFILE, "audioenc.helix.audience"},
		{IDC_MUSIC, "audioenc.helix.type", 0},
		{IDC_VOICE, "audioenc.helix.type", 1},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_HELIX;
}

CDlgHelixMP3::CDlgHelixMP3()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_TB_BITRATE, "audioenc.helixmp3.quality"},
		{IDC_BITRATE, "audioenc.helixmp3.quality"},
		{IDC_STEREO_MODE, "audioenc.helixmp3.stereo"},
		{IDC_VBR, "audioenc.helixmp3.mode", 0},
		{IDC_CBR, "audioenc.helixmp3.mode", 1},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_HELIXMP3;
}

void CDlgHelixMP3::Refresh()
{
	int range[2];
	int mode = mcGetPrefValueInt("audioenc.helixmp3.mode");
	ctrls[0].prefkey = mode ? "audioenc.helixmp3.bitrate" : "audioenc.helixmp3.quality";
	ctrls[1].prefkey = ctrls[0].prefkey;
	mcGetPrefProperty(ctrls[0].prefkey, PP_RANGE, range);
	SendDlgMsg(IDC_TB_BITRATE,TBM_SETRANGE,0,MAKELPARAM(range[0],range[1]));
	SendDlgMsg(IDC_TB_BITRATE,TBM_SETPAGESIZE,0,mode ? 16 : 10);
	ShowWindow(GetHandle(IDC_KBPS), mode == 0 ? SW_HIDE : SW_SHOW);
}

CDlgGeneric::CDlgGeneric()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_STDIN, "audioenc.cli.stdin"},
		{IDC_RAWPCM, "audioenc.cli.pcm"},
		{IDC_CMD, "audioenc.cli.command"},
		{IDC_EXTNAME, "audioenc.cli.fileext"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_GENCLI;
}

CDlgWavPack::CDlgWavPack()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_HYBRID, "audioenc.wavpack.hybrid"},
		{IDC_LEVEL, "audioenc.wavpack.bitrate"},
		{IDC_BITRATE, "audioenc.wavpack.bitrate"},
		{IDC_CORRECTION, "audioenc.wavpack.correction"},
		{IDC_BEST, "audioenc.wavpack.best"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_WAVPACK;
}

void CDlgWavPack::Refresh()
{
	BOOL fEnabled = mcGetPrefValueInt("audioenc.wavpack.hybrid");
	EnableCtrl(IDC_LEVEL ,fEnabled);
	EnableCtrl(IDC_CORRECTION ,fEnabled);
	EnableCtrl(IDC_BITRATE ,fEnabled);
}

CDlgFlac::CDlgFlac()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_LEVEL, "audioenc.flac.level"},
		{IDC_LEVEL_TEXT, "audioenc.flac.level"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_FLAC;
}

CDlgApe::CDlgApe()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_LEVEL, "audioenc.ape.level"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_APE;
}

CDlgAls::CDlgAls()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_CHECK1, "audioenc.als.adaptive"},
		{IDC_CHECK2, "audioenc.als.dualStereo"},
		{IDC_CHECK3, "audioenc.als.ltp"},
		{IDC_COMBO1, "audioenc.als.rlsmls"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_ALS;
}

CDlgPcm::CDlgPcm()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_RADIO1, "audioenc.pcm.format", 0},
		{IDC_RADIO2, "audioenc.pcm.format", 1},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_PCM;
}

static const unsigned char wma_qualities[] = {10, 25, 50, 75, 90, 100, 0};
static const unsigned char wma_bitrates[] = {48, 64, 80, 96, 112, 128, 160, 192, 0};

CDlgWma::CDlgWma()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_CODEC, "audioenc.wma.codec"},
		{IDC_MODE, "audioenc.wma.mode"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_WMA;
}

void CDlgWma::Refresh()
{
	int i;
	CString str;
	int codec = mcGetPrefValueInt("audioenc.wma.codec");
	int mode = mcGetPrefValueInt("audioenc.wma.mode");
	if (codec == 2)
		mcSetPrefValueInt("audioenc.wma.mode", mode = 2);

	SendDlgMsg(IDC_BITRATE, CB_RESETCONTENT, 0, 0);		
	if (codec == 2) {
		SendDlgMsg(IDC_BITRATE, CB_ADDSTRING, 0, (LPARAM)"Lossless");
		i = 0;
	} else if (mode == 2) {
		for (i = 0; wma_qualities[i]; i++) {
			str.Format("Quality %d", wma_qualities[i]);
			SendDlgMsg(IDC_BITRATE, CB_ADDSTRING, 0, (LPARAM)str.GetBuffer());
		}
		i = mcGetPrefValueInt("audioenc.wma.quality");
	} else {
		for (i = 0; wma_bitrates[i]; i++) {
			str.Format("%d Kbps", wma_bitrates[i]);
			SendDlgMsg(IDC_BITRATE, CB_ADDSTRING, 0, (LPARAM)str.GetBuffer());
		}
		for (i--; i >= 0 && wma_bitrates[i] > mcGetPrefValueInt("audioenc.wma.bitrate"); i--);
	}
	SendDlgMsg(IDC_BITRATE, CB_SETCURSEL, i, 0);
}

bool CDlgWma::onSelChange(int dlgid)
{
	switch (dlgid) {
	case IDC_BITRATE: {
		int i = SendDlgMsg(IDC_BITRATE, CB_GETCURSEL, 0, 0);
		if (mcGetPrefValueInt("audioenc.wma.mode") == 2) {
			mcSetPrefValueInt("audioenc.wma.quality", i);
		} else {
			mcSetPrefValueInt("audioenc.wma.bitrate", wma_bitrates[i]);
		}
		} break;
	default:
		return false;
	}
	return true;
}

CDlgNeroRefEnc::CDlgNeroRefEnc()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_SLIDER1, "audioenc.nero.quality"},
		{IDC_BITRATE, "audioenc.nero.quality"},
		{IDC_COMBO1, "audioenc.nero.format"},
		{IDC_COMBO2, "audioenc.nero.mode"},
		{IDC_HINT, "audioenc.nero.hint"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_NEROREF;
}

bool CDlgNeroRefEnc::onSelChange(int dlgid)
{
	if (dlgid == IDC_COMBO1) {
		if (ComboGetSel(dlgid)) {
			MsgBox(IDS_NERO_TIP);
		}
	}
	return false;
}

void CDlgNeroRefEnc::Refresh()
{
	int range[2];
	int mode = mcGetPrefValueInt("audioenc.nero.mode");
	ctrls[0].prefkey = mode ? "audioenc.nero.bitrate" : "audioenc.nero.quality";
	ctrls[1].prefkey = ctrls[0].prefkey;
	mcGetPrefProperty(ctrls[0].prefkey, PP_RANGE, range);
	SendDlgMsg(IDC_SLIDER1,TBM_SETRANGE,0,MAKELPARAM(range[0],range[1]));
	SendDlgMsg(IDC_SLIDER1,TBM_SETPAGESIZE,0,mode ? 16 : 1);
	ShowWindow(GetHandle(IDC_KBPS), mode == 0 ? SW_HIDE : SW_SHOW);
}

CDlgNull::CDlgNull()
{
	dialogid = IDD_NO_OPTS;
}

int ParamNotifyCallback(int idx, void* value);

bool CDlgNull::onClick(int dlgid)
{
	ShowWeb(mcShowConfig(CD_AUDIO_ENC, 0));
	return true;
}
