/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. The name of the author may not be used to endorse or promote products
*    derived from this software without specific prior permission.
*
* THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*************************************************************************/

#include "config.h"
#include <commctrl.h>
#include <shlobj.h>
#include <stdio.h>
#include <time.h>
#include "atlstr.h"
#include "resource.h"
#include "common.h"
#include "menc.h"
#include "discnav.h"
#include "oleidl.h"
#include "audiodialogs.h"
#include "httpget.h"
#include "revision.h"
#include "optiondialogs.h"
#include "cfilename.h"
#include "processpil.h"
#include "browser.h"
#include "devplugin.h"
#include "deviceplugins.h"
#include "cropper.h"

#define CONSOLE_CAPTION "MediaCoder Console Window"
#define SPLASH_SHOW_TIME 2000

#define TAB_VIEW_HEIGHT 71
#define INFO_BOX_HEIGHT 46
#define PROGRESS_BAR_HEIGHT 18
#define PROGRESS_TEXT_WIDTH 80
#define MODE_OPTION_WIDTH 70
#define OPT_TAB_HEIGHT 180
#define OPT_TAB_WIDTH 280
#define TOOLBAR_HEIGHT 40
#define TOOLBAR_WIDTH 400
#define MARGIN 10

enum { IS_READY = 0, IS_DONE, IS_SKIPPED, IS_ERROR };
static char *itemstate[] = {"Ready", "Done", "Skipped", "Error"};

void RegisterDropWindow(HWND hwnd);
void UnregisterDropWindow(HWND hwnd);
VOID CALLBACK StatsTimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);

int UpdateTime();
void RefreshOptionTab();
void RefreshAudioTab();
void GlobalUninit();
void UpdateTaskStats();

HINSTANCE hInst;
char apppath[MAX_PATH];
HICON hAppIcon;
static char *caption;
static HANDLE hThread;

CMenc mc;
CDlgDevNav devnav;
CWindow wndStats;
CAddTrackDialog trackdlg;
CDeviceDialog inputdlg;
extern CDeviceDialog device;
static COptionDialog* AudioDialogs[AC_COUNT];
static COptionDialog* optionDialog[OD_COUNT];
static CDlgContainer* container;
static CWebBrowser* web = 0;
static CDlgNull* dlgnull = 0;

int iAudCodecDlg;
int iOptionDialog;
int iQueueItemIndex;

int EventHandler(MC_EVENT event, void* arg);
int ShowStats();
#define EVENT EventHandler
MC_STATE* mcstate;

char* getNetVideoUrl(char* pageurl)
{
	char *p = strstr(pageurl, "://");
	char *s;
	if (!p) return 0;
	p += 3;
	s = strchr(p, '.');
	p = strchr(p, '/');
	if (p && s && s > p)
		return strdup(pageurl);

	char *buf = (char*)malloc(4096);
	httpRequest req;
	sprintf(buf, "http://videodownloader.net/get/?url=%s&orig=ffp&ver=1", pageurl);
	p = mcGetPrefValueStr("overall.generic.proxy");
	if (!*p) p = 0;
	initHttpReq(&req, buf, p);
	req.buffer = buf;
	req.flags = FLAG_CLOSE_CONN;
	req.iDataSize = 4096;
	if (httpGet(&req)) return 0;
	
	p = strstr(req.buffer, "<span id=\"download_details2\">");
	if (!p) return 0;
	p = strstr(p, "<a href=\"");
	if (!p) return 0;
	req.url = p + 9;
	p = strchr(req.url, '\"');
	if (p) *p = 0;
	if (!IsNetworkPath(req.url)) return 0;
	req.url = strdup(req.url);
	req.method = HM_HEAD;
	req.flags = FLAG_CLOSE_CONN | FLAG_KEEP_HEADER;

	for(s = 0; ; ) {
		if (httpGet(&req)) return 0;
		p = strstr(req.header, "Location: ");
		if (!p) break;
		s = p + 10;
		p = strchr(s, '\r');
		*p = 0;
		free(req.url);
		req.url = strdup(s);
	}
	free(req.header);
	free(buf);
	return req.url;
}

class CDlgUrl:public CWindow {
private:
	bool onClick(int dlgid)
	{
		char buf[512];
		if (dlgid != IDOK) return false;
		if (GetText(IDC_EDIT1, buf, sizeof(buf))) {
			SetCaption("Resolving URL...");
			mc.UpdateStatusBar("Resolving URL...", 0, 0);
			char *p = getNetVideoUrl(buf);
			mcSetState(STATE_IDLE);
			if (p) {
				EnqueueFile(p);
				free(p);
			} else {
				MsgBox(IDS_URL_ERROR);
			}
		}
		Close();
		return true;
	}
};

class CFileInfo:public CWindow {
public:
	void ShowInfo(T_ITEM* item);
private:
	bool onCreate();
	bool onCommand(int dlgid, int code);
	void SetText(int dlgid, CString& str);
};

CFileInfo fileinfo;

class CSplash:public CWindow {
private:
	bool onPaint() {
		HDC hdc,hdcmem;
		HBITMAP hbm;
		PAINTSTRUCT ps;
		hdc = BeginPaint(hwnd, &ps);
		hdcmem=CreateCompatibleDC(hdc);
		hbm=LoadBitmap(hInst,MAKEINTRESOURCE(IDB_SPLASH));
		SelectObject(hdcmem,hbm);
		BitBlt(hdc,0,0,372,340,hdcmem,0,0,SRCCOPY);
		DeleteObject(hbm);
		DeleteDC(hdcmem);
		EndPaint(hwnd, &ps);
		return true;
	}
	bool onMouseClick(DWORD flags) { return onTimer(0); }
	bool onTimer(int id) { 
		KillTimer(hwnd,id);
#ifndef CONFIG_WIN9X
		for (int i = 255; i>=5; i-=10) {
			SetLayeredWindowAttributes(hwnd, 0, i, LWA_ALPHA);
			Sleep(10);
		}
#endif
		Destroy();
		return true; 
	}
public:
	void Open(int timeout, CWindow* parent) {
#ifndef CONFIG_WIN9X
		Create(WS_POPUP|WS_BORDER,WS_EX_TOPMOST|WS_EX_LAYERED, parent);
		SetLayeredWindowAttributes(hwnd, 0, 0, LWA_ALPHA);
#else
		Create(WS_POPUP|WS_BORDER,WS_EX_TOPMOST, parent);
#endif
		MoveCenter(374,345);
		Show();
		onPaint();
#ifndef CONFIG_WIN9X
		for (int i = 15; i<=255; i+=20) {
			SetLayeredWindowAttributes(hwnd, 0, i, LWA_ALPHA);
			Sleep(10);
		}
#endif
		if (timeout) SetTimer(hwnd,0,timeout,0);
	}
};

static CSplash splash;

static CCropper cropper;

void ShowCropper(CWindow* parent, float aspectratio)
{
	T_ITEM* item = GetSelectedItem();
	if (item) {
		if (!mcPlayerStop()) Sleep(500);
		cropper.aspect = aspectratio;
		if (cropper.Open(parent, item)) {
			mc.MsgBox(IDS_UNABLE_CROP);
		}
	} else {
		mc.MsgBox(IDS_NO_FILE);
	}
}

#define NOW GetTickCount()

static char* mirrors[] = {"mediacoder.sourceforge.net", 0};
static int alivemirror = -1;
static HANDLE hThreadCheckUpdate = 0;

void CheckForUpdate(bool startup)
{
	static updating = false;
	if (updating) return;
	updating = true;
	bool showupdate;
	httpRequest req;
	CString url;
	int i;
	char* proxy = mcGetPrefValueStr("overall.generic.proxy");
	if (!*proxy) proxy = 0;

	if (startup) {
		alivemirror = -1;
		showupdate = false;
		for (i = 0; mirrors[i]; i++) {
			url.Format("http://%s/version.js", mirrors[i]);
			initHttpReq(&req, url.GetBuffer(), proxy);
			if (!httpGet(&req)) {
				alivemirror = i;
				break;
			}
		}
		if (alivemirror >= 0) {
			int newBuild = 0;
			char *p = strstr(req.buffer, "mcbuild=");
			if (p) {
				p += 8;
				newBuild = atoi(p);
				if (newBuild > BUILD_NO && newBuild > mcGetPrefValueInt("overall.generic.ignoredBuild")) {
					showupdate = true;
				}
			}
		}
		if (req.buffer) free(req.buffer);
	} else {
		if (alivemirror == -1) alivemirror = 0;
		showupdate = true;
	}
	if (showupdate) {
		url.Format("http://%s/checkupdate.htm?timestamp=%d&ver=%d.%d.%d&build=%d&localport=%d",
			mirrors[alivemirror], GetTickCount(), VER_MAJOR, VER_MINOR, VER_REV, BUILD_NO, mcstate->httpdPort);
		ShellExecute(mc.hwnd, "open", url.GetBuffer(), 0, 0, SW_SHOW);
	}
	updating = false;
}

void VisitWebsite(int mirrorid)
{
	CString url;
	url.Format("http://%s/", mirrors[mirrorid]);
	ShellExecute(mc.hwnd, "open", url.GetBuffer(), 0, 0, SW_SHOW);
}

CWebBrowser* CreateBrowserInstance()
{
	if (!mcGetPrefValueInt("overall.generic.httpd")) {
		mc.MsgBox(IDS_START_HTTPD);
		mcSetPrefValueInt("overall.generic.httpd", 1);
		mcStartHttpd();
	}
	return new CWebBrowser;
}

int ShowWeb(char* path)
{
	if (!path) {
		if (web) web->Close();
		return 0;
	}
	if (!web) web = CreateBrowserInstance();
	return web->Open(path);
}

void DestroyBrowserInstance(CWebBrowser **pbrowser)
{
	if (*pbrowser) {
		delete *pbrowser;
		*pbrowser = 0;
	}
}
void EnqueueItem(T_ITEM* item, char* alias)
{
	mc.AddItem(item, alias);
}

void EnqueueFile(char* filename, char* alias)
{
    T_ITEM* item = mcMakeItem(filename);
	mc.AddItem(item, alias);
	mc.UpdateStatusBar("Item added", 0, 0);
}

void ListEnums(char* key, HWND hCtrl, UINT msg)
{
	char *p;
	for (int i = 0; (p = mcGetPrefValueEnum(key, i)) && *p; i++) {
		if (*p == '\t' || *p == ' ') p++;
		SendMessage(hCtrl, msg, 0, (LPARAM)p);
	}
}

void MsgBoxStr(UINT uID)
{
	CString str;
	str.Format(uID);
	EVENT(MC_MSG_POPUP, str.GetBuffer());
}

void RefreshGui()
{
	mc.Refresh();
}

HWND CreateSBar(HWND hwndParent,char *initialText,int nrOfParts,int ID)
{
	#define SBarParts 3
    HWND hWndStatusbar;
	int   ptArray[SBarParts]={100,250,-1};   // Array defining the number of parts/sections

	hWndStatusbar = CreateStatusWindow(WS_CHILD | WS_VISIBLE | WS_BORDER|SBARS_SIZEGRIP,initialText,hwndParent,ID);

	if(hWndStatusbar) {
		SendMessage(hWndStatusbar,
					SB_SETPARTS,
					SBarParts,
					(LPARAM)ptArray);
	}
    return hWndStatusbar;
}

void CMenc::ShowStatsWindow(bool fShow)
{
	if (fShow) {
		if (!wndStats.hwnd && mc.GetShowState() != SW_MAXIMIZE) {
			wndStats.CreateDlg(0, IDD_STATS, &mc);
			onMove();
			wndStats.Show();
		} else {
			wndStats.SendDlgMsg(IDC_STAT_POS,WM_SETTEXT,0,0);
			wndStats.SendDlgMsg(IDC_STAT_FRAME,WM_SETTEXT,0,0);
			wndStats.SendDlgMsg(IDC_STAT_FPS,WM_SETTEXT,0,0);
			wndStats.SendDlgMsg(IDC_STAT_ESTTIME,WM_SETTEXT,0,0);
			wndStats.SendDlgMsg(IDC_STAT_ESTSIZE,WM_SETTEXT,0,0);
			wndStats.SendDlgMsg(IDC_STAT_AB,WM_SETTEXT,0,0);
			wndStats.SendDlgMsg(IDC_STAT_VB,WM_SETTEXT,0,0);
		}
	} else {
		wndStats.Close();
	}

}

void CMenc::SetInitialPos()
{
	char *p = mcGetPrefValueStr("overall.ui.param");
	int x = ORIGINAL, y = ORIGINAL, alpha = 0;
	int w = 600, h = 560;

	switch (mcGetPrefValueInt("overall.ui.mode")) {
	case UI_SIMPLE:
		h = 480;
		break;
	case UI_DEVICE:
		h = 520;
		break;
	}

	if (*p) w = atoi(p);
	if ((p = strchr(p, ','))) h = atoi(++p);
	if (p && (p = strchr(p, ','))) x = atoi(++p);
	if (p && (p = strchr(p, ','))) y = atoi(++p);
#ifndef CONFIG_WIN9X
	if (p && (p = strchr(p, ','))) alpha = atoi(p);
	if (alpha == 0 || alpha == 255) {
		UnsetWindowFlag(GWL_EXSTYLE, 0x80000);
	} else {
		SetWindowFlag(GWL_EXSTYLE, 0x80000);
		SetAlpha((int)alpha);
	}
#endif
	
	if (x == ORIGINAL && y == ORIGINAL) {
		MoveCenter(w, h);
	} else {
		Move(x, y, w, h);
	}
}

void CMenc::SwitchUIMode()
{
	RECT rc;
	int uimode = mcGetPrefValueInt("overall.ui.mode");
	GetClientRect(hwnd, &rc);
	SetMenuChecked(IDM_MODE_SIMPLE, false);
	SetMenuChecked(IDM_MODE_DEVICE, false);
	SetMenuChecked(IDM_MODE_NORMAL, false);
	SetMenuChecked(IDM_MODE_EXPERT, false);
	HideCtrl(IDC_MODE);
	HideCtrl(IDC_MODE_LABEL);
	HideCtrl(IDC_TAB_VIEW);
	ShowWindow(hInfoBox, SW_HIDE);
	if (uimode != UI_DEVICE) {
		devnav.Close();
	}
	switch (uimode) {
	case UI_SIMPLE:
		opttab.Hide();
		audiotab.Hide();
		SetMenuChecked(IDM_MODE_SIMPLE, true);
		break;
	case UI_DEVICE:
		opttab.Hide();
		audiotab.Hide();
		SetMenuChecked(IDM_MODE_DEVICE, true);
		devnav.CreateDlg(hInst, IDD_DEVICE_NAV, this);
		break;
	case UI_NORMAL:
		opttab.Show();
		audiotab.Show();
		SetMenuChecked(IDM_MODE_NORMAL, true);
		break;
	case UI_EXPERT:
		ShowCtrl(IDC_TAB_VIEW);
		ShowWindow(hInfoBox, SW_SHOW);
		opttab.Show();
		audiotab.Show();
		SetMenuChecked(IDM_MODE_EXPERT, true);
		ShowCtrl(IDC_MODE);
		ShowCtrl(IDC_MODE_LABEL);
		UpdateTabbedView();
		break;
	}
	onSize(rc.right - rc.left, rc.bottom - rc.top);
}

int ShutDownNT(int fPowerOff)
{
	HANDLE hProcessHandle,hTokenHandle;
	LUID tmpLuid;
	TOKEN_PRIVILEGES tkpNew;
	TOKEN_PRIVILEGES tkpPrevious;
	DWORD lBufferNeeded;

	hProcessHandle = GetCurrentProcess();
	if (!OpenProcessToken(hProcessHandle, TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hTokenHandle)) return -1;

	//Get the LUID for the shutdown privilege
	LookupPrivilegeValue("", "SeShutdownPrivilege", &tmpLuid);
	tkpNew.PrivilegeCount = 1; // One privilege to set
	tkpNew.Privileges[0].Luid = tmpLuid;
	tkpNew.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;

	// Enable the shutdown privilege in the access token of this process.
	lBufferNeeded = 0;
	AdjustTokenPrivileges(hTokenHandle, false, &tkpNew, sizeof(tkpPrevious), &tkpPrevious, &lBufferNeeded);

	// Force a Reboot/Poweroff (no option to save files to cancel out)
	ExitWindowsEx(fPowerOff?(EWX_FORCE|EWX_SHUTDOWN|EWX_POWEROFF):(EWX_FORCE|EWX_SHUTDOWN), 0xffff);
	return 0;
}

char* SetFileExt(char* filename, char* fileext)
{
	char *p;
	p=strrchr(filename,'.');
	if (p)
		strcpy(p+1,fileext);
	else
		strcat(filename,fileext);
	return filename;
}

int EventHandler(MC_EVENT event, void* arg)
{
	switch (event) {
	case MC_WRITE_CONSOLE:
		if (mcGetPrefValueInt("overall.ui.expertView") == VIEW_CONSOLE)
			SendMessage(mc.hInfoBox,WM_SETTEXT,0,(LPARAM)arg);
		break;
	case MC_UPDATE_STATE:
		mc.ShowState((int)arg);
		break;
	case MC_UPDATE_TITLE: 
		mc.UpdateTitle((char*)arg);
		break;
	case MC_UPDATE_TIME:
		UpdateTime();
		break;
	case MC_UPDATE_STATS:
		ShowStats();
		break;
	case MC_UPDATE_PROGRESS:
		SendDlgItemMessage(mc.hwnd,IDC_PROGRESS,PBM_SETPOS,(WPARAM)arg,0);
		break;
	case MC_MSG_POPUP:
		MessageBox(0, (char*)arg, APPNAME, MB_OK|MB_ICONINFORMATION);
		break;
	case MC_MSG_ALERT:
		MessageBox(0, (char*)arg, "Alert", MB_OK|MB_ICONEXCLAMATION);
		break;
	case MC_MSG_YESNO:
		return MessageBox(0, (LPCTSTR)arg, "Confirm", MB_YESNO|MB_ICONQUESTION);
	case MC_MSG_YESNOCANCEL:
		return MessageBox(0, (LPCTSTR)arg, "Confirm", MB_YESNOCANCEL|MB_ICONQUESTION);
	case MC_MSG_INFO: {
		DWORD bytes;
		HANDLE hStdout=GetStdHandle(STD_OUTPUT_HANDLE);
		WriteFile(hStdout, (char*)arg, strlen((char*)arg), &bytes, 0);
		} break;
	case MC_MSG_CMD: {
		DWORD bytes;
		HANDLE hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
		if (hStdout != INVALID_HANDLE_VALUE) {
			int len = strlen((char*)arg);
			WriteFile(hStdout,"\n# ",3,&bytes,0);
			WriteFile(hStdout, arg, len,&bytes,0);
			WriteFile(hStdout,"\n\n",2,&bytes,0);
		}
		} break;
	case MC_UPDATE_UI:
		RefreshAudioTab();
		RefreshOptionTab();
		RefreshGui();
		break;
	case MC_UPDATE_LOG:
		if (IsWindowVisible(mc.hInfoBox)) {
			SendMessage(mc.hInfoBox, WM_SETTEXT, 0, (LPARAM)mcGetLogText(0));
		}
		break;
	case MC_JOB_START: {
		T_ITEM* item = (T_ITEM*)arg;
		if (iQueueItemIndex == 0) {
			mc.list.Scroll(0, -(mc.list.GetCount() * 12));
		}
		mc.list.SetSelection(iQueueItemIndex, 1);
		mc.list.SetListFocus(iQueueItemIndex);
		mcGetMediaInfo(item);
		mc.UpdateStatusBar(item->filename,2,0);
		} break;
	case MC_JOB_FINISH:	{
		char *p = itemstate[IS_ERROR];
		switch (mcstate->state) {
		case STATE_SKIPPING:
			p = itemstate[IS_SKIPPED];
			break;
		case STATE_STOPPING:
			p = itemstate[IS_READY];
			break;
		case STATE_ENCODING:
			if (((T_ITEM*)arg)->flags & IF_DONE) {
				p = itemstate[IS_DONE];
				break;
			}
		default:
			p = itemstate[IS_ERROR];
		}
        mc.list.SetText(iQueueItemIndex, LC_STATE, p);
		mc.SendDlgMsg(IDC_PROGRESSALL,PBM_SETPOS, mcstate->stats.fileprocessed * 100 / mcstate->totalfiles ,0);
		UpdateTaskStats();
		if (iQueueItemIndex >= 3)
			mc.list.Scroll(0, 12);
		iQueueItemIndex++;
		} break;
	default:
		return ER_BYPASSED;
	}
	return ER_OK;
}

void ShowWizard(CWindow* parent)
{
}

void CMenc::UpdateTitle(char* text)
{
	if (text) {
		char buf[256];
		sprintf(buf, "%s - %s", caption, text);
		SetCaption(buf);
	} else {
		SetCaption(caption);
	}
}

void RefreshAudioTab()
{
	if (iAudCodecDlg >= 0) AudioDialogs[iAudCodecDlg]->RefreshDialog();
}

void RefreshOptionTab()
{
	if (iOptionDialog >= 0) optionDialog[iOptionDialog]->RefreshDialog();
}

static HBITMAP hBmpPlay[2];
static HBITMAP hBmpStart[2];

DWORD WINAPI TranscodeThread()
{
	int state;

	state = mcTranscodeQueue(0);
	UpdateTaskStats();
	mc.SendDlgMsg(IDC_PROGRESS,PBM_SETPOS,0,0);
	mc.SendDlgMsg(IDC_PROGRESSALL,PBM_SETPOS,0,0);
	EVENT(MC_WRITE_CONSOLE, 0);
    mc.ShowStatsWindow(false);
	mc.SendDlgMsg(IDC_TRANSCODE,BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)hBmpStart[0]);
	if (mcstate->state == STATE_SHUTDOWN) {
		mcSaveParam(CFGFILE);
		ShutDownNT(1);
		mc.Close();
	}
	mc.ShowState(mcstate->state);
	RefreshGui();
	mc.SetCaption(caption);
	return 0;
}

static void UpdateTaskStats()
{
	char buf[16];
	HWND hwndStats = wndStats.hwnd;
	if (!hwndStats) return;
	sprintf(buf,"%d",mcstate->totalfiles);
	SendDlgItemMessage(hwndStats,IDC_STAT_TOTAL_FILES,WM_SETTEXT,0,(LPARAM)buf);
	sprintf(buf,"%d",mcstate->stats.encodedFiles);
	SendDlgItemMessage(hwndStats,IDC_STAT_ENCODED_FILES,WM_SETTEXT,0,(LPARAM)buf);
	sprintf(buf,"%d",mcstate->stats.skippedFiles);
	SendDlgItemMessage(hwndStats,IDC_STAT_SKIPPED_FILES,WM_SETTEXT,0,(LPARAM)buf);
	sprintf(buf,"%d",mcstate->stats.errorFiles);
	SendDlgItemMessage(hwndStats,IDC_STAT_ERROR_FILES,WM_SETTEXT,0,(LPARAM)buf);
	sprintf(buf,"%u", mcstate->stats.decodedMB);
	SendDlgItemMessage(hwndStats,IDC_STAT_ENCODED_KB,WM_SETTEXT,0,(LPARAM)buf);
}

int UpdateTime()
{
	char buf[16];
	HWND hwndStats = wndStats.hwnd;
	DWORD tm = GetTickCount();
	if (!hwndStats) return 0;
	int sec = mcstate->stats.dwElapsed >> 10;
	sprintf(buf,"%d:%02d", sec / 60, sec % 60);
	SendDlgItemMessage(hwndStats,IDC_STAT_ELAPSED,WM_SETTEXT,0,(LPARAM)buf);
	sec = (tm - mcstate->stats.dwEncStartTime)>>10;
	sprintf(buf,"%d:%02d", sec / 60, sec % 60);
	SendDlgItemMessage(hwndStats,IDC_STAT_TOTALTIME,WM_SETTEXT,0,(LPARAM)buf);
	float percent = (mcstate->stage == ST_PROCESSING_AUDIO) ? mcstate->astats.percent : mcstate->vstats.percent;
	if (percent) {
		sec = (int)(mcstate->stats.dwElapsed / percent * (100 - percent)) >> 10;
		if (sec > 0) {
			sprintf(buf,"%dmins", sec / 60);
			SendDlgItemMessage(hwndStats,IDC_STAT_ESTTIME,WM_SETTEXT,0,(LPARAM)buf);
		}
	}
	return 0;
}

int UpdateVideoStats()
{
	char buf[128], *p;
	VIDEO_STATS* vstats = &mcstate->vstats;
	HWND hwndStats = wndStats.hwnd;

	p = buf;
	if (mc.GetShowState() != SW_SHOWMINIMIZED)
		p += sprintf(p, "%s - ", caption);
	sprintf(p, mcstate->totalpass > 1 ? "%4.1f%% done (pass %d of %d)" : "%4.1f%% done", 
		vstats->percent,
		mcstate->currentpass,
		mcstate->totalpass);
	mc.SetCaption(buf);

	if (hwndStats) {
		sprintf(buf, "%d:%02d", vstats->pos / 60, vstats->pos % 60);
		SendDlgItemMessage(hwndStats,IDC_STAT_POS,WM_SETTEXT,0,(LPARAM)buf);
		sprintf(buf, "%df", vstats->frame);
		SendDlgItemMessage(hwndStats,IDC_STAT_FRAME,WM_SETTEXT,0,(LPARAM)buf);
		sprintf(buf, "%4.1ffps", vstats->fps);
		SendDlgItemMessage(hwndStats,IDC_STAT_FPS,WM_SETTEXT,0,(LPARAM)buf);
		sprintf(buf, "%dmins", vstats->remaintime);
		SendDlgItemMessage(hwndStats,IDC_STAT_ESTTIME,WM_SETTEXT,0,(LPARAM)buf);
		sprintf(buf, "%dMB", vstats->estsize  >> 10);
		SendDlgItemMessage(hwndStats,IDC_STAT_ESTSIZE,WM_SETTEXT,0,(LPARAM)buf);
		sprintf(buf, "%dKbps", vstats->abitrate);
		SendDlgItemMessage(hwndStats,IDC_STAT_AB,WM_SETTEXT,0,(LPARAM)buf);
		sprintf(buf, "%dKbps", vstats->vbitrate);
		SendDlgItemMessage(hwndStats,IDC_STAT_VB,WM_SETTEXT,0,(LPARAM)buf);
	}
	return 0;
}

T_ITEM* GetSelectedItem(int defidx)
{
	T_ITEM* item;
	int i = mc.list.GetFocus();
	if (i >= 0) {
		item = mcGetItemByIndex(i);
	} else {
		if (defidx < 0) return 0;
		item = mcGetItemByIndex(defidx);
	}
	return item;
}

int StartTranscode(int mode)
{
	if (mc.list.GetCount()<=0) {
		mc.MsgBox(IDS_NO_FILE);
		return -1;
	}
	
	{
		DWORD attr;
		CString dir;
		dir = mcGetPrefValueStr("overall.task.destdir");
		if (!dir.IsEmpty() && !IsDriveValid(dir.GetAt(0))) {
			mc.MsgBox(IDS_DRIVE_ABSENT);
			return -1;
		}
		if (!mcGetPrefValueInt("overall.task.autoCreateDir") && !dir.IsEmpty()) {
			if (dir.Right(1) == '\\') dir.Delete(dir.GetLength() - 1);
			attr = GetFileAttributes(dir.GetBuffer());
			if (attr == INVALID_FILE_ATTRIBUTES || !(attr & FILE_ATTRIBUTE_DIRECTORY)) {
				mc.MsgBox(IDS_CHANGE_DEST_DIR);
				optionDialog[OD_OUTPUT]->onClick(IDC_BROWSE1);
			}
		}
	}

	T_ITEM* item = mcGetNextItem(0);
	int i = 0;
	int readyitems = 0;
	for (; item; item = mcGetNextItem(item), i++) {
		if (!(item->flags & IF_DONE)) readyitems++;
		mc.list.SetSelection(i, 0);
	}
	if (!readyitems) {
		if (mc.Ask(IDS_NO_READY_ITEM, MB_YESNO) == IDYES) {
			for (i = 0, item = mcGetNextItem(0); item; item = mcGetNextItem(item), i++) {
				item->flags &= ~IF_DONE;
				mc.list.SetText(i, LC_STATE, itemstate[IS_READY]);
			}
		} else {
			mc.MsgBox(IDS_NO_ITEM);
			return -1;
		}
	}

	if (mcGetPrefValueInt("overall.task.deleteOriginal")) {
		if (mc.Ask("You have chosen to delete original files after transcoding. Are you sure you want this?", MB_YESNO) == IDNO)
			return -1;
	}

	if (mode ==0 && mcGetPrefValueInt("overall.ui.stats") == 0) {
		mc.ShowStatsWindow(true);
		mc.SetForeground();
	}

	EVENT(MC_WRITE_CONSOLE, 0);
	mc.SendDlgMsg(IDC_TRANSCODE,BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)hBmpStart[1]);
	mc.SendDlgMsg(IDC_PROGRESSALL,PBM_SETPOS,0,0);
	mcstate->totalfiles = mc.list.GetCount();
	iQueueItemIndex = 0;
	UpdateTaskStats();

	for (int i = mc.list.GetCount() - 1; i >= 0; i--)
		mc.list.SetSelection(i, 0);

	DWORD id;
	hThread = CreateThread(0,0,(LPTHREAD_START_ROUTINE)TranscodeThread,0,0,&id);
	return 0;
}

static char** exts;

int CMenc::EnumDir(char* pchDir)
{
	WIN32_FIND_DATA* findData;
	CString fullpath;
	HANDLE hFind;
	int ret = 0;

	findData=(WIN32_FIND_DATA*)malloc(sizeof(WIN32_FIND_DATA));
	fullpath.Format("%s\\*.*",pchDir);
	memset(findData->cFileName, 0, MAX_PATH);
	hFind = FindFirstFile(fullpath.GetBuffer(),findData);
	if (hFind == INVALID_HANDLE_VALUE) return 1;
	do {
		if (findData->cFileName[0]=='.' && (findData->cFileName[1]==0 || *(WORD*)(findData->cFileName+1)=='.')) continue;
		fullpath.Format("%s\\%s",pchDir,findData->cFileName);
		if (findData->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			ret=0;
			if (mcGetPrefValueInt("overall.generic.recurse")) 
				ret = EnumDir(fullpath.GetBuffer());
			if (ret)
				break;
			else
				continue;
		}
		char* p=strrchr(findData->cFileName,'.');
		if (p++) {
			for (int i=0; exts[i]; i++) {
				if (!stricmp(p, exts[i])) {
					T_ITEM* item = mcMakeItem(fullpath.GetBuffer());
					AddItem(item);
					break;
				}
			}
		}
	} while (FindNextFile(hFind,findData));
	FindClose(hFind);
	free(findData);
	return ret;
}

void CMenc::AddFolder(char* dir)
{
	char *incfiletypes = strdup(mcGetPrefValueStr("overall.generic.fileTypes"));
	char *p; 
	int i = strlen(dir);
	if (dir[i-1] == '\\') dir[i - 1] = 0;
	for (i = 1, p = incfiletypes; *p; p++) {
		if (*p == ' ') i++;
	}
	exts = (char**)malloc((i+ 1) * sizeof(char*));
	i = 0;
	for (char* p=strtok(incfiletypes," "); p != 0; p=strtok(0," ")) {
		exts[i++] = p;
	}
	exts[i] = 0;
	EnumDir(dir);
	free(incfiletypes);
	free(exts);
}

void CMenc::ShowSubDialog(int idx)
{
	if (iOptionDialog>=0 && optionDialog[iOptionDialog]) optionDialog[iOptionDialog]->Hide();
	optionDialog[idx]->RefreshDialog(0);
	optionDialog[idx]->Show();
	iOptionDialog=idx;
}

void CMenc::ShowAudioDialog(int idx)
{
	if (idx<0) return;
	if (iAudCodecDlg>=0 && AudioDialogs[iAudCodecDlg]) AudioDialogs[iAudCodecDlg]->Hide();
	iAudCodecDlg=idx;
	TabCtrl_SetCurSel(audiotab.hwnd,idx);
	if (AudioDialogs[iAudCodecDlg]) {
		AudioDialogs[iAudCodecDlg]->Show();
		AudioDialogs[iAudCodecDlg]->RefreshDialog(0);
	}
}

bool CMenc::onNotify(LPNMHDR pnmh)
{
	int idx;
	switch (pnmh->code) {
	case LVN_ITEMACTIVATE:
		onClick(IDC_PLAY);
		break;
	case NM_CLICK: {
		T_ITEM* item = GetSelectedItem(-1);
		mcGetMediaInfo(item);
		RefreshOptionTab();
		} break;
	case TCN_SELCHANGE:
		idx = TabCtrl_GetCurSel(pnmh->hwndFrom);
		switch (pnmh->idFrom) {
		case IDC_TAB:
			ShowSubDialog(idx);
			break;
		case IDC_TAB_AUDIO:
			if (mcstate->state==STATE_IDLE) {
				mcSetPrefValueInt("overall.audio.encoder", idx);
			}
			ShowAudioDialog(idx);
			break;
		case IDC_TAB_VIEW:
			mcSetPrefValueInt("overall.ui.expertView", idx);
			UpdateTabbedView();
			break;
		}
		break;
	case LVN_KEYDOWN: {
		LPNMLVKEYDOWN pnkd = (LPNMLVKEYDOWN)pnmh;
		switch (pnkd->wVKey) {
		case VK_DELETE:
			onClick(IDC_REMOVE);
			break;
		}
		} break;
	}
	return true;
}

bool CFileInfo::onCommand(int dlgid, int code)
{
	switch (dlgid) {
	case IDCANCEL:
		Close();
		break;
	default:
		return false;
	}
	return true;
}

void CFileInfo::SetText(int dlgid, CString& str)
{
	SendDlgMsg(dlgid, WM_SETTEXT, 0, (LPARAM)str.GetBuffer());
}

bool CFileInfo::onCreate()
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));       // zero out structure
	lf.lfHeight = 14;                      // request a 12-pixel-height font
	lf.lfWeight = FW_NORMAL;
	strcpy(lf.lfFaceName, "Courier New");
	HFONT hFont=CreateFontIndirect(&lf);
	SendDlgMsg(IDC_DETAIL, WM_SETFONT, (WPARAM)hFont, 0);
	return true;
}

void CFileInfo::ShowInfo(T_ITEM* item)
{
	if (!item) return;
	if (!hwnd) CreateDlg(0, IDD_FILEINFO, &mc);
	int i = 0;
	CString info;
	mcGetMediaInfo(item);
	SetText(IDC_EDIT1, (info = item->filename));
	SetText(IDC_EDIT3, (info = item->info.tags.title));
	SetText(IDC_EDIT4, (info = item->info.tags.artist));
	SetText(IDC_EDIT5, (info = item->info.tags.album));
	if (item->info.tags.year) {
		info.Format("%d", item->info.tags.year);
		SetText(IDC_EDIT6, info);
	}
	if (item->info.track) {
		info.Format("%d", item->info.track);
		SetText(IDC_EDIT7, info);
	} else if (item->info.tags.track) {
		info.Format("%d", item->info.tags.track);
		SetText(IDC_EDIT7, info);
	}
	if (item->info.chapter) {
		info.Format("%d", item->info.chapter);
		SetText(IDC_EDIT8, info);
	}
	SetText(IDC_EDIT9, (info = item->info.tags.genre));
	SetText(IDC_EDIT10, (info = item->info.tags.comment));
	if (item->info.filesize > 0) {
		info.Format("%u KB", (unsigned int)(item->info.filesize >> 10));
		SetText(IDC_EDIT13, info);
	}
	info.Format("%d ms", item->info.begin);
	SetText(IDC_EDIT11, info);
	if (item->info.end) {
		info.Format("%d ms", item->info.end - item->info.begin);
		SetText(IDC_EDIT12, info);
	} else if (item->info.duration) {
		info.Format("%d ms", item->info.duration);
		SetText(IDC_EDIT12, info);
	}
	SetText(IDC_EDIT14, (info = item->info.discroot));
	char *p = mcRunMediaTool(MT_MEDIAINFO, 0, item);
	if (p) {
		info = p;
		SetText(IDC_DETAIL, info);
	}
}

void GetFormatString(T_ITEM* item, CString& str)
{
	if (item->flags & IF_PRIVATE_FORMAT) {
		if (item->target.container != CF_DEFAULT) {
			str += mcGetToolName(TL_CONTAINER, 0, item->target.container);
			str += '/';
		}
		if (item->target.videoformat != VC_NONE) {
			str += mcGetToolName(TL_VIDEO_CODEC, 0, item->target.videoformat);
			str += '/';
		}
		if (item->target.audioformat != AC_NONE) {
			str += mcGetToolName(TL_AUDIO_ENCODER, 0, item->target.audioformat);
		}
		if (str.Right(1) == '/') {
			str.Delete(str.GetLength() - 1, 1);
		}
	} else {
		str += "<Use Global Settings>";
	}
}

bool CMenc::onCommand(int dlgid, int code)
{
	if (code == EN_CHANGE) {
		if (dlgid == IDC_OUTPUT_FOLDER) {
			char *p = mcGetPrefValueStr("overall.task.destdir");
			int len = SendDlgMsg(IDC_OUTPUT_FOLDER,WM_GETTEXT,256,(LPARAM)p);
			if (len == 0) {
				*p = 0;
			} else {
				p += len;
				if (*(p - 1) != '\\') *(WORD*)p = '\\';
			}
			return true;
		}
	}
	return false;
}

bool CMenc::onSelChange(int dlgid)
{
	switch (dlgid) {
	case IDC_MODE:
		mcSetPrefValueInt("overall.task.mode", SendDlgMsg(IDC_MODE, CB_GETCURSEL, 0, 0));
		break;
	default:
		return onClick(dlgid);
	}
	return true;
}

static int optMenuId[] = {IDM_OPT_DELETE_ORIGINAL, IDM_OPT_POWEROFF};
static char* optPrefId[] = {"overall.task.deleteOriginal", "overall.task.shutdown"};

void CMenc::Refresh()
{
	mcApplyParam();
	for (int i = 0; i < sizeof(optMenuId) / sizeof(int); i++) {
		SetMenuChecked(optMenuId[i], mcGetPrefValueInt(optPrefId[i]) != 0);
	}
	
	CString statestr;
	if (mcstate->state != STATE_ENCODING) {
		int container = mcGetPrefValueInt("overall.video.container");
		int video = mcGetPrefValueInt("overall.video.format");
		int audio = mcGetPrefValueInt("overall.audio.encoder");
		if (container == 0) {
			if (video == VC_NONE) {
				statestr.Format("Audio Only: %s", mcGetToolName(TL_AUDIO_ENCODER, 1, audio));
			} else {
				statestr.Format("Video: %s Audio: %s ",
					mcGetToolName(TL_VIDEO_CODEC, 0, video),
					mcGetToolName(TL_AUDIO_ENCODER, 0, audio)
				);
			}
		} else {
			if (video == VC_NONE) {
				statestr.Format("%s (%s)",
					mcGetToolName(TL_CONTAINER, 0, container),
					mcGetToolName(TL_AUDIO_ENCODER, 0, audio)
				);
			} else {
				statestr.Format("%s (%s + %s)",
					mcGetToolName(TL_CONTAINER, 0, container),
					mcGetToolName(TL_VIDEO_CODEC, 0, video),
					mcGetToolName(TL_AUDIO_ENCODER, 0, audio)
				);
			}
		}
		UpdateStatusBar(statestr.GetBuffer(),2,0);
	}
	UpdateTabbedView();
	SendDlgMsg(IDC_OUTPUT_FOLDER,WM_SETTEXT,0,(LPARAM)mcGetPrefValueStr("overall.task.destdir"));
	SendDlgMsg(IDC_MODE, CB_SETCURSEL, mcGetPrefValueInt("overall.task.mode"), 0);
}

void CMenc::UpdateTabbedView()
{
	if (mcGetPrefValueInt("overall.ui.mode") != UI_EXPERT) return;
	int view = mcGetPrefValueInt("overall.ui.expertView");
	TabCtrl_SetCurSel(GetHandle(IDC_TAB_VIEW), view);
	char* p = 0;
	switch (view) {
	case VIEW_LOG:
		SendMessage(hInfoBox, WM_SETTEXT, 0, 0);
		SendMessage(hInfoBox, EM_REPLACESEL, 0, (LPARAM)mcGetLogText(0));
		return;
	case VIEW_AUDIO:
		mcApplyParam();
		p = mcGetToolCommand(TL_AUDIO_ENCODER, 0);
		break;
	case VIEW_VIDEO: {
		CString s;
		mcApplyParam();
		for (int i = 0; i < 3; i++) {
			p = mcGetToolCommand(TL_VIDEO_CODEC, i);
			if (!p) break;
			if (i > 0) s += "\r\n";
			s += p;
			s += "\r\n";
		}
		SendMessage(hInfoBox,WM_SETTEXT,0,(LPARAM)((i == 0) ? 0 : s.GetBuffer()));
		return;
		}
	case VIEW_CONTAINER:
		p = mcGetToolCommand(TL_CONTAINER, 0);
		break;
	}
	SendMessage(hInfoBox,WM_SETTEXT,0,(LPARAM)p);
}

// mode: 0 - toggle, 1 - set private, 2 - set public
int CMenc::SetFormatMode(int mode)
{
	for (int i = list.GetCount() - 1; i>=0; i--) {
		T_ITEM* item;
		if (list.GetSelection(i)) {
			{
				item = mcGetItemByIndex(i);
			}
			if (item) {
				switch (mode) {
				case 0:
					if (item->flags & IF_PRIVATE_FORMAT)
						item->flags &= ~IF_PRIVATE_FORMAT;
					else
						item->flags |= IF_PRIVATE_FORMAT;
					break;
				case 1:
					item->flags |= IF_PRIVATE_FORMAT;
					break;
				case 2:
					item->flags &= ~IF_PRIVATE_FORMAT;
					break;
				}
				if (item->flags & IF_PRIVATE_FORMAT) {
					item->target.audioformat = (AUDIO_CODEC)mcGetPrefValueInt("overall.audio.encoder");
					item->target.videoformat = (VIDEO_CODEC)mcGetPrefValueInt("overall.video.format");
					item->target.container = (CONTAINER_FORMAT)mcGetPrefValueInt("overall.video.container");
					item->target.videoenc = (VIDEO_ENCODER)mcGetPrefValueInt("overall.video.encoder");
				} 
				CString str;
				GetFormatString(item, str);
				list.SetText(i, LC_FORMAT, str.GetBuffer());
			}
		}
	}
	
	return 0;
}

typedef struct {
	char* key;
	void* val;
} PREF_VAL;

void CMenc::RevertPref()
{
	PREF_VAL keptval[] = {
		{"overall.ui.showsetup"}
	};
	PREF_VAL keptvalstr[] = {
		{"overall.task.destdir"}
	};
	int i;
	for (i = 0; i < sizeof(keptval) / sizeof(PREF_VAL); i++) {
		keptval[i].val = mcGetPrefValue(keptval[i].key);
	}
	for (i = 0; i < sizeof(keptvalstr) / sizeof(PREF_VAL); i++) {
		keptvalstr[i].val = strdup(mcGetPrefValueStr(keptvalstr[i].key));
	}
	mcLoadParam(0, 1);
	for (i = 0; i < sizeof(keptval) / sizeof(PREF_VAL); i++) {
		mcSetPrefValue(keptval[i].key, keptval[i].val);
	}
	for (i = 0; i < sizeof(keptvalstr) / sizeof(PREF_VAL); i++) {
		mcSetPrefValueStr(keptvalstr[i].key, (char*)keptvalstr[i].val);
		free(keptvalstr[i].val);
	}
	SwitchUIMode();
	SetInitialPos();
	Refresh();
	RefreshAudioTab();
	RefreshOptionTab();
}

bool CMenc::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_PLAY:
	case IDM_PLAY: {
		T_ITEM* item = GetSelectedItem();
		if (item) {
			mcPlayerStart(item, 0, 0, 0);
		}
		} break;
	case IDM_PLAY_MPUI: {
		T_ITEM* item = GetSelectedItem();
		if (item) {
			ShellExecute(hwnd, "open", ".\\mplayer\\mpui.exe", item->filename, 0, SW_SHOW);
		}
		} break;
	case IDM_REMOVE_FINISHED:
		if (mcstate->state != STATE_ENCODING) {
			for (int i = list.GetCount() - 1; i>=0; i--) {
				T_ITEM* item = mcGetItemByIndex(i);
				if (!item) break;
				if (item->flags & IF_DONE) {
					if (mcRemoveItem(item) != -1) {
						list.RemoveItem(i);
						list.SetSelection(i, 1);
					}
				}
			}
			SetFocus(list.hwnd);
		}
		break;
	case IDM_REMOVE_ALL_ITEMS:
	case IDC_CLEAR:
		if (mcstate->state != STATE_ENCODING) {
			list.Reset();
			mcRemoveAllItems();
			RefreshOptionTab();
		}
		break;
	case IDM_REMOVE_ITEM:
	case IDC_REMOVE:
		if (mcstate->state != STATE_ENCODING) {
			for (int i = list.GetCount() - 1; i>=0; i--) {
				if (list.GetSelection(i)) {
					T_ITEM* item = mcGetItemByIndex(i);
					if (!item) break;
					if (mcRemoveItem(item) < 0) break;
					list.RemoveItem(i);
					list.SetSelection(i, 1);
				}
			}
			SetFocus(list.hwnd);
			RefreshOptionTab();
		}
		break;
	case IDM_SET_STATE_DONE: {
		T_ITEM* item = mcGetNextItem(0);
		for (int i = 0; item; item = mcGetNextItem(item), i++) {
			if (list.GetSelection(i)) {
				item->flags |= IF_DONE;
				list.SetText(i, LC_STATE, itemstate[IS_DONE]);
			}
		}
		} break;
	case IDM_SET_STATE_UNDONE: {
		T_ITEM* item = mcGetNextItem(0);
		for (int i = 0; item; item = mcGetNextItem(item), i++) {
			if (list.GetSelection(i)) {
				item->flags &= ~IF_DONE;
				list.SetText(i, LC_STATE, itemstate[IS_READY]);
			}
		}
		} break;
	case IDM_RESET_STATE: {
		T_ITEM* item = mcGetNextItem(0);
		for (int i = 0; item; item = mcGetNextItem(item), i++) {
			item->flags &= ~IF_DONE;
			list.SetText(i, LC_STATE, itemstate[IS_READY]);
		}
		} break;
	case IDC_ADDDIR:
	case IDM_ADDFOLDER:
		if (mcstate->state != STATE_ENCODING) {
			char buf[MAX_PATH];
			if (BrowseForPath(buf)) AddFolder(buf);
		}
		break;
	case IDC_SETOUTPUT:
	case IDM_SETOUTPUT:
		optionDialog[OD_OUTPUT]->onClick(IDC_BROWSE1);
		break;
	case IDC_OPEN_OUTPUT:
		ShellExecute(mc.hwnd, "open", mcGetPrefValueStr("overall.task.destdir"), 0, 0, SW_SHOW);
		break;
	case IDC_SETTEMP: {
		char* p = mcGetPrefValueStr("overall.task.tempdir");
		if (BrowseForPath(p))
			SendDlgMsg(IDC_TEMP_FOLDER,WM_SETTEXT,0,(LPARAM)p);
		} break;
	case IDC_ADDTRACK:
	case IDM_ADDTRACK:
		if (mcstate->state != STATE_ENCODING) {
			trackdlg.CreateDlg(0, IDD_ADDTRACK, hwnd);
		}
		break;
	case IDC_ADDFILE:
	case IDM_ADDFILE:
		if (mcstate->state != STATE_ENCODING) {
			OPENFILENAME ofn;
			int i;
			char buf[2048],*p,*q,*s;
			*buf=0;
			memset(&ofn, 0, sizeof(ofn));
			// generate file extension filter
			p = buf + 4;
			ofn.lpstrFilter = p;
			p += sprintf(p,"Media Files");
			*(p++)=0;
			FILE_TYPE* ft;
			for (i = 0; ; i++) {
				ft = mcGetFileTypeByIndex(i);
				if (!ft->fileext) break;
				p += sprintf(p,"*.%s;", ft->fileext);
			}
			*(p-1)=0;
			for (i = 0; ; i++) {
				ft = mcGetFileTypeByIndex(i);
				if (!ft->fileext) break;
				p += sprintf(p, "%s (%s)", ft->desc, ft->fileext);
				*(p++) = 0;
				p += sprintf(p,"*.%s", ft->fileext);
				*(p++) = 0;
			}
			memcpy(p,"All Files\0*.*\0",15);
			// show file dialog
			ofn.lpstrFile = buf;
			ofn.nMaxFile = sizeof(buf);
			ofn.Flags = OFN_ALLOWMULTISELECT;
			ofn.lpstrTitle="Add File";
			if (ShowFileDialog(0, &ofn)) break;
			// add selected files
			s=strchr(buf,0);
			if (*(WORD*)s) {
				p=s;
				do {
					for (q=++p; *q; q++);
					if (*(s-1)!='\\') *(s++)='\\';
					strcpy(s,p);
					T_ITEM* item = mcMakeItem(buf);
					AddItem(item);
					p=q;
				} while (*(WORD*)p);
			} else {
				T_ITEM* item = mcMakeItem(buf);
				AddItem(item);
			}
		}
		break;
	case IDM_ENCODEAUDIO:
		mcSetPrefValueInt("overall.video.format", VC_NONE);
		RefreshGui();
	case IDM_ENCODE:
	case IDC_TRANSCODE:
		switch (mcstate->state) {
		case STATE_ENCODING:
			mcSetState(STATE_STOPPING);
			break;
		case STATE_IDLE:
			StartTranscode();
			SetForeground();
			break;
		}
		break;
	case IDM_STOP_ENCODE:
		mcSetState(STATE_STOPPING);
		break;
	case IDC_SKIP:
		if (mcGetState()==STATE_ENCODING) mcSetState(STATE_SKIPPING);
		break;
	case IDM_PLAY_ENCODED:
		mcPlayerStart(GetSelectedItem(), 0, PF_PREVIEW, 0);
		break;
	case IDM_STOP_PLAY:
		mcPlayerStop();
		break;
	case IDM_CONSOLE: {
		int console = mcGetPrefValueInt("overall.ui.console");
		console = console ? 0 : 1;
		mcSetPrefValueInt("overall.ui.console", console);
		ShowConsole(console);
		SetForegroundWindow(hwnd);
		} break;
	case IDM_TOGGLE_STATS:
		if (wndStats.hwnd) {
			ShowStatsWindow(0);
		} else {
			ShowStatsWindow(1);
			if (mcGetState() == STATE_ENCODING) UpdateTaskStats();
		}
		break;
	case IDM_CLEARLOG:
		mcSetLogText("");
		UpdateTabbedView();
		break;
	case IDM_AUDIO_ENCODER_VER: {
		int id = mcGetPrefValueInt("overall.audio.encoder");
		char *ver = mcGetToolVersion(TL_AUDIO_ENCODER, id);
		if (ver) {
			MsgBox(ver, "Audio encoder version");
		} else {
			CString str;
			str.Format(IDS_NO_ENCODER_VER, mcGetToolName(TL_AUDIO_ENCODER, 1, id));
			MsgBox(str.GetBuffer());
		}
		} break;
	case IDM_VIDEO_ENCODER_VER: {
		int id = mcGetPrefValueInt("overall.video.encoder");
		char *ver = mcGetToolVersion(TL_VIDEO_ENCODER, id);
		if (ver) {
			MsgBox(ver, "Video encoder version");
		} else {
			CString str;
			str.Format(IDS_NO_ENCODER_VER, mcGetToolName(TL_VIDEO_ENCODER, 1, id));
			MsgBox(str.GetBuffer());
		}
		} break;
	case IDM_MUXER_VER: {
		int id = mcGetPrefValueInt("overall.video.container");
		char *ver = mcGetToolVersion(TL_CONTAINER, id);
		if (ver) {
			MsgBox(ver, "Container muxer version");
		} else {
			CString str;
			str.Format(IDS_NO_ENCODER_VER, mcGetToolName(TL_CONTAINER, 1, id));
			MsgBox(str.GetBuffer());
		}
		} break;
	case IDM_INFO_ITEM:
		fileinfo.ShowInfo(GetSelectedItem());
		break;
	case IDM_INFO_CONVERTED: {
		T_ITEM* item = GetSelectedItem();
		if (item->destfile) {
			item = mcMakeItem(item->destfile);
			fileinfo.ShowInfo(item);
			mcFreeItem(item);
		}
		} break;
	case IDM_ADDURL: {
		CDlgUrl dlg;
		dlg.ShowDlg(hInst, IDD_ADDURL, this);
		} break;
	case IDM_CROPPER:
		mcGetMediaInfo(GetSelectedItem());
		ShowCropper(this);
		break;
	case IDM_EXIT:
		onClose();
		break;
	case IDC_PREF:
	case IDM_PREFERENCE:
		ShowWeb(mcShowConfig(CD_PREF, 0));
		break;
	case IDM_CFG_XUL:
		ShowWeb("about:config");
		break;
	case IDM_SETUP_WIZARD:
		ShowWeb(mcShowConfig(CD_SETUP, 0));
		break;
	case IDM_SWITCH_AUDIO:
		mcPlayerCommand("switch_audio");
		break;
	case IDM_LEFT_CHANNEL:
		mcPlayerCommand("switch_audio 1");
		break;
	case IDM_RIGHT_CHANNEL:
		mcPlayerCommand("switch_audio 2");
		break;
	case IDM_STEREO:
		mcPlayerCommand("switch_audio 0");
		break;
	case IDM_LEFT25_RIGHT75:
		mcPlayerCommand("switch_audio 5");
		break;
	case IDM_LEFT50_RIGHT50:
		mcPlayerCommand("switch_audio 4");
		break;
	case IDM_LEFT75_RIGHT25:
		mcPlayerCommand("switch_audio 3");
		break;
	case IDM_PAUSE:
		mcPlayerCommand("pause");
		break;
	case IDM_STOP:
		mcPlayerCommand("quit");
		break;
	case IDM_INC_VOL:
		mcPlayerCommand("volume 1");
		break;
	case IDM_DEC_VOL:
		mcPlayerCommand("volume -1");
		break;
	case IDM_MUTE:
		mcPlayerCommand("mute");
		break;
	case IDM_FULLSCREEN:
		mcPlayerCommand("vo_fullscreen");
		break;
	case IDM_ONTOP:
		mcPlayerCommand("vo_ontop");
		break;
	case IDM_WA_CONFIG:
		ShowWeb(mcShowConfig(CD_DSP_PLUGIN, 0));
		break;
	case IDM_REVERT_ALL:
		RevertPref();
		break;
	case IDM_SAVEASDEFAULT:
		mcSaveParam(CFGFILE);
		UpdateStatusBar("Saved as default",0,0);
		break;
	case IDM_LOADQUEUE: {
		char buf[MAX_PATH];
		OPENFILENAME ofn;
		memset(&ofn, 0, sizeof(ofn));
		buf[0]=0;
		ofn.lpstrFile = buf;
		ofn.nMaxFile = sizeof(buf);
		ofn.lpstrFilter = "M3U List\0*.m3u\0Text File List\0*.lst;*.txt\0";
		ofn.lpstrTitle = "Load a list";
		if (!ShowFileDialog(0,&ofn) && LoadQueue(buf) > 0) {
			UpdateStatusBar("Queue loaded",0,0);
		}
		} break;
	case IDM_SAVEQUEUE: {
		char buf[MAX_PATH];
		OPENFILENAME ofn;
		memset(&ofn, 0, sizeof(ofn));
		buf[0] = 0;
		ofn.lpstrFile = buf;
		ofn.nMaxFile = sizeof(buf);
		ofn.lpstrFilter = "M3U List\0*.m3u\0";
		ofn.lpstrTitle = "Save a list";
		if (ShowFileDialog(1,&ofn)) break;
		if (!strstr(buf, ".m3u")) strcat(buf,".m3u");
		if (SaveQueue(buf) > 0) {
			UpdateStatusBar("Queue saved",0,0);
		}
		} break;
	case IDM_LOAD_PRESET: {
		char buf[MAX_PATH];
		OPENFILENAME ofn;
		memset(&ofn, 0, sizeof(ofn));
		buf[0]=0;
		ofn.lpstrFile = buf;
		ofn.nMaxFile = sizeof(buf);
		ofn.lpstrFilter = "Presets\0*.xml\0";
		ofn.lpstrTitle = "Load a Preset";
        ofn.lpstrInitialDir = buf + 16;
		sprintf((char*)ofn.lpstrInitialDir, "%spresets", apppath);
		if (!ShowFileDialog(0,&ofn) && !mcLoadParam(buf, 1)) {
			Refresh();
			RefreshOptionTab();
			RefreshAudioTab();
			UpdateStatusBar("Preset loaded",0,0);
		}
		} break;
	case IDM_SAVE_PRESET: {
		char buf[MAX_PATH];
		OPENFILENAME ofn;
		memset(&ofn, 0, sizeof(ofn));
		buf[0] = 0;
		ofn.lpstrFile = buf;
		ofn.nMaxFile = sizeof(buf);
		ofn.lpstrFilter = "Presets\0*.xml\0";
		ofn.lpstrTitle = "Save a Preset";
        ofn.lpstrInitialDir = buf + 16;
		sprintf((char*)ofn.lpstrInitialDir, "%spresets", apppath);
		if (ShowFileDialog(1,&ofn)) break;
		if (!strstr(buf, ".xml")) strcat(buf,".xml");
		if (!mcSaveParam(buf)) {
			UpdateStatusBar("Preset saved",0,0);
		}
		} break;
	case IDM_SET_FORMAT_PUBLIC:
		SetFormatMode(2);
		break;
	case IDM_SET_FORMAT_PRIVATE:
		SetFormatMode(1);
		break;
	case IDM_TOGGLE_FORMAT:
		SetFormatMode(0);
		break;
	case IDM_VERSION: {
		MsgBox(version);
		} break;
	case IDM_VISIT_WEBSITE:
		VisitWebsite(0);
		break;
	case IDM_LINK_FORUM:
		ShellExecute(hwnd, "open", "http://mediacoder.sourceforge.net/forum/", 0, 0, SW_SHOW);
		break;
	case IDM_LINK_WIKI:
		ShellExecute(hwnd, "open", "http://mediacoder.sourceforge.net/wiki/", 0, 0, SW_SHOW);
		break;
	case IDM_LINK_DONATE:
		ShellExecute(hwnd, "open", "http://mediacoder.sourceforge.net/donors.htm", 0, 0, SW_SHOW);
		break;
	case IDM_SPLASH:
		splash.Open(0, this);
		break;
#ifdef SPECIAL_REL
	case IDM_CHECKFORUPDATE:
		MsgBox("This function is not available in special release.");
		break;
	case IDC_DEVICE:
		device.Open(DEV_TYPE_UNSPEC);
		break;
#else
	case IDM_CHECKFORUPDATE: {
		DWORD dwid;
		if (hThreadCheckUpdate) {
			DWORD code;
			if (GetExitCodeThread(hThreadCheckUpdate, &code)) {
				if (code == STILL_ACTIVE)
					TerminateThread(hThreadCheckUpdate, 0);
				CloseHandle(hThreadCheckUpdate);
			}
		}
		hThreadCheckUpdate = CreateThread(0, 0, (LPTHREAD_START_ROUTINE)CheckForUpdate, 0, 0, &dwid);
		} break;
	case IDM_VIDEO_DISC:
		device.Open(DEV_TYPE_DISCPLAYER);
		break;
	case IDM_MOBILE:
		device.Open(DEV_TYPE_PHONE);
		break;
	case IDM_PSP:
		device.Open(DEV_TYPE_GAMECONSOLE);
		break;
	case IDM_DEV_DMP:
		device.Open(DEV_TYPE_DMP);
		break;
	case IDM_DEVICE_PDA:
		device.Open(DEV_TYPE_PDA);
		break;
	case IDM_DEVICE_OTHER:
		device.Open(DEV_TYPE_UNSPEC);
		break;
	case IDC_DEVICE: {
		HMENU hmenu;            // top-level menu 
		HMENU hmenuTrackPopup;  // shortcut menu 
		RECT rc;

		// Load the menu resource. 
	 
		if ((hmenu = LoadMenu(hInst, MAKEINTRESOURCE(IDR_MENU))) == NULL) 
			break; 

		GetWindowRect(GetHandle(IDC_DEVICE), &rc);
		// TrackPopupMenu cannot display the menu bar so get 
		// a handle to the first shortcut menu. 
	 
		hmenuTrackPopup = GetSubMenu(hmenu, 5); 
	 
		// Display the shortcut menu. Track the right mouse 
		// button. 
	 
		TrackPopupMenu(hmenuTrackPopup, 
				TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, 
				rc.left, rc.bottom, 0, hwnd, NULL); 
	 
		// Destroy the menu. 
		DestroyMenu(hmenu); 
		} break;
	case IDM_MODE_DEVICE:
		mcSetPrefValueInt("overall.ui.mode", UI_DEVICE);
		SwitchUIMode();
		break;
	case IDM_MODE_NORMAL:
		mcSetPrefValueInt("overall.ui.mode", UI_NORMAL);
		SwitchUIMode();
		break;
	case IDM_MODE_EXPERT:
		mcSetPrefValueInt("overall.ui.mode", UI_EXPERT);
		SwitchUIMode();
		break;
#endif
	case IDM_MODE_SIMPLE:
		mcSetPrefValueInt("overall.ui.mode", UI_SIMPLE);
		SwitchUIMode();
		break;
	default:
		for (int i = 0; i < sizeof(optMenuId) / sizeof(int); i++) {
			if (optMenuId[i] == dlgid) {
				bool checked = IsMenuChecked(dlgid);
				mcSetPrefValueInt(optPrefId[i], checked ? 0 : 1);
				SetMenuChecked(dlgid, !checked);
				break;
			}
		}
		break;
	}
	return true;
}

int CMenc::Init()
{
	HWND hwndTab;
	TCITEM tie;
	int i;
	RECT rc;
	static char* tabTitle[] = {"Logger", "Console", "Audio Options", "Video Options", "Container Options", 0};
	hwndTab=GetHandle(IDC_TAB_VIEW);
    tie.mask = TCIF_TEXT | TCIF_IMAGE; 
    tie.iImage = -1; 
	for (i = 0; tabTitle[i]; i++) {
		tie.pszText = tabTitle[i]; 
		TabCtrl_InsertItem(hwndTab, i, &tie);
	}
	GetClientRect(audiotab.hwnd, &rc);
	for (i = 0; i < AC_COUNT; i++) {
		COptionDialog* ad;
		tie.pszText = mcGetToolName(TL_AUDIO_ENCODER, 0, i);
		TabCtrl_InsertItem(audiotab.hwnd, i, &tie);
		switch (i) {
		case AC_LAMEMP3:	ad = new CDlgLameMP3;		break;
		case AC_3GPPAAC:	ad = new CDlg3GPP;			break;
		case AC_AMR:		ad = new CDlgAMR;			break;
		case AC_AACPLUS:	ad = new CDlgAACPlus;		break;
		case AC_HELIX:		ad = new CDlgHelix;			break;
		case AC_FAAC:		ad = new CDlgFaac;			break;
		case AC_VORBIS:		ad = new CDlgVorbis;		break;
		case AC_WAVPACK:	ad = new CDlgWavPack;		break;
		case AC_FLAC:		ad = new CDlgFlac;			break;
		case AC_APE:		ad = new CDlgApe;			break;
		case AC_CLI:		ad = new CDlgGeneric;		break;
		case AC_WMA:		ad = new CDlgWma;			break;
		case AC_HMP3:		ad = new CDlgHelixMP3;		break;
		case AC_FFMPEG:		ad = new CDlgFFMPEG;		break;
		case AC_ALS:		ad = new CDlgAls;			break;
		case AC_PCM:		ad = new CDlgPcm;			break;
		case AC_NEROREF:	ad = new CDlgNeroRefEnc;	break;
		default:
			if (!dlgnull) dlgnull = new CDlgNull();
			ad = dlgnull;
			break;
		}
		ad->CreateDlg(hInstance, ad->dialogid, &audiotab);
		if (ad->hwnd) {
			int width, height;
			ad->GetWindowPos(0, 0, &width, &height);
			ad->Move((rc.right - rc.left - width) / 2, 15 + (rc.bottom - rc.top - height) / 2);
		}
		AudioDialogs[i] = ad;
	}
	ShowAudioDialog(mcGetPrefValueInt("overall.audio.encoder"));

	GetClientRect(opttab.hwnd, &rc);
	for (i = 0; i < OD_COUNT; i++) {
		COptionDialog* od;
		switch (i) {
		case OD_VIDEO:		od = new CDlgVideoOption;	break;
		case OD_AUDIO:		od = new CDlgAudio;			break;
		case OD_OUTPUT:		od = new CDlgOutput;		break;
		case OD_CONTAINER:	od = new CDlgContainer;		break;
		case OD_PICTURE:	od = new CDlgPicture;		break;
		case OD_SUMMARY:	od = new CDlgSummary;		break;
		case OD_TIME:		od = new CDlgTime;			break;
		case OD_TAGGING:	od = new CDlgTagging;		break;
		case OD_PREVIEW:	od = new CDlgPreview;		break;
		}
		od->CreateDlg(hInstance, od->dialogid, &opttab);
		tie.pszText = od->GetName(); 
		TabCtrl_InsertItem(opttab.hwnd, i, &tie);
		int width, height;
		od->GetWindowPos(0, 0, &width, &height);
		od->Move((rc.right - rc.left - width) / 2, 15 + (rc.bottom - rc.top - height) / 2);
		optionDialog[i] = od;
	}
	return 0;
}

void CMenc::ShowState(int state)
{
	char* s = "Idle";

	switch (state) {
	case STATE_ENCODING:
		switch (mcstate->stage) {
		case ST_PROCESSING_AUDIO:
			s = "Encoding audio";
			break;
		case ST_PROCESSING_VIDEO:
			s = "Encoding video";
			break;
		case ST_MUXING:
			s = "Merging";
			break;
		}
		break;
	case STATE_PLAYING:
		s = "Playing";
		break;
	case STATE_SHUTDOWN:
		s = "Shutting down";
		break;
	}
	UpdateStatusBar(s,0,0);
}

bool CMenc::onCreate()
{
	static COL_INFO cols[]={{"File Name", 150}, {"File Size", 56}, {"File Type", 125}, {"State", 50}, {"Target Format", 120}, {"Full Path", 250}, {0}};
	static const int bmp[]={IDB_ADDFILE, IDB_ADDFOLDER, IDB_ADDTRACK, IDB_REMOVE, IDB_CLEAR, IDB_PLAY, IDB_SKIP, IDB_PREF, IDB_START, IDB_DEVICE};
	static const int buttons[]={IDC_ADDFILE, IDC_ADDDIR, IDC_ADDTRACK, IDC_REMOVE, IDC_CLEAR, IDC_PLAY, IDC_SKIP, IDC_PREF, IDC_TRANSCODE, IDC_DEVICE, 0};

	opttab.hwnd = GetHandle(IDC_TAB);
	audiotab.hwnd = GetHandle(IDC_TAB_AUDIO);
	SetClassLongPtr(hwnd,GCL_HICON,(LONG)hAppIcon); 

	ListEnums("overall.task.mode",GetHandle(IDC_MODE),CB_ADDSTRING);

	for (int i = 0; buttons[i]; i++)
		SendDlgMsg(buttons[i],BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)(LoadBitmap(hInst,MAKEINTRESOURCE(bmp[i]))));
	
	hBmpPlay[0] = LoadBitmap(hInst,MAKEINTRESOURCE(IDB_PLAY));
	hBmpPlay[1] = LoadBitmap(hInst,MAKEINTRESOURCE(IDB_STOPPLAY));
	hBmpStart[0] = LoadBitmap(hInst,MAKEINTRESOURCE(IDB_START));
	hBmpStart[1] = LoadBitmap(hInst,MAKEINTRESOURCE(IDB_STOP));

	list.hwnd = GetHandle(IDC_FILELIST);
	list.SetExtendedStype(LVS_EX_FULLROWSELECT);
	list.AddColumns(cols);
	SendDlgMsg(IDC_PROGRESS,PBM_SETRANGE,true,MAKELONG(0,100));
	RegisterDropWindow(hwnd);
	hInfoBox = GetHandle(IDC_INFO);
	SetParent(hInfoBox, GetHandle(IDC_TAB_VIEW));
	hWndStatusbar=CreateSBar(hwnd,NULL,3,300);
	ShowState(mcstate->state);

	SetCaption(caption);

#ifdef SPECIAL_REL
	//HideCtrl(IDC_DEVICE);
	HideCtrl(IDC_LABEL_OUTPUT);
	HideCtrl(IDC_OUTPUT_FOLDER);
	HideCtrl(IDC_SETOUTPUT);
	HideCtrl(IDC_OPEN_OUTPUT);

	HMENU hMenu = GetMenu(hwnd);
	DeleteMenu(hMenu, 4, MF_BYPOSITION);
	DeleteMenu(hMenu, 4, MF_BYPOSITION);

#endif
	return true;
}

int CMenc::LoadQueue(char* filename)
{
	char *p, *nl;
	T_ITEM itemdata;
	char *text = 0;
	char *alias;
	int count = 0;
	bool m3u = false;
	if (LoadTextFile(filename, &text, 0) <= 0) return -1;
	p = text;
	if (!strncmp(p, "#EXTM3U", 7)) {
		m3u = true;
		p += 7;
	}
	memset(&itemdata, 0, sizeof(T_ITEM));
	for (; p; p = nl) {
		while (*(p++) != '\n');
		nl = strchr(p, '\r');
		if (!nl) nl = strchr(p, '\n');
		if (nl) *nl = 0;
		if (!*p) continue;
		if (m3u && *p == '#') {
			p++;
			if (!strncmp(p, "EXTINF:", 7)) {
				p += 7;
				itemdata.info.duration = atoi(p);
				p = strchr(p, ',');
				if (!p) continue;
				alias = ++p;
			} else if (!strncmp(p, "EXTINF_INFO:", 12)) {
				p += 12;
				itemdata.info.filesize = atoi(p);
				if ((p = strchr(p, ','))) itemdata.flags = atoi(++p);
			} else if (!strncmp(p, "EXTINF_TIME:", 12)) {
				p += 12;
				itemdata.info.track = atoi(p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.info.chapter = atoi(++p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.info.begin = atoi(++p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.info.end = atoi(++p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.info.audiodelay = atoi(++p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.info.discroot = mcStrDup(++p);
			} else if (!strncmp(p, "EXTINF_FMT:", 11)) {
				p += 11;
				itemdata.flags |= IF_PRIVATE_FORMAT;
				itemdata.target.container = (CONTAINER_FORMAT)atoi(p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.target.audioformat = (AUDIO_CODEC)atoi(++p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.target.videoformat = (VIDEO_CODEC)atoi(++p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.target.videoenc = (VIDEO_ENCODER)atoi(++p);
			} else if (!strncmp(p, "EXTINF_SUB:", 11)) {
				p += 11;
				if (*p) mcSetItemStr(&itemdata.subfile, p);
			} else if (!strncmp(p, "EXTINF_CROP:", 12)) {
				p += 12;
				itemdata.target.crop.x = atoi(p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.target.crop.y = atoi(++p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.target.crop.w = atoi(++p);
				if (!(p = strchr(p, ','))) continue;
				itemdata.target.crop.h = atoi(++p);
			}
			continue;
		}
		T_ITEM* item = mcMakeItem(p);
		memcpy(&item->info, &itemdata.info, sizeof(MEDIA_INFO));
		memcpy(&item->target, &itemdata.target, sizeof(TARGET_SETTINGS));
		memcpy(&item->target.crop, &itemdata.target.crop, sizeof(CROP_SETTINGS));
		item->subfile = itemdata.subfile;
		item->info.filetype = mcIdentifyFileType(item->filename);
		item->flags = itemdata.flags;
		AddItem(item, alias);
		memset(&itemdata, 0, sizeof(T_ITEM));
		count++;
	}
	free(text);
	return count;
}

int CMenc::SaveQueue(char* filename)
{
	int count = list.GetCount();
	if (count>0) {
		FILE *fp=fopen(filename,"w");
		if (fp>0) {
			int i = 0;
			fprintf(fp, "#EXTM3U\n");
			for (T_ITEM* item = mcGetNextItem(0); item; item = mcGetNextItem(item), i++) {
				char buf[MAX_PATH];
				list.GetText(i, LC_FILENAME, buf, sizeof(buf));
				fprintf(fp,"#EXTINF:%d,%s\n", item->info.duration, buf);
				fprintf(fp,"#EXTINF_INFO:%d,%d\n", (unsigned)item->info.filesize, item->flags & ~IF_INFO_RETRIEVED);
				fprintf(fp,"#EXTINF_TIME:%d,%d,%d,%d,%d",
					item->info.track, item->info.chapter, item->info.begin, item->info.end, item->info.audiodelay);
				if (item->info.discroot)
					fprintf(fp,",%s",item->info.discroot);
				fprintf(fp, "\n");
				if (item->flags & IF_PRIVATE_FORMAT) {
					fprintf(fp,"#EXTINF_FMT:%d,%d,%d,%d\n",
						item->target.container, item->target.audioformat, item->target.videoenc, item->target.videoformat);
				}
				if (item->target.crop.x | item->target.crop.y | item->target.crop.w | item->target.crop.h) {
					fprintf(fp,"#EXTINF_CROP:%d,%d,%d,%d\n",
						item->target.crop.x, item->target.crop.y, item->target.crop.w, item->target.crop.h);
				}
				if (item->subfile) {
					fprintf(fp,"#EXTINF_SUB:%s\n", item->subfile);
				}
				fprintf(fp,"%s\n", item->filename);
			}
			fclose(fp);
		}
	}
	return count;
}

bool CMenc::onClose()
{
	if (mcGetState() == STATE_ENCODING) {
		if (Ask(IDS_EXIT_CONFIRM, MB_YESNO) == IDNO) return true;
	}
	int x, y, w, h;
	char *p = mcGetPrefValueStr("overall.ui.param");
	mcSetState(STATE_SHUTDOWN);
	ShowWindow(hwnd, SW_RESTORE);
	GetWindowPos(&x, &y, &w, &h);
	sprintf(p, "%d,%d,%d,%d", w, h, x, y);
	Hide();
	mcPlayerStop();
	ShowConsole(0);
	mcSetPrefValueStr("overall.ui.lastLoadedPlugin", device.pluginFile);
	device.Close();
	DestroyBrowserInstance(&web);
	WaitForSingleObject(hThread,3000);
	// save encoding list
	DeleteFile(LISTFILE);
	if (mcGetPrefValueInt("overall.generic.saveQueue")) {
		SaveQueue(LISTFILE);
	}
	UnregisterDropWindow(hwnd);
	GlobalUninit();
	Destroy();
	PostQuitMessage(0);
	return true;
}

void CMenc::ShowConsole(int fShow)
{
	char buf[512];
	if (fShow) {
		if (fConsole || !AllocConsole()) return;
#ifndef CONFIG_WIN9X
		SetConsoleTitle(CONSOLE_CAPTION);
		MoveWindow(GetConsoleWindow(),0,0,700,540,true);
#endif
		sprintf(buf, "%s\n\nProcessor information:\n%s\n\n", version, mcstate->cpucaps.cpuFullName);
		EVENT(MC_MSG_INFO, buf);
		fConsole=true;
	} else if (fConsole) {
		FreeConsole();
		fConsole=false;
	}
}

int CMenc::AddMultipleFiles(char* filenames)
{
	int count = 0;
	char *p = filenames;
	do {
		char *s = p;
		if (*p == '\"') {
			s++;
			for (p++; *p && *p!= '\"'; p++);
			if (*p) {
				*(p++) = 0;
			}
		} else {
			for (; *p && *p!= ' '; p++);
		}
		T_ITEM* item = mcMakeItem(s);
		AddItem(item);
		count++;
	} while (*(p++) == ' ');
	return count;
}

int CMenc::AddItem(T_ITEM* item, char* aliasstr)
{
	CString alias = aliasstr;
	DWORD attr = GetFileAttributes(item->filename);
	if (attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY)) {
		AddFolder(item->filename);
		mcFreeItem(item);
		return 0;
	}

	DWORD filesize = 0;
	int count = 0;
	int type = item->info.filetype->type;
	CCueSheet* cue = 0;
	int track = 1;
	int trackcount = 1;
	CString discroot;

	if (type == CT_CUE) {
		int ret;
		cue = new CCueSheet;
		ret = cue->LoadCueSheet(item->filename);
		cue->ParseCueSheet();
		discroot = item->filename;
		mcFreeItem(item);
		if (ret) return -1;
		trackcount = cue->trackcount;
	}
	do {
		if (type == CT_CUE) {
			item = mcMakeItem(cue->file);
			cue->FillCueSheetItem(track, item);
			item->info.discroot = mcStrDup(discroot.GetBuffer());
			if (item->info.tags.valid) {
				if (item->info.tags.artist) {
					alias.Format("%d. %s - %s", track, item->info.tags.artist, item->info.tags.title);
				} else {
					alias = item->info.tags.title;
				}
			} else {
				CFileName fn;
				fn = cue->file;
				alias.Format("Track %d of ", track);
				fn.GetName(alias);
			}
		}
		if (alias.IsEmpty()) {
			CFileName fn;
			fn = item->filename;
			fn.GetName(alias);
		}
		if (!mcAddItem(item)) {
			char* items[LC_COUNT + 1];
			CString format;
			char buf[64], *p = buf;
			items[LC_FILENAME] = alias.GetBuffer();
			if (!item->info.filesize)
				item->info.filesize = GetFileBytes(item->filename);
			if (item->info.filesize >= 1024 * 1024 * 10) {
				items[LC_SIZE] = p;
				p += sprintf(p, "%u MB", (unsigned int)(item->info.filesize >> 20));
			} else if (item->info.filesize > 0) {
				items[LC_SIZE] = p;
				p += sprintf(p, "%u KB", (unsigned int)(item->info.filesize >> 10));
			} else {
				items[LC_SIZE] = "";
			}
			items[LC_TYPE] = item->info.filetype ? item->info.filetype->desc : "";
			items[LC_STATE] = itemstate[item->flags & IF_DONE ? 1 : 0];
			items[LC_FULLPATH] = item->filename;
			GetFormatString(item, format);
			items[LC_FORMAT] = format.GetBuffer();
			items[LC_COUNT] = 0;
			list.AddItem(items);
		}
	} while (++track <= trackcount);
	if (cue) delete cue;
	return 0;
}

void CMenc::UpdateStatusBar(LPSTR lpszStatusString, WORD partNumber, WORD displayFlags)
{
    SendMessage(hWndStatusbar,SB_SETTEXT,partNumber | displayFlags,(LPARAM)lpszStatusString);
}

bool CMenc::onMove()
{
	RECT rc;
	GetWindowRect(hwnd, &rc);
	if (wndStats.hwnd) {
		int sh;
		wndStats.GetWindowPos(0, 0, 0, &sh);
		wndStats.Move(rc.right, rc.bottom - sh);
	}
	if (device.hwnd && (device.flags & DEV_FLAG_DOCK)) {
		int w;
		device.GetWindowPos(0, 0, &w, 0);
		device.Move(rc.left - w, rc.top);
	}
	return true;
}

bool CMenc::onSize(int width, int height)
{
	RECT rc;
	int mode = mcGetPrefValueInt("overall.ui.mode");
	int h, w;
	int vpos;
	int progressBarWidth = width - MARGIN * 2 - PROGRESS_TEXT_WIDTH + 4;
	HWND hCtrl;

	MoveWindow(GetHandle(IDC_SEPERATOR), 0, TOOLBAR_HEIGHT + 2, width + 1, 2, TRUE);
	// resize status bar
	GetWindowRect(hWndStatusbar, &rc);
	h = rc.bottom - rc.top;
	MoveWindow(hWndStatusbar, 0, height - (rc.bottom - rc.top), width, h, TRUE);

	height -= h;
	h = -MARGIN;
	vpos = height;
	switch (mode) {
	case UI_EXPERT:
		h = TAB_VIEW_HEIGHT;
		// resize info tab
		hCtrl = GetHandle(IDC_TAB_VIEW);
		GetWindowRect(hCtrl, &rc);
		MoveWindow(hInfoBox, 2, 22, width - MARGIN * 2 - 4, INFO_BOX_HEIGHT, FALSE);
		MoveWindow(hCtrl, MARGIN, height - h - MARGIN, width - MARGIN * 2, h, TRUE);
		progressBarWidth -= MODE_OPTION_WIDTH + MARGIN;
	case UI_NORMAL:
		w = OPT_TAB_WIDTH * 2 + MARGIN * 3;
		if (width > w) w = width;
		vpos -= h + OPT_TAB_HEIGHT + MARGIN * 2;
		opttab.Move((w / 2 - OPT_TAB_WIDTH) / 2 + 4, vpos, OPT_TAB_WIDTH, OPT_TAB_HEIGHT);
		audiotab.Move((w + w / 2 - OPT_TAB_WIDTH) / 2, vpos, OPT_TAB_WIDTH, OPT_TAB_HEIGHT);
		break;
	case UI_DEVICE:
		vpos = height - MARGIN - NAV_BUTTON_HEIGHT;
		devnav.Move(MARGIN, vpos, width - MARGIN * 2, NAV_BUTTON_HEIGHT);
	case UI_SIMPLE:
		vpos -= MARGIN;
		break;
	}
	
	// resize progress bars
	vpos -= MARGIN + PROGRESS_BAR_HEIGHT - 7;

	MoveWindow(GetHandle(IDC_STATIC_TEXT2), MARGIN + 2, vpos, PROGRESS_TEXT_WIDTH - 10, 20, TRUE);
	MoveWindow(GetHandle(IDC_PROGRESSALL), MARGIN + PROGRESS_TEXT_WIDTH - 4, --vpos, progressBarWidth, PROGRESS_BAR_HEIGHT, TRUE);

	if (mode == UI_EXPERT) {
		MoveWindow(GetHandle(IDC_MODE), width - MODE_OPTION_WIDTH - MARGIN, vpos - 3, MODE_OPTION_WIDTH, 80, TRUE);
		vpos -= PROGRESS_BAR_HEIGHT + 3;
		MoveWindow(GetHandle(IDC_MODE_LABEL), width - MODE_OPTION_WIDTH - MARGIN + 1, vpos + 2, MODE_OPTION_WIDTH, 40, TRUE);
	} else {
		vpos -= PROGRESS_BAR_HEIGHT + 3;
	}

	MoveWindow(GetHandle(IDC_STATIC_TEXT1), MARGIN + 2, vpos, PROGRESS_TEXT_WIDTH - 10, 20, TRUE);
	MoveWindow(GetHandle(IDC_PROGRESS), MARGIN + PROGRESS_TEXT_WIDTH - 4, --vpos, progressBarWidth, PROGRESS_BAR_HEIGHT, TRUE);
	// resize list view
	list.Move(MARGIN, TOOLBAR_HEIGHT + MARGIN, width - MARGIN * 2, vpos - MARGIN * 2 - TOOLBAR_HEIGHT);
	UpdateWindow(hwnd);
	onMove();
	if (mode == UI_DEVICE) {
		devnav.Show();
	}
	return true;
}

int GetFileProgress()
{
	switch (mcstate->stage) {
	case ST_PROCESSING_AUDIO:
		return (int)mcstate->astats.percent;
	case ST_PROCESSING_VIDEO:
		return (int)mcstate->vstats.percent;
	default:
		return 0;
	}
}

static int ShowStats()
{
	char buf[128];
	int percent = 0;
	switch (mcstate->stage) {
	case ST_PROCESSING_AUDIO: {
		int length = mcstate->astats.length / 1000;
		int pos = mcstate->astats.pos / 1000;
		percent = (int)mcstate->astats.percent;
		if (mc.GetShowState() == SW_SHOWMINIMIZED) {
			if (length) {
				sprintf(buf, "%d%% (%d/%d)", percent, pos, length);
				mc.UpdateTitle(buf);
			}
		} else {
			if (length) {
				if (mcstate->stage == ST_EXTRACTING_PCM) {
					sprintf(buf,"extracting PCM (%3.1f%% done)", mcstate->astats.percent);					
				} else {
					sprintf(buf,"transcoding audio (%3.1f%% done)", mcstate->astats.percent);
				}
				mc.UpdateTitle(buf);
			}
			HWND hwndStats = wndStats.hwnd;
			if (hwndStats) {
				sprintf(buf,"%ds",pos);
				SendDlgItemMessage(hwndStats,IDC_STAT_POS,WM_SETTEXT,0,(LPARAM)buf);
				sprintf(buf,"%dK", (int)(mcstate->astats.bytes>>10));
				SendDlgItemMessage(hwndStats,IDC_STAT_FRAME,WM_SETTEXT,0,(LPARAM)buf);
				if (mcstate->astats.speed > 0) {
					sprintf(buf,"%d.%dX", mcstate->astats.speed / 10, mcstate->astats.speed % 10);
					SendDlgItemMessage(hwndStats,IDC_STAT_FPS,WM_SETTEXT,0,(LPARAM)buf);
				}
				int n;
				if (n = mcstate->astats.bitrate) {
					sprintf(buf, "%dKbps", n);
					SendDlgItemMessage(hwndStats,IDC_STAT_AB,WM_SETTEXT,0,(LPARAM)buf);
				}
				if (n = mcstate->astats.estsize) {
					n /= 100;
					sprintf(buf, "%d.%dMB", n / 10, n % 10);
					SendDlgItemMessage(hwndStats,IDC_STAT_ESTSIZE,WM_SETTEXT,0,(LPARAM)buf);
				}
			}
		}
		UpdateTime();
		} break;
	case ST_PROCESSING_VIDEO:
		percent = (int)mcstate->vstats.percent;
		if (mc.GetShowState() == SW_SHOWMINIMIZED) {
			char buf[32];
			sprintf(buf,"%4.1f%% done (pass %d of %d)",
				mcstate->vstats.percent,
				mcstate->currentpass,
				mcstate->totalpass);
			mc.SetCaption(buf);
		} else {
			UpdateVideoStats();
		}
		UpdateTime();
		break;
	default:
		return -1;
	}
	if (percent >= 0 && percent <= 100) {
		char buf[16];
		sprintf(buf, "%d%%", percent);
		mc.list.SetText(iQueueItemIndex, LC_STATE, buf);
		mc.SendDlgMsg(IDC_PROGRESS, PBM_SETPOS, percent, 0);
	}
	return 0;
}

CMenc::CMenc()
{
	CString str;
	iOptionDialog=-1;
	str.Format("%s %d.%d.%d",APPNAME,VER_MAJOR,VER_MINOR,VER_REV,BUILD_NO);
#ifdef CONFIG_WIN9X
	str += " (Win9x)";
#endif
	caption=strdup(str.GetBuffer());
	str.Format("%s Version %d.%d.%d build %d (built on %s)\n",
		APPNAME, VER_MAJOR, VER_MINOR, VER_REV, BUILD_NO, BUILD_DATE);
#ifndef STANDALONE
	str.AppendFormat("Based on %s\n", mcGetVersionInfo());
#endif
	str += "This is a free software written by Stanley Huang <reachme@citiz.net>. Enjoy!";
	version=strdup(str.GetBuffer());
}

int GetAppPath(char *buf,int bufsize)
{
	char *p;
	p=buf+GetModuleFileName(GetModuleHandle(NULL),buf,bufsize);
	while(p>buf && *(--p)!='\\');
	*(++p)=0;
	return p-buf;
}

void DropInCallback(DROPFILES *dropfiles)
{
	WORD* unames = (WORD*)((char*)dropfiles + dropfiles->pFiles);
	char* anames;
	int len, i;
	for (i = 0, len = 0; ; i++) {
		if (!unames[i]) {
            anames = (char*)malloc(len*2);
			memset(anames, 0, len*2);
			WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)(unames + i - len), len, anames, len*2, 0, 0);
			T_ITEM* item = mcMakeItem(anames);
			mc.AddItem(item);
			free(anames);
			len = 0;
			if (!unames[i+1]) break;
		} else {
			len++;
		}
	}
}

int GlobalInit()
{
	{
		WSADATA wsaData;
		WSAStartup( MAKEWORD( 1, 1 ), &wsaData );
	}
	GetAppPath(apppath,sizeof(apppath));
	SetCurrentDirectory(apppath);

	mcstate = mcInit(&EventHandler);
	if (!mcstate) {
		mc.MsgBox(mcGetLastError());
		return -1;
	}

	mcLoadParam(CFGFILE, 0);

#ifndef SPECIAL_REL
	if (mcGetPrefValueInt("overall.ui.showsetup") < mcGetBuildNo()) {
		ShowWeb(mcShowConfig(CD_SETUP, (void*)1));
		ShowWeb(mcShowConfig(CD_SETUP, (void*)2));
		if (mcGetPrefValueInt("overall.ui.showsetup") < mcGetBuildNo()) {
			Sleep(1000);
			return -1;
		}
		mcSetPrefValueStr("overall.ui.param", 0);
	}
#endif

	hAppIcon = LoadIcon(hInst,MAKEINTRESOURCE(IDI_MAIN));
	return 0;
}

bool CMenc::onShow(bool fShow)
{
	return true;
}

void GlobalUninit()
{
	mcSaveParam(CFGFILE);
	mcUninit();
	WSACleanup();
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, INT nCmdShow)
{
	MSG msg;
	struct HACCEL__ * hAccelTable=LoadAccelerators(hInstance,MAKEINTRESOURCE(IDR_ACCEL));
	hInst = hInstance;

	if (GlobalInit()) {
		return -1;
	}

	if (mcGetPrefValueInt("overall.ui.splash"))
		splash.Open(SPLASH_SHOW_TIME, 0);

#ifndef SPECIAL_REL
	if (mcGetPrefValueInt("overall.generic.checkUpdate")) {
		DWORD dwid;
		hThreadCheckUpdate = CreateThread(0, 0, (LPTHREAD_START_ROUTINE)CheckForUpdate, (void*)1, 0, &dwid);
	}
#endif

	if (!mc.CreateDlg(0, IDD_MAIN, (HWND)0) || mc.Init()) return 0;

	if (*lpCmdLine) {
		mc.AddMultipleFiles(lpCmdLine);
	} else if (mcGetPrefValueInt("overall.generic.saveQueue")) {
		mc.LoadQueue(LISTFILE);
	}
	mcPostInit(mc.hwnd);

	// show CPU speed
	{
	CString str = mcstate->cpucaps.vendor;
	if (mcstate->cpucaps.freq > 1000) {
		int spd = (mcstate->cpucaps.freq + 100) / 100;
		str.AppendFormat(" @ %d.%d Ghz", spd / 10, spd % 10);
	} else {
		str.AppendFormat(" @ %d Mhz", mcstate->cpucaps.freq);
	}
	mc.UpdateStatusBar(str.GetBuffer(),1,0);
	}

	mc.SwitchUIMode();
	mc.SetInitialPos();
	// show up console window
	if (mcGetPrefValueInt("overall.ui.console"))
		mc.ShowConsole(1);

	mc.ShowSubDialog(0);
	mc.Show();

	char *pluginFile = mcGetPrefValueStr("overall.ui.lastLoadedPlugin");
	if (*pluginFile) {
		device.Open(pluginFile);
	} else {
		mc.SetForeground();
	}


	while (GetMessage(&msg,0,0,0)) {
		if (!TranslateAccelerator(mc.hwnd,hAccelTable,&msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	return msg.wParam;
}
