/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#include "config.h"
#include <commctrl.h>
#include "atlstr.h"
#include "resource.h"
#include "common.h"
#include "menc.h"
#include "optiondialogs.h"
#include "audiodialogs.h"
#include "deviceplugins.h"

void RefreshGui();
void ListEnums(char* key, HWND hCtrl, UINT msg);
void RefreshOptionTab();

extern CMenc mc;
extern CDeviceDialog device;

static char* langcode[] = {"en", "zh", "dut", "fin", "fre", "ger", "ita", "jpn", "kor", "nor", "por", "rus", "spa", "swe"};
static char* langname[] = {"English", "Chinese", "Dutch", "Finnish", "French", "German", "Italian", "Japanese", "Korean", "Norwegian", "Portuguese", "Russian", "Castilian", "Swedish"};

void AddLanguages(HWND hCtrl, UINT msg)
{
	for (int i = 0; i < sizeof(langname) / sizeof(char*); i++) {
		char buf[64];
		sprintf(buf, "%s (%s)", langcode[i], langname[i]);
		SendMessage(hCtrl, msg, 0, (LPARAM)buf);
	}
}

#define PT_DEFAULT 0
#define PT_INT 1
#define PT_STRING 2
#define PT_ENUM 3
#define PT_BOOL 4
#define PT_FLOAT 5
#define PT_PTR 6
#define PT_FILE 7
#define PT_DIR 8
#define PT_SEPERATOR 0xe
#define PT_CATALOG 0xf

static PREF_CTRL nullctrl = {0};

COptionDialog::COptionDialog()
{
	ctrls = &nullctrl;
	dialogid = 0;
	refreshing = 0;
}

bool COptionDialog::onCreate()
{
	for (int i = 0; ctrls[i].ctrlid; i++) {
		int type;
		char wndclass[64];
		if (!ctrls[i].prefkey) continue;
		GetClassName(GetHandle(ctrls[i].ctrlid), wndclass, sizeof(wndclass));
		mcGetPrefProperty(ctrls[i].prefkey, PP_TYPE, &type);
		if (!strcmp(wndclass, "ComboBox")) {
			switch (type) {
			case PT_ENUM:
				ListEnums(ctrls[i].prefkey, GetHandle(ctrls[i].ctrlid), CB_ADDSTRING);
				break;
			}
			ctrls[i].ctrltype = CTRL_COMBO;
		} else if (!strcmp(wndclass, "msctls_trackbar32")) {
			int range[2];
			mcGetPrefProperty(ctrls[i].prefkey, PP_RANGE, range);
			SendDlgMsg(ctrls[i].ctrlid,TBM_SETRANGE,TRUE,MAKELPARAM(range[0],range[1]));
			ctrls[i].ctrltype = CTRL_SLIDER;
		} else if (!strcmp(wndclass, "Button")) {
			DWORD style = GetWindowLong(GetHandle(ctrls[i].ctrlid), GWL_STYLE);
			if ((style & BS_RADIOBUTTON) == BS_RADIOBUTTON || (style & BS_AUTORADIOBUTTON) == BS_AUTORADIOBUTTON) {
				ctrls[i].ctrltype = CTRL_RADIO;
			} else if ((style & BS_CHECKBOX) == BS_CHECKBOX || (style & BS_AUTOCHECKBOX) == BS_AUTOCHECKBOX) {
				ctrls[i].ctrltype = CTRL_CHECK;
			} else {
				ctrls[i].ctrltype = CTRL_BUTTON;
			}
		} else if (!strcmp(wndclass, "TextBox")) {
			ctrls[i].ctrltype = CTRL_TEXT;
		}
	}
	return onCreate2();
}

void COptionDialog::RefreshDialog(int activectrl)
{
	int dlgid;
	refreshing = 1;
	Refresh();
	for (int i = 0; (dlgid = ctrls[i].ctrlid); i++) {
		if (dlgid == activectrl || !ctrls[i].prefkey) continue;
		int type;
		mcGetPrefProperty(ctrls[i].prefkey, PP_TYPE, &type);
		switch (type) {
		case PT_STRING:
		case PT_DIR:
		case PT_FILE:
			SendDlgMsg(dlgid, WM_SETTEXT, 0, (LPARAM)mcGetPrefValueStr(ctrls[i].prefkey));
			continue;
		}
		switch (ctrls[i].ctrltype) {
		case CTRL_SLIDER:
			SendDlgMsg(dlgid,TBM_SETPOS,TRUE,mcGetPrefValueInt(ctrls[i].prefkey));
			break;
		case CTRL_TEXT:
			SetTextInt(dlgid, mcGetPrefValueInt(ctrls[i].prefkey));
			break;
		case CTRL_COMBO:
			switch (type) {
			case PT_INT:
				SetTextInt(dlgid, mcGetPrefValueInt(ctrls[i].prefkey));
				break;
			case PT_ENUM:
				SendDlgMsg(dlgid, CB_SETCURSEL, mcGetPrefValueInt(ctrls[i].prefkey), 0);
				break;
			}
			break;
		case CTRL_CHECK:
			SetItemChecked(dlgid, mcGetPrefValueInt(ctrls[i].prefkey) != 0);
			break;
		case CTRL_RADIO:
			SetItemChecked(dlgid, mcGetPrefValueInt(ctrls[i].prefkey) == ctrls[i].index);
			break;
		}
	}
	mcApplyParam();
	RefreshGui();
	refreshing = 0;
}

bool COptionDialog::onScroll(int dlgid, int code)
{
	if (refreshing)
		return false;
	for (int i = 0; ctrls[i].ctrlid; i++) {
		if (dlgid != ctrls[i].ctrlid || !ctrls[i].prefkey) continue;
		mcSetPrefValueInt(ctrls[i].prefkey, SendDlgMsg(dlgid,TBM_GETPOS,0,0));
		RefreshDialog(dlgid);
		return true;
	}
	onScroll2(dlgid, code);
	return false;
}

bool COptionDialog::onCommand(int dlgid, int code)
{
	if (refreshing)
		return false;
	for (int i = 0; ctrls[i].ctrlid; i++) {
		if (dlgid != ctrls[i].ctrlid || !ctrls[i].prefkey) continue;
		int type;
		mcGetPrefProperty(ctrls[i].prefkey, PP_TYPE, &type);
		switch (code) {
		case EN_CHANGE:
			switch (type) {
			case PT_STRING:
			case PT_DIR:
			case PT_FILE:
				SendDlgMsg(dlgid, WM_GETTEXT, mcGetPrefValueStrMax(ctrls[i].prefkey), (LPARAM)mcGetPrefValueStr(ctrls[i].prefkey));
				break;
			case PT_INT: {
				int v = GetTextInt(dlgid);
				int range[2];
				mcGetPrefProperty(ctrls[i].prefkey, PP_RANGE, range);
				if (v >= range[0] && (v <= range[1] || !range[1])) {
					mcSetPrefValueInt(ctrls[i].prefkey, v);
				}
				} break;
			}
			break;
		case CBN_SELCHANGE:
			switch (type) {
			case PT_ENUM:
				mcSetPrefValueInt(ctrls[i].prefkey, SendDlgMsg(dlgid, CB_GETCURSEL, 0, 0));
				break;
			case PT_INT:
				mcSetPrefValueInt(ctrls[i].prefkey, GetTextInt(dlgid));
				break;
			case PT_FLOAT:
				mcSetPrefValueFloat(ctrls[i].prefkey, (float)GetTextFloat(dlgid));
				break;
			}
			break;
		case BN_CLICKED:
			switch (ctrls[i].ctrltype) {
			case CTRL_CHECK:
                mcSetPrefValueInt(ctrls[i].prefkey, IsItemChecked(dlgid) ? 1 : 0);
				break;
			case CTRL_RADIO:
				mcSetPrefValueInt(ctrls[i].prefkey, ctrls[i].index);
				break;
			}
			break;
		default:
			return false;
		}
		RefreshDialog(dlgid);
		return true;
	}
	return onCommand2(dlgid, code);
}

CDlgVideoOption::CDlgVideoOption()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_VIDEO_BITRATE, "overall.video.bitrate"},
		{IDC_SLIDER_VIDEO, "overall.video.bitrate"},
		{IDC_ENCODER, "overall.video.encoder"},
		{IDC_CODECS, "overall.video.format"},
		{IDC_ADJUST, "overall.video.bitrateAdjust"},
		{IDC_OPT_FIXRATE, "overall.video.mode", 0},
		{IDC_OPT_FIXQUANT, "overall.video.mode", 1},
		{IDC_OPT_TWOPASS, "overall.video.mode", 2},
		{IDC_OPT_THREEPASS, "overall.video.mode", 3},
		{0}
	};
	ctrls = s_ctrls;
	dialogid = IDD_VIDEO;
}

void CDlgVideoOption::Refresh()
{
	int mode = mcGetPrefValueInt("overall.video.mode");
	int adjust = mcGetPrefValueInt("overall.video.bitrateAdjust");
	EnableCtrl(IDC_OPT_FIXRATE, adjust != 1);
	EnableCtrl(IDC_OPT_FIXQUANT, adjust == 0);
	EnableCtrl(IDC_SLIDER_VIDEO, adjust <= 1);
	if (adjust == 1 && mode < 2)
		mode = 2;
	else if (adjust == 2 && mode == 1)
		mode = 0;

	if (adjust == 2) {
		T_ITEM* item = GetSelectedItem();
		SetText(IDC_SLIDER_LABEL, "Desired video stream size");
		SetText(IDC_KBPS, "KB");
		ctrls[0].prefkey = 0;
		ctrls[1].prefkey = 0;
		SendDlgMsg(IDC_SLIDER_VIDEO,TBM_SETRANGE,0,0);
		SendDlgMsg(IDC_SLIDER_VIDEO,TBM_SETPOS,0,0);
		if (!item) {
			SetText(IDC_VIDEO_BITRATE, 0);
			DisableCtrl(IDC_VIDEO_BITRATE);
		} else {
			SetTextInt(IDC_VIDEO_BITRATE, item->target.destsize);
			EnableCtrl(IDC_VIDEO_BITRATE);
		}
	} else if (adjust == 3) {
		T_ITEM* item = GetSelectedItem();
		SetText(IDC_SLIDER_LABEL, "Desired compression ratio");
		SetText(IDC_KBPS, ": 1");
		ctrls[0].prefkey = 0;
		ctrls[1].prefkey = 0;
		SendDlgMsg(IDC_SLIDER_VIDEO,TBM_SETRANGE,0,0);
		SendDlgMsg(IDC_SLIDER_VIDEO,TBM_SETPOS,0,0);
		if (!item) {
			SetText(IDC_VIDEO_BITRATE, 0);
			DisableCtrl(IDC_VIDEO_BITRATE);
		} else {
			char buf[16];
			sprintf(buf, "%.3f", item->target.ratio);
			SetText(IDC_VIDEO_BITRATE, buf);
			EnableCtrl(IDC_VIDEO_BITRATE);
		}
	} else {
		int range[2];
		if (mode == 1) {
			SetText(IDC_SLIDER_LABEL, "Quality");
			SetText(IDC_KBPS, 0);
			ctrls[0].prefkey = "overall.video.quality";
		} else {
			SetText(IDC_SLIDER_LABEL, "Bitrate");
			SetText(IDC_KBPS, "Kbps");
			ctrls[0].prefkey = "overall.video.bitrate";
		}
		ctrls[1].prefkey = ctrls[0].prefkey;
		mcGetPrefProperty(ctrls[0].prefkey, PP_RANGE, range);
		SendDlgMsg(IDC_SLIDER_VIDEO,TBM_SETRANGE,0,MAKELPARAM(range[0],range[1]));
		SendDlgMsg(IDC_SLIDER_VIDEO,TBM_SETPAGESIZE,0,mode == 1 ? 10 : 256);
		mcSetPrefValueInt("overall.video.mode", mode);
	}
}

bool CDlgVideoOption::onCommand2(int dlgid, int code)
{
	if (code == EN_CHANGE && dlgid == IDC_VIDEO_BITRATE) {
		T_ITEM* item = GetSelectedItem();
		int adjust = mcGetPrefValueInt("overall.video.bitrateAdjust");
		if (item) {
			switch (adjust) {
			case 2:
				item->target.destsize = GetTextInt(dlgid);
				break;
			case 3:
				item->target.ratio = (float)GetTextFloat(dlgid);
				break;
			}
		}
		return true;
	}
	return false;
}

CDlgContainer::CDlgContainer()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_FORMAT, "overall.video.container"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_CONTAINER;
}

int CDlgContainer::WrapFiles(char* destfile)
{
	int i, count;
	char buf[MAX_PATH];

	count=SendDlgMsg(IDC_LIST,LB_GETCOUNT,0,0);
	for (i=0; i<count; i++) {
		SendDlgMsg(IDC_LIST,LB_GETTEXT,i,(LPARAM)buf);
		FILE_TYPE* ft = mcIdentifyFileType(buf);
		mcContainerAddFile(ft->type, buf, DUMMY_FLAG);
	}
	/*
	if (destfile)
		return mcContainerMuxFiles(0, destfile);
	else
	*/
		return 0;
}

bool CDlgContainer::onClick(int dlgid)
{
	char buf[MAX_PATH];
	OPENFILENAME ofn;

	switch (dlgid) {
	case IDC_ADD:
		strcpy(buf,"*.*");
		memset(&ofn, 0, sizeof(ofn));
		ofn.lpstrFile = buf;
		ofn.nMaxFile = sizeof(buf);
		ofn.lpstrFilter = "All Files\0*.*\0";
		ofn.lpstrTitle = "Add File";
		if (!ShowFileDialog(0,&ofn)) {
			SendDlgMsg(IDC_LIST,LB_ADDSTRING,0,(LPARAM)buf);
		}
		break;
	case IDC_REMOVE:
		SendDlgMsg(IDC_LIST,LB_DELETESTRING,SendDlgMsg(IDC_LIST,LB_GETCURSEL,0,0),0);
		break;
	case IDC_WRAP:
		buf[0]=0;
		memset(&ofn, 0, sizeof(ofn));
		ofn.lpstrFile = buf;
		ofn.nMaxFile = sizeof(buf);
		ofn.lpstrFilter = "All Files\0*.*\0";
		ofn.lpstrTitle = "Set Target File";
		if (!ShowFileDialog(1, &ofn)) {
			WrapFiles(buf);
		}
		break;
	case IDC_OPTION:
		ShowWeb(mcShowConfig(CD_CONTAINER, 0));
		break;
	default:
		return false;
	}
	return true;
}

static int srate[] = {8000, 11025, 16000, 22050, 32000, 44100, 48000};
static int volgain[] = {-200, -60, -40, -20, -10, -6, -3, -1, 0, 1, 3, 6, 10, 20, 40, 60};
#define GAIN_OPT_COUNT 15

CDlgAudio::CDlgAudio()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_DECODER, "overall.audio.source"},
		{IDC_ENCODER, "overall.audio.encoder"},
		{IDC_DOWNMIX, "audiofilter.internal.downmix"},
		{IDC_NORMALIZE, "audiofilter.volume.normalization"},
		{IDC_AID, "overall.decoding.aid"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid =IDD_AUDIO;
}

bool CDlgAudio::onCreate2()
{
	char buf[16];
	int i;
	SendDlgMsg(IDC_RESAMPLE, CB_ADDSTRING, 0, (LPARAM)"Original");
	for (i = 0; i < sizeof(srate) / sizeof(int); i++) {
		sprintf(buf, "%d Hz", srate[i]);
		SendDlgMsg(IDC_RESAMPLE, CB_ADDSTRING, 0, (LPARAM)buf);
	}
	for (i = 0; i < GAIN_OPT_COUNT; i++) {
		sprintf(buf, "%d dB", volgain[i]);
		SendDlgMsg(IDC_VOLUME, CB_ADDSTRING, 0, (LPARAM)buf);
	}
	AddLanguages(GetHandle(IDC_LANGUAGE), CB_ADDSTRING);
	return true;
}

bool CDlgAudio::onCommand2(int dlgid, int code)
{
	char *p;
	switch (code) {
	case CBN_EDITUPDATE:
		switch (dlgid) {
		case IDC_LANGUAGE:
			p = mcGetPrefValueStr("overall.decoding.alang");
			SendDlgMsg(IDC_LANGUAGE, WM_GETTEXT, 3, (LPARAM)p);
			p = strchr(p, ' ');
			if (p) *p = 0;
			break;
		}
		break;
	default:
		return false;
	}
	return true;
}

bool CDlgAudio::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_BUTTON1:
		ShowWeb(mcShowConfig(CD_AUDIO_SRC, 0));
		break;
	case IDC_BUTTON2:
		ShowWeb(mcShowConfig(CD_AUDIO_ENC, 0));
		break;
	default:
		return false;
	}
	return true;
}

bool CDlgAudio::onSelChange(int dlgid)
{
	int i;
	switch (dlgid) {
	case IDC_RESAMPLE:
		i = SendDlgMsg(IDC_RESAMPLE,CB_GETCURSEL,0,0);
		mcSetPrefValueInt("audiofilter.resample.samplerate", i == 0 ? 0 : srate[i - 1]);
		break;
	case IDC_VOLUME:
		mcSetPrefValueFloat("audiofilter.volume.gain", (float)volgain[SendDlgMsg(IDC_VOLUME,CB_GETCURSEL,0,0)]);
		break;
	case IDC_LANGUAGE:
		i = SendDlgMsg(IDC_LANGUAGE,CB_GETCURSEL,0,0);
		if (i > 0) {
			strcpy(mcGetPrefValueStr("overall.decoding.aid"), langcode[i]);
		}
		break;
	default:
		return false;
	}
	Refresh();
	RefreshGui();
	return true;
}

void CDlgAudio::Refresh()
{
	int n = mcGetPrefValueInt("audiofilter.resample.samplerate");
	if (n) {
		int i;
		for (i = 0; srate[i] && srate[i] < n; i++);
		if (srate[i]) SendDlgMsg(IDC_RESAMPLE,CB_SETCURSEL,i + 1,0);	
	} else {
		SendDlgMsg(IDC_RESAMPLE,CB_SETCURSEL,0,0);
	}
	
	float gain = mcGetPrefValueFloat("audiofilter.volume.gain");
	SendDlgMsg(IDC_VOLUME, CB_SETCURSEL, 0, 0);
	for (int i = GAIN_OPT_COUNT - 1; i > 0; i--) {
		if ((volgain[i - 1] < gain && gain <= volgain[i])) {
			SendDlgMsg(IDC_VOLUME, CB_SETCURSEL, i, 0);
			break;
		}
	}
	EnableCtrl(IDC_VOLUME, !mcGetPrefValueInt("audiofilter.volume.normalization"));
}

void CDlgSummary::Refresh()
{
	CString text;
	int i;
	int abitrate = 0;
	int vbitrate = 0;
	T_ITEM* item = GetSelectedItem(-1);
	TVINSERTSTRUCT tvis;
	char buf[256];

	HWND hTree=GetHandle(IDC_TREE);
	ShowWindow(hTree, SW_HIDE);
	TreeView_DeleteAllItems(hTree);
	memset(&tvis, 0, sizeof(tvis));
	tvis.item.mask=TVIF_TEXT|TVIF_STATE;
	tvis.hParent = TVI_ROOT;
	tvis.hInsertAfter = TVI_LAST;
	tvis.item.pszText = buf;
	tvis.item.state = TVIS_EXPANDED;
	tvis.item.stateMask = TVIS_EXPANDED;

	if (item && !mcGetMediaInfo(item)) {
		char *p = strrchr(item->filename, '\\');
		sprintf(buf, "Properties of %s", p ? p + 1 : item->filename);
		tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
		if (item->info.container) {
			_snprintf(buf, sizeof(buf), "Container: %s", item->info.container);
			tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
			if (item->info.duration) {
				_snprintf(buf, sizeof(buf), "Duration: %d.%d sec(s)", item->info.duration / 1000, (item->info.duration / 100) % 10);
				TreeView_InsertItem(hTree,(LPARAM)&tvis);
			}
			tvis.hParent = TreeView_GetParent(hTree, tvis.hParent);
		}
		if (item->info.video.width && item->info.video.height) {
			_snprintf(buf, sizeof(buf), "Video: %s", item->info.video.codec ? item->info.video.codec : (item->info.container ? item->info.container : ""));
			tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
			if (item->info.video.bitrate) {
				_snprintf(buf, sizeof(buf), "Bitrate: %d Kbps", item->info.video.bitrate / 1000);
				TreeView_InsertItem(hTree,(LPARAM)&tvis);
			}
			if (item->info.video.width && item->info.video.height) {
				_snprintf(buf, sizeof(buf), "Resolution: %dx%d", item->info.video.width, item->info.video.height);
				TreeView_InsertItem(hTree,(LPARAM)&tvis);
			}
			float fps = (float)GetFPS(item->info.video.fps, item->info.video.fpsScale);
			if (fps) {
				_snprintf(buf, sizeof(buf), "Frame Rate: %.2f fps", fps);
				TreeView_InsertItem(hTree,(LPARAM)&tvis);
			}
			tvis.hParent = TreeView_GetParent(hTree, tvis.hParent);
		}
		if (item->info.audio.codec || item->info.audio.samplerate || item->info.audio.bitrate) {
			_snprintf(buf, sizeof(buf), "Audio: %s", item->info.audio.codec ? item->info.audio.codec : "");
			tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
			if (item->info.audio.bitrate) {
				_snprintf(buf, sizeof(buf), "Bitrate: %d Kbps", item->info.audio.bitrate / 1000);
				TreeView_InsertItem(hTree,(LPARAM)&tvis);
			}
			if (item->info.audio.bitratemode) {
				_snprintf(buf, sizeof(buf), "Bitrate Mode: %s", item->info.audio.bitrate == 1 ? "CBR" : "VBR");
				TreeView_InsertItem(hTree,(LPARAM)&tvis);
			}
			if (item->info.audio.samplerate) {
				_snprintf(buf, sizeof(buf), "Sample Rate: %d Hz", item->info.audio.samplerate);
				TreeView_InsertItem(hTree,(LPARAM)&tvis);
			}
			if (item->info.audio.channels) {
				_snprintf(buf, sizeof(buf), "Channel(s): %d", item->info.audio.channels);
				TreeView_InsertItem(hTree,(LPARAM)&tvis);
			}
		}
	} else {
		strcpy(buf, "(Select a file to see its properties)");
		TreeView_InsertItem(hTree,(LPARAM)&tvis);
	}
	tvis.hParent = TVI_ROOT;

	strcpy(buf, "Target Format");
	tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
	int audio = mcGetPrefValueInt("overall.audio.encoder");
	int video = mcGetPrefValueInt("overall.video.format");
	int container = mcGetPrefValueInt("overall.video.container");

	if (container) {
		_snprintf(buf, sizeof(buf), "Container: %s", mcGetToolName(TL_CONTAINER, 0, container));
		TreeView_InsertItem(hTree,(LPARAM)&tvis);
	}
	if (video != VC_NONE) {
		_snprintf(buf, sizeof(buf), "Video: %s", mcGetToolName(TL_VIDEO_CODEC, 0, video));
		tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
		if (mcGetPrefValueInt("overall.video.mode") == 1) {
			_snprintf(buf, sizeof(buf), "Quality: %d", mcGetPrefValueInt("overall.video.quality"));
		} else {
			vbitrate = mcGetPrefValueInt("overall.video.bitrate");
			_snprintf(buf, sizeof(buf), "Bit Rate: %d Kbps", vbitrate);
		}
		TreeView_InsertItem(hTree,(LPARAM)&tvis);
		if (mcGetPrefValueInt("videofilter.scale.enabled")) {
			_snprintf(buf, sizeof(buf), "Resolution: %dx%d",
				mcGetPrefValueInt("videofilter.scale.width"),
				mcGetPrefValueInt("videofilter.scale.height"));
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		float fps = (float)GetFPS(mcGetPrefValueInt("overall.video.fps"), mcGetPrefValueInt("overall.video.fpsScale"));
		if (fps) {
			sprintf(buf, "Frame Rate: %.2f fps", fps);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		tvis.hParent = TreeView_GetParent(hTree, tvis.hParent);
	}
	if (audio != AC_NONE) {
		_snprintf(buf, sizeof(buf), "Audio: %s", mcGetToolName(TL_AUDIO_ENCODER, 1, audio));
		tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
		if ((abitrate = mcGetTargetParam(TP_AUDIO_BITRATE))) {
			_snprintf(buf, sizeof(buf), "Bit Rate: %d Kbps", abitrate);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		if ((i = mcGetPrefValueInt("audiofilter.resample.samplerate"))) {
			_snprintf(buf, sizeof(buf), "Sample Rate: %d Hz", i);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		tvis.hParent = TreeView_GetParent(hTree, tvis.hParent);
	}
	if (item && item->info.duration) {
		int bitrate = 0;
		if (video != VC_NONE && item->info.video.width && item->info.video.height) bitrate += vbitrate;
		bitrate += abitrate;
		int estsize = (item->info.duration >> 10 ) * bitrate / 8;
		if (estsize > 0) {
			if (estsize >= 10240) {
				_snprintf(buf, sizeof(buf), "Estimated File Size: %d MB", estsize >> 10);
			} else {
				_snprintf(buf, sizeof(buf), "Estimated File Size: %d KB", estsize);
			}
			tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
			tvis.hParent = TreeView_GetParent(hTree, tvis.hParent);
		}
	}
	
	int startpos = mcGetPrefValueInt("overall.decoding.startTime");
	int duration = mcGetPrefValueInt("overall.decoding.duration");
	if (startpos || duration) {
		strcpy(buf, "Positioning");
		tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
		_snprintf(buf, sizeof(buf), "Start Offset: %d sec(s)", startpos);
		TreeView_InsertItem(hTree,(LPARAM)&tvis);
		if (i) {
			_snprintf(buf, sizeof(buf), "Length: %d sec(s)", duration);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
	}
	ShowWindow(hTree, SW_SHOW);
}

bool CDlgPicture::SaveChange(int dlgid, char* value)
{
	char *p;
	switch (dlgid) {
	case IDC_FPS: {
		int fps, fpsScale;
		GetFPSScale(atof(value), &fps, &fpsScale);
		mcSetPrefValueInt("overall.video.fps", fps);
		mcSetPrefValueInt("overall.video.fpsScale", fpsScale);
		} break;
	case IDC_SCALE_PRESET:
		p = strchr(value, 'x');
		if (!p) p = strchr(value, 'X');
		if (!p) p = strchr(value, ':');
		if (p && *(p + 1)) {
			mcSetPrefValueInt("videofilter.scale.width", atoi(value));
			mcSetPrefValueInt("videofilter.scale.height", atoi(p + 1));
		}
		break;
	case IDC_ASPECT:
		p = strchr(value, ':');
		if (p) *p = '/';
		mcSetPrefValueStr("overall.video.aspect", value);
		break;
	case IDC_CROP_PARAM: {
		T_ITEM* item = GetSelectedItem();
		if (item) {
			item->target.crop.x = atoi(value);
			if (!(p = strchr(value, ':'))) break;
			item->target.crop.y = atoi(++p);
			if (!(p = strchr(p, ':'))) break;
			item->target.crop.w = atoi(++p);
			if (!(p = strchr(p, ':'))) break;
			item->target.crop.h = atoi(++p);
		}
		} break;
	default:
		return false;
	}
	return true;
}

bool CDlgPicture::onSelChange(int dlgid)
{
	char *s = ComboGetString(dlgid);
	if (!s) return false;
	switch (dlgid) {
	case IDC_FPS:
		if (ComboGetSel(IDC_FPS) == 0) {
			mcSetPrefValueInt("overall.video.fps", 0);
		} else {
			SaveChange(IDC_FPS, s);
		}
		break;
	case IDC_SCALE_PRESET:
		SaveChange(IDC_SCALE_PRESET, s);
		break;
	case IDC_ASPECT:
		SaveChange(IDC_ASPECT, s);
		break;
	}
	free(s);
	Refresh();
	return true;
}

bool CDlgPicture::onCommand2(int dlgid, int code)
{
	char *p;
	switch (code) {
	case CBN_EDITUPDATE:
	case EN_CHANGE:
		p = GetText(dlgid);
		if (p) {
			SaveChange(dlgid, p);
			free(p);
			RefreshGui();
		}
		break;
	default:
		return false;
	}
	return true;
}

CDlgPicture::CDlgPicture()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_ROTATE, "videofilter.rotate.mode"},
		{IDC_DENOISE, "videofilter.denoise.mode"},
		{IDC_PP, "videofilter.postproc.mode"},
		{IDC_DEINT, "videofilter.postproc.deint"},
		{IDC_TDN, "videofilter.postproc.denoiser"},
		{IDC_DO_CROP, "videofilter.crop.enabled"},
		{IDC_DO_SCALE, "videofilter.scale.enabled"},
		{IDC_DO_ROTATE, "videofilter.rotate.enabled"},
		{0}
	};
	ctrls = s_ctrls;
	dialogid =IDD_PICTURE;
}

bool CDlgPicture::onCreate2()
{
	static float fps[]= {8, 10, 12, 12.5f, 14.985f, 15, 20, 23.976f, 24, 25, 29.97f, 30};
	static char* aspect = "4:3\0 16:9\0 5:4\0 3:2\0 2:1\0";
	static char* res = "768x576\0 720x576\0 720x480\0 640x480\0 624x352\0 608x336\0 512x384\0 480x272\0 352x576\0 352x480\0 352x288\0 352x240\0 320x240\0 320x160\0 208x176\0 176x208\0 176x144\0 160x128\0 160x96\0 128x96\0 128x64";

	SendDlgMsg(IDC_FPS, CB_ADDSTRING, 0, (LPARAM)"Original");
	for (int i = 0; i < sizeof(fps) / sizeof(float); i++) {
		char buf[8];
		sprintf(buf, "%.3f", fps[i]);
        SendDlgMsg(IDC_FPS, CB_ADDSTRING, 0, (LPARAM)buf);
	}
	ComboAddStrings(IDC_SCALE_PRESET, res);
	ComboAddStrings(IDC_ASPECT, aspect);
	return true;
}

void CDlgPicture::Refresh()
{
	int i;
	char buf[64];
	T_ITEM* item = GetSelectedItem();
	mcGetMediaInfo(item);
	float fps = (float)GetFPS(mcGetPrefValueInt("overall.video.fps"), mcGetPrefValueInt("overall.video.fpsScale"));
	if (fps) {
		sprintf(buf, "%5.3f", fps);
		SetText(IDC_FPS,buf);
	} else {
		ComboSetSel(IDC_FPS, 0);
	}
	int w = mcGetPrefValueInt("videofilter.scale.width");
	int h = mcGetPrefValueInt("videofilter.scale.height");
	if (w && h) {
		sprintf(buf, "%dx%d", w, h);
		SetText(IDC_SCALE_PRESET, buf);
	} else {
		SetText(IDC_SCALE_PRESET, "Original");
	}
	EnableCtrl(IDC_SCALE_PRESET, mcGetPrefValueInt("videofilter.scale.enabled"));
	EnableCtrl(IDC_ROTATE, mcGetPrefValueInt("videofilter.rotate.enabled"));
	EnableCtrl(IDC_CROP_PARAM, mcGetPrefValueInt("videofilter.crop.enabled"));
	char *p = mcGetPrefValueStr("overall.video.aspect");
	if (*p) {
		SetText(IDC_ASPECT, p);
		EnableCtrl(IDC_ASPECT);
	} else {
		SetText(IDC_ASPECT, 0);
		DisableCtrl(IDC_ASPECT);
	}
	ComboSetSel(IDC_ROTATE,mcGetPrefValueInt("videofilter.rotate.mode"));
	if (item) {
		_snprintf(buf, sizeof(buf), "%d:%d:%d:%d", item->target.crop.x, item->target.crop.y, item->target.crop.w, item->target.crop.h);
		SendDlgMsg(IDC_CROP_PARAM, WM_SETTEXT, 0, (LPARAM)buf);
	} else {
		SendDlgMsg(IDC_CROP_PARAM, WM_SETTEXT, 0, (LPARAM)"0:0:0:0");
	}
	i = mcGetPrefValueInt("videofilter.postproc.mode");
	EnableCtrl(IDC_DEINT, i != 0);
	EnableCtrl(IDC_TDN, i != 0);
}

bool CDlgPicture::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_BUTTON_CROP:
		ShowCropper(&mc, 0);
		return true;
	case IDC_DO_ASPECT:
		if (IsItemChecked(IDC_DO_ASPECT)) {
			ComboSetSel(IDC_ASPECT, 0);
			onSelChange(IDC_ASPECT);
		} else {
			mcSetPrefValueStr("overall.video.aspect", 0);
			Refresh();
		}
		break;
	default:
		return false;
	}
	return true;
}

CDlgTagging::CDlgTagging()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_TAG, "overall.tagging.tag"},
		{IDC_FORMAT_NAME, "overall.tagging.formatting"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid =IDD_TEXTS;
}

bool CDlgTagging::onCreate2()
{
	char* formatstr[] = {
		"$(Artist) - $(Title)",
		"$(Artist) - $(Album) - $(Title)",
		"$(Artist)\\$(Title)",
		"$(Artist)\\$(Artist) - $(Title)",
		"$(Artist)\\$(Track). $(Title)",
		"$(Artist)\\$(Album)\\$(Title)",
		"$(Artist)\\$(Album)\\$(Track). $(Title)",
		"$(Year)\\$(Artist) - $(Title)",
		"$(Year)\\$(Artist)\\$(Title)",
		"$(Genre)\\$(Artist) - $(Title)",
		"$(Genre)\\$(Artist)\\$(Title)",
		"$(SourceFileName)-transcoded",
		0};
	for (int i = 0; formatstr[i]; i++) {
		SendDlgMsg(IDC_FORMAT, CB_ADDSTRING, 0, (LPARAM)formatstr[i]);
	}
	AddLanguages(GetHandle(IDC_LANGUAGE), CB_ADDSTRING);
	return true;
}

void CDlgTagging::Refresh()
{
	BOOL v;
	v = mcGetPrefValueInt("overall.tagging.formatting");
	EnableCtrl(IDC_FORMAT, v);
	EnableCtrl(IDC_KEEP_DIR, !v);
	SetText(IDC_FORMAT, mcGetPrefValueStr("overall.tagging.fileNameFormat"));
	SetText(IDC_LANGUAGE, mcGetPrefValueStr("overall.decoding.slang"));
	T_ITEM* item = GetSelectedItem();
	if (item) {
		SendDlgMsg(IDC_SUB_FILE, WM_SETTEXT, 0, (LPARAM)item->subfile);
	}
	EnableCtrl(IDC_SUB_FILE, (item != 0));
	EnableCtrl(IDC_BROWSE, (item != 0));
}

bool CDlgTagging::onSelChange(int dlgid)
{
	switch (dlgid) {
	case IDC_FORMAT: {
		char *p = mcGetPrefValueStr("overall.tagging.fileNameFormat");
		SendDlgMsg(IDC_FORMAT, CB_GETLBTEXT, SendDlgMsg(IDC_FORMAT, CB_GETCURSEL, 0, 0), (LPARAM)p);
		} break;
	case IDC_LANGUAGE: {
		char *p, *s = ComboGetString(IDC_LANGUAGE, ComboGetSel(IDC_LANGUAGE));
		if (s) {
			p = strchr(s, ' ');
			if (p) *p = 0;
			strcpy(mcGetPrefValueStr("overall.decoding.slang"), s);
			free(s);
		}
		} break;
	default:
		return false;
	}
	return true;
}

bool CDlgTagging::onCommand2(int dlgid, int code)
{
	switch (code) {
	case EN_CHANGE:
		switch (dlgid) {
		case IDC_SUB_FILE:
				T_ITEM* item = GetSelectedItem();
				if (item) {
					int len = SendDlgMsg(IDC_SUB_FILE, WM_GETTEXTLENGTH, 0, 0);
					if (len == 0) {
						if (item->subfile) {
							free(item->subfile);
							item->subfile = 0;
						}
					} else if (len > 0) {
						mcAlloc((void**)&item->subfile, len + 1);
						GetText(IDC_SUB_FILE, item->subfile, len + 1);
					}
				}
			break;
		}
		break;
	case CBN_EDITUPDATE:
		switch (dlgid) {
		case IDC_FORMAT:
			GetText(IDC_FORMAT, mcGetPrefValueStr("overall.tagging.fileNameFormat"), mcGetPrefValueStrMax("overall.tagging.fileNameFormat"));
			break;
		case IDC_LANGUAGE: {
			char *p = mcGetPrefValueStr("overall.decoding.slang");
			GetText(IDC_LANGUAGE, p, 4);
			p = strchr(p, ' ');
			if (p) *p = 0;
			} break;
		default:
			return false;
		}
		break;
	default:
		return false;
	}
	return true;
}
bool CDlgTagging::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_BROWSE: {
		T_ITEM* item = GetSelectedItem();
		if (item) {
			char buf[512];
			OPENFILENAME ofn;
			memset(&ofn, 0, sizeof(ofn));
			buf[0]=0;
			ofn.lpstrFile = buf;
			ofn.nMaxFile = sizeof(buf) - 1;
			ofn.lpstrFilter = "Subtitle Files\0*.sub;*.idx;*.srt\0All Files\0*.*\0";
			ofn.lpstrTitle = "Load subtitle";
			if (!ShowFileDialog(0,&ofn)) {
				mcSetItemStr(&item->subfile, buf);
				Refresh();
			}
		}
		} break;
	default:
		return false;
	}
	return true;
}

CDlgTime::CDlgTime()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_SPLIT_MODE, "overall.task.splitMode"},
		{IDC_SPLIT_PARAM, "overall.task.splitParam"},
		{0}
	};
	ctrls = s_ctrls;
	dialogid = IDD_TIME;
}

void CDlgTime::FormatTime(char* str, int timeval)
{
	int i;
	char *p = str;
	if (timeval < 0) {
		*(p++) = '-';
		timeval = -timeval;
	}
	i = timeval / 3600000;
	if (i) {
		p += sprintf(p, "%d:", i);
		timeval -= i * 3600000;
	}
	i = timeval / 60000;
	p += sprintf(p, "%02d:", i);
	timeval -= i * 60000;
	i = timeval / 1000;
	p += sprintf(p, "%02d", i);
	timeval -= i * 1000;
	if (timeval) {
		p += sprintf(p, ":%03d", timeval);
	}
}

void CDlgTime::Refresh()
{
	bool split = mcGetPrefValueInt("overall.task.mode") == TM_SPLIT;
	SetItemChecked(IDC_SPLIT, split);
	Enable(IDC_SPLIT_MODE, split);
	Enable(IDC_SPLIT_PARAM, split);
	SetText(IDC_SPLIT_UNIT, !mcGetPrefValueInt("overall.task.splitMode") ? "ms" : "KB");
	RefreshTime(-1);
}

void CDlgTime::RefreshTime(int idx)
{
	char buf[64];
	T_ITEM* item = GetSelectedItem();
	int v;
	if (!item) return;
	refreshing = 1;
	v = item->info.begin ? item->info.begin : mcGetPrefValueInt("overall.decoding.startTime") * 1000;
	if (idx & 0x1) {
		FormatTime(buf, v);
		SetText(IDC_EDIT1, buf);
	}
	if (idx & 0x2) {
		sprintf(buf, "%.3f", (float)v / 1000);
		SetText(IDC_EDIT4, buf);
	}
	v = item->info.end ? item->info.end : (mcGetPrefValueInt("overall.decoding.duration") + mcGetPrefValueInt("overall.decoding.startTime")) * 1000;
	if (idx & 0x4) {
		FormatTime(buf, v);
		SetText(IDC_EDIT2, buf);
	}
	if (idx & 0x8) {
		sprintf(buf, "%.3f", (float)v / 1000);
		SetText(IDC_EDIT5, buf);
	}
	v = item->info.audiodelay ? item->info.audiodelay : mcGetPrefValueInt("overall.audio.delay") * 1000;
	if (idx & 0x10) {
		FormatTime(buf, v);
		SetText(IDC_EDIT3, buf);
	}
	if (idx & 0x20) {
		sprintf(buf, "%.3f", (float)v / 1000);
		SetText(IDC_EDIT6, buf);
	}
	refreshing = 0;
}

bool CDlgTime::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_SPLIT:
		mcSetPrefValueInt("overall.task.mode", IsItemChecked(IDC_SPLIT) ? TM_SPLIT : TM_NORMAL);
		Refresh();
		break;
	}
	return true;
}

bool CDlgTime::onCommand2(int dlgid, int code)
{
	if (refreshing) return false;
	switch (code) {
	case EN_CHANGE: {
		char buf[64];
		char* p;
		int val = 0, i;
		T_ITEM* item = GetSelectedItem();
		if (!item) break;
		switch (dlgid) {
		case IDC_EDIT4:
		case IDC_EDIT5:
		case IDC_EDIT6:
			val = (int)(GetTextFloat(dlgid) * 1000);
			switch (dlgid) {
			case IDC_EDIT4:
				item->info.begin = val;
				RefreshTime(0x1);
				break;
			case IDC_EDIT5:
				item->info.end = val;
				RefreshTime(0x4);
				break;
			case IDC_EDIT6:
				item->info.audiodelay = val;
				RefreshTime(0x10);
				break;
			}
			return true;
		}
		if (GetText(dlgid, buf, sizeof(buf)) > 0) {
			int tok[4];
			char* lasttok;
			bool hasms;
			int sign = 1;
			for (i = 0, p = strtok(buf, ":"); p && i < 4; i++, p = strtok(0, ":")) {
				lasttok = p;
				tok[i] = atoi(p);
			}
			hasms = (strlen(lasttok) > 2);
			if (tok[0] < 0) {
				tok[0] = -tok[0];
				sign = -1;
			}
			switch (i) {
			case 1:
				val = tok[0] * 1000;
				break;
			case 2:
				if (hasms)
					val = tok[0] * 1000 + tok[1];
				else
					val = (tok[0] * 60 + tok[1]) * 1000;
				break;
			case 3:
				if (hasms)
					val = (tok[0] * 60 + tok[1]) * 1000 + tok[2];
				else if (tok[0] <= 4)
					val = (tok[0] * 3600 + tok[1] * 60 + tok[2]) * 1000;
				break;
			case 4:
				if (tok[0] <= 4)
					val = (tok[0] * 3600 + tok[1] * 60 + tok[2]) * 1000 + tok[4];
				break;
			}
			val *= sign;
		}
		switch (dlgid) {
		case IDC_EDIT1:
			item->info.begin = val;
			RefreshTime(0x2);
			break;
		case IDC_EDIT2:
			item->info.end = val;
			RefreshTime(0x8);
			break;
		case IDC_EDIT3:
			item->info.audiodelay = val;
			RefreshTime(0x20);
			break;
		}
		} break;
	default:
		return false;
	}
	return true;
}

bool CDlgPreview::onCreate2()
{
	SendDlgMsg(IDC_PREVIEW_TYPE, CB_ADDSTRING, 0, (LPARAM)"Original");
	SendDlgMsg(IDC_PREVIEW_TYPE, CB_ADDSTRING, 0, (LPARAM)"Converted");
	SendDlgMsg(IDC_PREVIEW_TYPE, CB_SETCURSEL, 0, 0);
	sliderMax = 0;
	return true;
}

#define PREVIEW_WIDTH 176
#define PREVIEW_HEIGHT 132

void CDlgPreview::SetControlMode(int mode)
{
	EnableCtrl(IDC_PLAY, !mode);
	EnableCtrl(IDC_STOP, mode);
	EnableCtrl(IDC_PAUSE, mode);
	EnableCtrl(IDC_FS, mode);
	EnableCtrl(IDC_SLIDER, mode);
	if (mode == 0) {
		sliderMax = 0;
		step = 0;
		KillTimer(hwnd, 0);
		video.Destroy();
	} else {
		SendDlgMsg(IDC_SLIDER,TBM_SETRANGE,TRUE,MAKELPARAM(0,sliderMax));
		if (sliderMax) SetTimer(hwnd, 0, 1000, 0);
	}
	SendDlgMsg(IDC_SLIDER, TBM_SETPOS, TRUE, 0);
}

bool CDlgPreview::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_PLAY: {
		T_ITEM* item = GetSelectedItem();
		if (SendDlgMsg(IDC_PREVIEW_TYPE, CB_GETCURSEL, 0, 0) == 0) {
			if (item) {
				video.Open(GetHandle(IDC_PREVIEW_BOX), WS_VISIBLE);
				if (mcPlayerStart(item, video.hwnd, 0, 0))
					break;
				sliderMax = item->info.duration / 1000;
				step = 1;
				SetControlMode(1);
			}
		} else {
			video.Open(GetHandle(IDC_PREVIEW_BOX), WS_VISIBLE);
			if (mcPlayerStart(item, video.hwnd, PF_PREVIEW, 0))
				break;
			step = 0;
			SetControlMode(1);
			EnableCtrl(IDC_SLIDER, FALSE);
		}
		video.Move(0, 0, PREVIEW_WIDTH, PREVIEW_HEIGHT);
		} break;
	case IDC_STOP:
		SetControlMode(0);
		mcPlayerStop();
		break;
	case IDC_PAUSE:
		step = 0;
		mcPlayerCommand("pause");
		break;
	case IDC_FS:
		mcPlayerCommand("vo_fullscreen");
		break;
	}
	return true;
}

bool CDlgPreview::onScroll(int dlgid, int code)
{
	int pos = SendDlgMsg(IDC_SLIDER,TBM_GETPOS,0,0);
	char buf[32];
	sprintf(buf, "seek %d 2", pos);
	mcPlayerCommand(buf);
	return true;
}

bool CDlgPreview::onTimer(int id)
{
	int pos = SendDlgMsg(IDC_SLIDER,TBM_GETPOS,0,0);
	if (step) {
		pos += step;
		if (mcPlayerCommand("get_time_pos")) {
			SetControlMode(0);
		}
	}
	if (sliderMax > 0) {
		if (pos > sliderMax) {
			SetControlMode(0);
		} else {
			SendDlgMsg(IDC_SLIDER, TBM_SETPOS, TRUE, pos);
		}
	}
	return true;
}

CDlgOutput::CDlgOutput()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_OUTPUT_FOLDER, "overall.task.destdir"},
		{IDC_TEMP_FOLDER, "overall.task.tempdir"},
		{IDC_KEEP_DIR, "overall.task.dupDir"},
		{0},
	};
	ctrls = s_ctrls;
	dialogid = IDD_OUTPUT;
}

bool CDlgOutput::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_BROWSE1:
		if (BrowseForPath(mcGetPrefValueStr("overall.task.destdir")))
			RefreshDialog();
		break;
	case IDC_BROWSE2:
		if (BrowseForPath(mcGetPrefValueStr("overall.task.tempdir")))
			RefreshDialog();
		break;
	case IDC_OPEN_OUTPUT:
		ShellExecute(mc.hwnd, "open", mcGetPrefValueStr("overall.task.destdir"), 0, 0, SW_SHOW);
		break;
	}
	return true;
}

static int devButtonId[] = {IDC_DEVICE_PHONE, IDC_DEVICE_DMP, IDC_DEVICE_GAME, IDC_DEVICE_DISC, IDC_DEVICE_PDA, IDC_DEVICE_OTHER};
static int devbmp[]={IDB_DEV_PHONE, IDB_DEV_DMP, IDB_DEV_GAME, IDB_DEV_DISC, IDB_DEV_PDA, IDB_DEV_OTHER};
#define DEV_BUTTON_COUNT (sizeof(devButtonId) / sizeof(int))
HBITMAP hDevBitmap[DEV_BUTTON_COUNT];

bool CDlgDevNav::onCreate()
{
	HBITMAP h = LoadBitmap(hInstance,MAKEINTRESOURCE(devbmp[0]));
	h = LoadBitmap(hInstance,MAKEINTRESOURCE(devbmp[0]));
	for (int i = 0; i < DEV_BUTTON_COUNT; i++) {
		if (devbmp[i]) {
			hDevBitmap[i] = LoadBitmap(hInstance,MAKEINTRESOURCE(devbmp[i]));
			SendDlgMsg(devButtonId[i],BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)hDevBitmap[i]);
		}
	}
	return true;
}

bool CDlgDevNav::onClose()
{
	for (int i = 0; i < DEV_BUTTON_COUNT; i++) {
		DeleteObject(hDevBitmap[i]);
	}
	return false;
}

bool CDlgDevNav::onSize(int width, int height)
{
	int space = width / DEV_BUTTON_COUNT - NAV_BUTTON_WIDTH;
	if (space < 0) space = 0;
	for (int i = 0, hpos = space / 2 + 2; i < DEV_BUTTON_COUNT; i++, hpos += (NAV_BUTTON_WIDTH + space)) {
		MoveWindow(GetHandle(devButtonId[i]), hpos, 0, NAV_BUTTON_WIDTH, NAV_BUTTON_HEIGHT, TRUE);
	}
	return true;
}

bool CDlgDevNav::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_DEVICE_PHONE:
		device.Open(DEV_TYPE_PHONE);
		break;
	case IDC_DEVICE_DMP:
		device.Open(DEV_TYPE_DMP);
		break;
	case IDC_DEVICE_GAME:
		device.Open(DEV_TYPE_GAMECONSOLE);
		break;
	case IDC_DEVICE_DISC:
		device.Open(DEV_TYPE_DISCPLAYER);
		break;
	case IDC_DEVICE_PDA:
		device.Open(DEV_TYPE_PDA);
		break;
	case IDC_DEVICE_OTHER:
		device.Open(DEV_TYPE_UNSPEC);
		break;
	}
	return true;
}

void CDlgPopup::Open(CWindow* parent)
{
	RECT rc;
	if (!dlg) return;
	Create(WS_POPUP|WS_SYSMENU|WS_BORDER|WS_CAPTION|WS_VISIBLE, WS_EX_TOOLWINDOW, parent);
	dlg->CreateDlg(hInstance, dlg->dialogid, this);
	GetWindowRect(dlg->hwnd, &rc);
	AdjustWindowRectEx(&rc, WS_POPUP|WS_SYSMENU|WS_BORDER|WS_CAPTION, FALSE, WS_EX_TOOLWINDOW);
	SetCaption(dlg->GetName());
	MoveCenter(rc.right - rc.left, rc.bottom - rc.top);
	if (parent) parent->Disable();
	dlg->Show();
	Refresh();
}

bool CDlgPopup::onClose()
{
	if (!dlg) return false;;
	dlg->Close();
	if (wndParent) {
		wndParent->Enable();
		SetFocus(wndParent->hwnd);
	}
	return false;
}

CDlgPopup::~CDlgPopup()
{
	if (dlg) delete dlg;
}

void CDlgPopup::Refresh()
{
	if (dlg) dlg->RefreshDialog();
}

void CDlgPopup::SetDlg(COptionDialog* newdlg)
{
	if (dlg) delete dlg;
	dlg = newdlg;
}
